/*
 *  adsp_21488.c is forza driver for custom dsp
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/slab.h>
#include <linux/namei.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/mount.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/delay.h>
#include "forza_lib.h"
#include "adsp_21488.h"
#include "forza_hwlow.h"
#include "../../common/h/codecinfotype.h"

#define ADSP_FW_PATH "/sony/lib/modules/dspfw/adsp_21488.bin"
void adsp_21488_mute(void){
  
}

static unsigned int adsp_21488_set_fs(unsigned int data, int fs){
  switch(fs){
  case 44100:
    data |= ADSP_SAMPLE_RATE_44_1;     break;
  case 48000:
    data |= ADSP_SAMPLE_RATE_48;       break;
  case 88200:
    data |= ADSP_SAMPLE_RATE_88_2;     break;
  case 96000:
    data |= ADSP_SAMPLE_RATE_96;       break;
  case 176400:
    data |= ADSP_SAMPLE_RATE_176_4;    break;
  case 192000:
    data |= ADSP_SAMPLE_RATE_192;      break;
  default:
    FORZA_INFO_LOG("ADSP_FS:skiped\n"); break;
  }
  return data;
}

static unsigned int adsp_21488_set_mode(unsigned int data, int mode){
  switch(mode){
  case SINGER_CODEC_TYPE_WAV_GROUP:
  case SINGER_CODEC_TYPE_WAV:
  case SINGER_CODEC_TYPE_AIFF_GROUP:
  case SINGER_CODEC_TYPE_AIFF:
  case SINGER_CODEC_TYPE_FLAC_GROUP:
  case SINGER_CODEC_TYPE_FLAC:
  case SINGER_CODEC_TYPE_ALAC_GROUP:
  case SINGER_CODEC_TYPE_ALAC:
  case SINGER_CODEC_TYPE_WMALossless:
    FORZA_INFO_LOG("HEQ->LPCM\n");
    data |= ADSP_LPCM;
    break;
  case SINGER_CODEC_TYPE_MP3_GROUP:
  case SINGER_CODEC_TYPE_MP3:
  case SINGER_CODEC_TYPE_WMA_GROUP:
  case SINGER_CODEC_TYPE_WMAv2:
  case SINGER_CODEC_TYPE_WMAPro:
    FORZA_INFO_LOG("HEQ->MP3/WMA\n");
    data |= ADSP_MP3;
    break;
  case SINGER_CODEC_TYPE_AAC_GROUP:
  case SINGER_CODEC_TYPE_LCAAC:
  case SINGER_CODEC_TYPE_HEAAC:
  case SINGER_CODEC_TYPE_HEAACv2:
    FORZA_INFO_LOG("HEQ->AAC\n");
    data |= ADSP_AAC;
    break;
  case SINGER_CODEC_TYPE_ATRAC_GROUP:
  case SINGER_CODEC_TYPE_ATRAC3:
  case SINGER_CODEC_TYPE_ATRACX:
    FORZA_INFO_LOG("HEQ->ATRAC\n");
    data |= ADSP_ATRAC;
    break;
  default:
    data |= ADSP_LPCM;
    FORZA_INFO_LOG("HEQ->Don'tCare:0x%02x\n",mode);
    break;
  }
  return data;
}

static unsigned int adsp_21488_set_bit(unsigned int data, int bit){
  switch(bit){
  case 16:
    data |= ADSP_BIT_16;
    break;
  case 24:
    data |= ADSP_BIT_24;
    break;
  case 32:
    data |= ADSP_BIT_32;
    break;
  default:
    FORZA_INFO_LOG("ADSP_BIT:skiped(%d)\n",data);
    break;
  }
  return data;
}

static unsigned int adsp_21488_set_rate(unsigned int data,int rate){
  if(rate <= 16000){ data |= 0x0; 
    FORZA_DEBUG_LOG("ADSP_RATE:under 16kps\n");
  }
  else if(( 16000 < rate) && ( rate <=  24000)){data |= 0x01;}
  else if(( 24000 < rate) && ( rate <=  32000)){data |= 0x02;}
  else if(( 32000 < rate) && ( rate <=  40000)){data |= 0x03;}
  else if(( 40000 < rate) && ( rate <=  48000)){data |= 0x04;}
  else if(( 48000 < rate) && ( rate <=  56000)){data |= 0x05;}
  else if(( 56000 < rate) && ( rate <=  64000)){data |= 0x06;}
  else if(( 64000 < rate) && ( rate <=  80000)){data |= 0x07;}
  else if(( 80000 < rate) && ( rate <=  96000)){data |= 0x08;}
  else if(( 96000 < rate) && ( rate <= 112000)){data |= 0x09;}
  else if((112000 < rate) && ( rate <= 128000)){data |= 0x0a;}
  else if((128000 < rate) && ( rate <= 160000)){data |= 0x0b;}
  else if((160000 < rate) && ( rate <= 192000)){data |= 0x0c;}
  else if((192000 < rate) && ( rate <= 224000)){data |= 0x0d;}
  else if((224000 < rate) && ( rate <= 256000)){data |= 0x0e;}
  else if((256000 < rate) && ( rate <= 288000)){data |= 0x0f;}
  else if((288000 < rate) && ( rate <= 320000)){data |= 0x10;}
  else{ data |= 0x11;
        FORZA_INFO_LOG("ADSP_RATE:over 320kps\n");
  }    
  return data;
}

unsigned int adsp_21488_send_data(struct snd_forza *chip, unsigned int d){
  unsigned int rdata;
  SPI_DSP_TX_REG_W(chip,d);
  udelay(20);
  SPI_DSP_TX_REG_W(chip,ADSP_DUMMY_DT);
  udelay(20);
  SPI_DSP_TX_REG_W(chip,d & ~0x01000000);
  udelay(20);
  SPI_DSP_TX_REG_W(chip,ADSP_DUMMY_DT);
  udelay(20);

  rdata = SPI_DSP_RX_REG_R(chip);
  FORZA_DEBUG_LOG("ADSP_DAT:W(0x%08x)/R(0x%08x)\n",d,rdata);
  udelay(50);
  return rdata;
}

unsigned int adsp_21488_read_data(struct snd_forza *chip, unsigned int d){
  unsigned int rdata;
  SPI_DSP_TX_REG_W(chip,d);
  udelay(20);
  SPI_DSP_TX_REG_W(chip,ADSP_DUMMY_DT);
  udelay(20);
  rdata = SPI_DSP_RX_REG_R(chip);
  FORZA_DEBUG_LOG("ADSP_DAT:R(0x%08x)\n",rdata);
  udelay(50);
  return rdata;
}

void adsp_21488_set_formats(struct snd_forza *chip, struct ioctl_struct *p)
{
  unsigned int d_11 =FORMATS_WR_HEAD;
  FORZA_DEBUG_LOG("set_formats=%d, fs=%d, bit=%d\n ", p->mode  ,p->fs,p->bit);
  d_11 = adsp_21488_set_mode(d_11, p->mode);
  d_11 = adsp_21488_set_bit(d_11, p->bit);
  d_11 = adsp_21488_set_fs(d_11, p->fs);
  adsp_21488_send_data(chip,d_11);
}

void adsp_21488_set_heq_params(struct snd_forza *chip, struct ioctl_struct *p)
{
  unsigned int d_11 =FORMATS_WR_HEAD, d_13 =RATE_WR_HEAD;

  FORZA_INFO_LOG("HEQ-Param mode=0x%02x, rate=%d, fs=%d, bit=%d\n", p->mode ,p->rate ,p->fs,p->bit);
  d_11 = adsp_21488_set_mode(d_11, p->mode);
  d_11 = adsp_21488_set_bit(d_11, p->bit);
  d_11 = adsp_21488_set_fs(d_11, p->fs);
  adsp_21488_send_data(chip,d_11);
  if((d_11 & 0xe0) > 0x20){
    d_13 = adsp_21488_set_rate(d_13, p->rate);
    adsp_21488_send_data(chip,d_13);
  }
   
}

void adsp_21488_spi_write(struct snd_forza *chip,unsigned int data){
  SPI_DSP_TX_REG_W(chip,data);
  udelay(20);
}

#define fw_max_size (64*1024)
void adsp_21488_write(struct snd_forza *chip,unsigned char *buf, int size){
  unsigned char *ptr =buf;
  unsigned int data, i=0;
  int delay_time = 500;
  while(1){
    data = 0;
    data = (*ptr) << 24 | *(ptr+1) << 16 | *(ptr+2) << 8 | *(ptr+3);
    adsp_21488_spi_write(chip,data);
    if(i == 0x600){
      FORZA_DEBUG_LOG("Delay:...%d_us for BL sequence\n", delay_time);
      udelay(delay_time);
    }
    if(i > size){   break;  }
    i += 4;
    ptr += 4;
  }
}

void adsp_21488_fwdl(struct snd_forza *chip, struct ioctl_struct *p){
  struct file *fp;
  char * filename = ADSP_FW_PATH;
  int read_size;
  unsigned char *buf;
  mm_segment_t oldfs;

  buf = kmalloc(fw_max_size,GFP_KERNEL);
  if(buf== NULL){
    FORZA_ERROR_LOG(">>Kmalloc\n");
    return;
  }else{
    memset(buf ,0 ,fw_max_size);
  }

  oldfs = get_fs();
  set_fs(get_ds());

  fp = filp_open(filename, O_RDONLY,0);
  if(!fp){
    FORZA_ERROR_LOG(">>FileOpenError\n");
    kfree(buf);
    return;
  }

  read_size = fp->f_op->read(fp,buf,fw_max_size -1 ,&fp->f_pos);

  FORZA_DEBUG_LOG("%d bytes read\n string=%s\n",read_size,buf);
  filp_close(fp, NULL);
  set_fs(oldfs);
  adsp_21488_write(chip, buf, read_size);
  kfree(buf);
  return;
}
