/*
 *  cdsp_cs48l10.c is forza driver for custom dsp
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include "forza_lib.h"
#include "forza_hwlow.h"
#include <linux/slab.h>
#include <linux/namei.h>
#include <linux/types.h>
#include <linux/stat.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/mount.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <linux/delay.h>
#include "cdsp_cs48l10.h"
#include "../../common/h/codecinfotype.h"

#define FWDL_FILECOUNT    2 
#define CONFIG_FILENAME  CONFIG_4UPSRC

#define CDSP_TMT_ON

#define BITRATE_TOTAL 		17
int heq_bitrate[BITRATE_TOTAL] = {
				   16000, 24000, 32000,
				   40000, 48000, 56000,
				   64000, 80000, 96000,
				   112000, 128000, 160000,
				   192000, 224000, 256000,
				   288000, 320000,
				};

#define BITRATE_TOTAL_ATRAC3 		10
int heq_bitrate_atrac3[BITRATE_TOTAL_ATRAC3] = {
				   56000, 64000, 96000,
				   128000, 132000, 160000,
				   192000, 256000, 320000,
				   352000,
				};

struct struct_fmdl_fileinfo {
  const char* name;
} fmdl_fileinfo[] = {
   { "/sony/lib/modules/dspfw/cdsp_48L10_1_os.bin"},
   { "/sony/lib/modules/dspfw/cdsp_48L10_2_cfg.bin"},
};

int check_interrupt(struct snd_forza *chip, int interrupt_bit)
{
  int wait_time = 0;
  int long_wait = 0;
  int reg;
  do {
    if (((reg=SPI_DSP_STATUS_REG_R(chip)) & interrupt_bit) == interrupt_bit)
      break;
    udelay(ALLEGRO_DELAY);
    wait_time += ALLEGRO_DELAY;
    if (wait_time >= MAX_WAIT_TIME){
      if(long_wait > 1000){
	FORZA_ERROR_LOG("%s: No DSPINT flag2 >>Long Wait fail\n", __FUNCTION__);
	return BOOT_FAILURE;
      }else{
	wait_time  = 0;
	long_wait += 1;
      }
    }
  } while(1);

  return BOOT_OK;
}

int read_rx_register(struct snd_forza *chip)
{
  int ret = 0;
  /* Set DSP and OE bit as 1, rest 0 */
  //  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_DSP_INTE | SPI_DSP_OE));
  /* Check the DSPINT bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_DSP_INT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_DSP_INT  \n", __FUNCTION__);
    return ret;
  }

  /* Write the READ_COMMAND */
  SPI_DSP_TX_REG_W(chip, READ_COMMAND);

#ifndef CDSP_TMT_ON
  FORZA_DEBUG_LOG("DELAY!!\n");
  udelay(8);
#endif

  /* Send the DUMMY DATA */
  SPI_DSP_TX_REG_W(chip, CDSP_DUMMY_DT);

#ifndef CDSP_TMT_ON
  udelay(19);
#endif

  /* Set RRDY and OE bit as 1, rest 0 */
  //  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_TRRDY | SPI_DSP_OE));

  /* Check the RRDY bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_RRDY)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_RRDY interrupt  \n", __FUNCTION__);
    return ret;
  }
  /* read the Rx register */
  return SPI_DSP_RX_REG_R(chip);
}

int send_slave_boot_message(struct snd_forza *chip)
{
  int ret = 0;

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, SLAVE_BOOT);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  return BOOT_OK;
}

int send_soft_reset(struct snd_forza *chip)
{
  int ret = 0;

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  SPI_DSP_TX_REG_W(chip, SOFT_RESET);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  return BOOT_OK;
}

int send_soft_boot(struct snd_forza *chip)
{
  int ret = 0;

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, SOFTBOOT1);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, SOFTBOOT2);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  return BOOT_OK;
}

int send_kickstart_message(struct snd_forza *chip)
{
  int ret = 0;

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, KICKSTART1);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, KICKSTART2);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  /*sleep as per the specification */
  msleep(200);

  return BOOT_OK;
}

int cdsp_CS48L10_filewrite(struct snd_forza *chip, unsigned char *buf, int size)
{
  unsigned char *ptr = buf;
  unsigned int data;
  int ret = 0;

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  while(size > 0) {
    data = 0;

    /* if remaining size is not full 4 byte */
    if (size < 4) {

      /* write the remaining data byte by byte */
      while(size >= 0) {
        data = 0;
        data = *(ptr+size) << 24;  
        size--;

        /* write the single byte */
        SPI_DSP_TX_REG_W(chip, data);

        /* Check the TX_Empty bit from status register */
        if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
          FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
          return ret;
        }
        /*clear the TMT bit */
        SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));
      }
      break;
    }
    else {
      data = (*ptr) << 24 | *(ptr+1) << 16 | *(ptr+2) << 8 | *(ptr+3);

      /* write the 4 byte data */
      SPI_DSP_TX_REG_W(chip, data);

      /* Check the TX_Empty bit from status register */
      if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
        FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
        return ret;
      }
      /*clear the TMT bit */
      SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

      /* do the pointer and size adjustments */ 
      ptr += 4;
      size -= 4;
    }
  }

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  return BOOT_OK;
}

int cdsp_CS48L10_fileop(struct snd_forza *chip, const char *filename)
{
  int ret = BOOT_FAILURE;
  struct file *fp;
  int read_size = 0;
  unsigned char *buf;
  mm_segment_t oldfs;
  struct kstat stat;

  oldfs = get_fs();
  set_fs(get_ds());

  if (vfs_stat(filename, &stat)) {
    FORZA_ERROR_LOG("%s: vfs_stat filed \n", __FUNCTION__);
    set_fs(oldfs);
    return ret;
  }
  FORZA_DEBUG_LOG("%s: filename:%s, SIZE:%ld \n", __FUNCTION__, filename, (long)stat.size);

  /* currenlty reading the whole file...later it will be enhanced (tmp code) */
  buf = kmalloc((stat.size+1),GFP_KERNEL | __GFP_ZERO);
  if(buf== NULL){
    FORZA_ERROR_LOG("%s: kmalloc failed  \n", __FUNCTION__);
    set_fs(oldfs);
    return ret;
  }

  FORZA_DEBUG_LOG("open:%s \n", filename);
  fp = filp_open(filename, O_RDONLY,0);
  if (IS_ERR(fp) || !fp->f_op){
    FORZA_ERROR_LOG("%s: FileOpenError \n", __FUNCTION__);
    set_fs(oldfs);
    kfree(buf);
    return ret;
  }

  /* currenlty reading the whole file...later it will be enhanced (tmp code) */
  read_size = fp->f_op->read(fp, buf, stat.size, &fp->f_pos);
  FORZA_DEBUG_LOG("readsize of %s:%d \n", filename, read_size);

  filp_close(fp, NULL);
  set_fs(oldfs);

  if ((ret = cdsp_CS48L10_filewrite(chip, buf, read_size)) != 0) {
    FORZA_ERROR_LOG("%s: error in cdsp_CS48L10_filewrite function \n", __FUNCTION__);
    kfree(buf);
    return ret;
  }
  // should be always displayed for confirmation
  printk(KERN_ALERT "[FORZA] download file %s\n", filename);

  kfree(buf);
  return BOOT_OK;
}

int cdsp_CS48L10_fwdl(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  int i;
  int val = 0;

  /* traverse all the firmware files one by one */
  for(i=0; i<FWDL_FILECOUNT; i++) {

    /* send SLAVE_BOOT message */
    FORZA_DEBUG_LOG("Sending SLAVE_BOOT message \n");
    if ((ret = send_slave_boot_message(chip)) != 0) {
      FORZA_ERROR_LOG("%s: error while sending SLAVE_BOOT \n", __FUNCTION__);
      return ret;
    }

    /* Check for BOOT_START message  */
    FORZA_DEBUG_LOG("Wait for BOOT_START message !!! \n");
    if ((val = read_rx_register(chip)) != BOOT_START) {
      FORZA_ERROR_LOG("%s: No BOOT_START message. val=%d  \n", __FUNCTION__, val);
      return BOOT_FAILURE;
    }
    FORZA_DEBUG_LOG("BOOT_START received !!! \n");

    /* Clear all the interrupts */
    SPI_DSP_STATUS_REG_W(chip, INTERRUPT_CLR);

    /* Set TMT and OE bit as 1, rest 0 */
    //    SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_ITMT | SPI_DSP_OE));

    /* actual firmware file download operation */
    if ((ret = cdsp_CS48L10_fileop(chip, fmdl_fileinfo[i].name)) != 0) {
      FORZA_ERROR_LOG("%s: error in cdsp_CS48L10_fileop function  \n", __FUNCTION__);
      return ret;
    }
 
    /* Check for BOOT_SUCCESS message  */
    FORZA_DEBUG_LOG("Wait for BOOT_SUCCESS message \n");
    if ((val = read_rx_register(chip)) != BOOT_SUCCESS) {
      FORZA_ERROR_LOG("%s: No BOOT_SUCCESS message. val=%d  \n", __FUNCTION__, val);
      return BOOT_FAILURE;
    }
  }
  return ret;
}

int cdsp_full_kickstart_seq(struct snd_forza *chip, const char *filename)
{
  int ret = 0;
  int val = 0;

  FORZA_DEBUG_LOG("Sending SOFT_RESET \n");
  if ((ret = send_soft_reset(chip)) != 0) {
    FORZA_ERROR_LOG("%s: error while sending SOFT_RESET \n", __FUNCTION__);
    return ret;
  }

  /* Check for APP_START message  */
  FORZA_DEBUG_LOG("Wait for APP_START message !!! \n");
  if ((val = read_rx_register(chip)) != APP_START) {
    FORZA_ERROR_LOG("%s: No APP_START message, val=%d  \n", __FUNCTION__, val);
    return BOOT_FAILURE;
  }
  FORZA_DEBUG_LOG("APP_START received !!! \n");

  /* Clear all the interrupts */
  SPI_DSP_STATUS_REG_W(chip, INTERRUPT_CLR);

  /* Set TMT and OE bit as 1, rest 0 */
  //  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_ITMT | SPI_DSP_OE));

  if ((ret = cdsp_CS48L10_fileop(chip, filename)) != 0) {
    FORZA_ERROR_LOG("%s: cdsp_CS48L10_fileop failed  \n", __FUNCTION__);
    return ret;
  }

  FORZA_DEBUG_LOG("Sending KICKSTART message \n");
  if ((ret = send_kickstart_message(chip)) != 0){
    FORZA_ERROR_LOG("%s: error while sending KICKSTART message  \n", __FUNCTION__);
    return ret;
  }
  
  return BOOT_OK;
}

int cdsp_audio_osc_change(struct snd_forza *chip, int freq)
{
  int ret = 0;
  FORZA_DEBUG_LOG("%s: freq:%d \n", __FUNCTION__, freq);

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  /* write operation as per specification */  
  SPI_DSP_TX_REG_W(chip, AUDIOFS_COMMON);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  if (FONE == freq)
    SPI_DSP_TX_REG_W(chip, FONE_AUDIOFS);

  else
    SPI_DSP_TX_REG_W(chip, FTWO_AUDIOFS);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));


  FORZA_DEBUG_LOG("FS:%d:OK \n",freq);
  return BOOT_OK;
}

extern int full_fw_download;
int cdsp_over_sampling_fschange(struct snd_forza *chip, const char *filename)
{
  int ret = 0;
  int val = 0;
  struct ioctl_struct *p = NULL;

  FORZA_DEBUG_LOG("Sending SOFT_BOOT \n");
  if ((ret = send_soft_boot(chip)) != 0) {
    FORZA_ERROR_LOG("%s: error while sending SOFT_BOOT \n", __FUNCTION__);
    return ret;
  }

  /* Check for BOOT_READY message  */
  FORZA_DEBUG_LOG("Wait for BOOT_READY message !!! \n");
  if ((val = read_rx_register(chip)) != BOOT_READY) {
    FORZA_ERROR_LOG("%s: No BOOT_READY message, val=%d  \n", __FUNCTION__, val);
    return BOOT_FAILURE;
  }
  FORZA_DEBUG_LOG("BOOT_READY received !!! \n");

  /* work around for sound quality issue */
  if (full_fw_download) {
    /* write the firmware files to DSP one by one */
    if ((ret = cdsp_CS48L10_fwdl(chip, p)) != 0){
      FORZA_ERROR_LOG("%s: error while firmware download  \n", __FUNCTION__);
      return ret;
    }
	full_fw_download = 0;
  }

  if ((ret = cdsp_full_kickstart_seq(chip, filename)) != 0) {
    FORZA_ERROR_LOG("%s: error in cdsp_full_kickstart_seq() \n", __FUNCTION__);
    return ret;
  }

  return BOOT_OK;
}

int cdsp_firmware_download(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  int val = 0;

  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);

  /* Check for BOOT_READY message  */
  FORZA_DEBUG_LOG("Wait for BOOT_READY message !!! \n");
  if ((val = read_rx_register(chip)) != BOOT_READY) {
    FORZA_ERROR_LOG("%s: No BOOT_READY message  %d\n", __FUNCTION__, val);
    return BOOT_FAILURE;
  }
  FORZA_DEBUG_LOG("BOOT_READY received !!! \n");

  /* write the firmware files to DSP one by one */
  if ((ret = cdsp_CS48L10_fwdl(chip, p)) != 0){
    FORZA_ERROR_LOG("%s: error while firmware download  \n", __FUNCTION__);
    return ret;
  }
  FORZA_DEBUG_LOG("Starge2 Clear\n");

if ((ret = cdsp_full_kickstart_seq(chip, CONFIG_FILENAME)) != 0) {
    FORZA_ERROR_LOG("%s: error in cdsp_full_kickstart_seq() \n", __FUNCTION__);
    return ret;
  }

  return BOOT_OK;
}

int cdsp_mute_unmute(struct snd_forza *chip, int flag)
{
  int ret = 0;

  FORZA_DEBUG_LOG("%s: flag=%d (1:MUTE, 2:UNMUTE)\n", __FUNCTION__, flag);

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

#ifndef CDSP_TMT_ON
  udelay(1);
#endif

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

#ifndef CDSP_TMT_ON
  udelay(8);
#endif

  /* write the common value */  
  SPI_DSP_TX_REG_W(chip, MUTE_UNMUTE_COMMON);

#ifndef CDSP_TMT_ON
  udelay(20);
#else
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0) {
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));
#endif

  if (MUTE == flag)
    SPI_DSP_TX_REG_W(chip, MUTE_VALUE);
  else
    SPI_DSP_TX_REG_W(chip, UNMUTE_VALUE);

  /* Check the TX_Empty bit from status register */
#ifndef CDSP_TMT_ON
  udelay(20);
#else
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0) {
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));
#endif

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  return BOOT_OK;
}

static int heq_prev_data = 0;
static int set_heq_param_flag = 0;

int cdsp_heq_onoff(struct snd_forza *chip, int flag)
{
  int ret = 0;

  FORZA_INFO_LOG("%s: flag=%d (1:HEQ_ON, 2:HEQ_OFF)\n", __FUNCTION__, flag);

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

#ifndef CDSP_TMT_ON
  udelay(1);
#endif

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

#ifndef CDSP_TMT_ON
  udelay(8);
#endif

  /* write the common value */
  SPI_DSP_TX_REG_W(chip, CDSP_HEQ_ADDR);

#ifndef CDSP_TMT_ON
  udelay(20);
#else
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0) {
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));
#endif

  if (HEQ_ON == flag) {
    set_heq_param_flag = 1;
  }
  else {
    set_heq_param_flag = 0;
    heq_prev_data = HEQ_OFF_DATA;
  }

  FORZA_DEBUG_LOG("%s: heq_prev_data = %x \n", __FUNCTION__, heq_prev_data);
  SPI_DSP_TX_REG_W(chip, heq_prev_data);

#ifndef CDSP_TMT_ON
  udelay(20);
#else
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0) {
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));
#endif

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  return BOOT_OK;
}

int search(int rate, int *index)
{
  int i;

  if (rate <= heq_bitrate[0]) {
    *index = 0;
    return BOOT_OK;
  }

  for(i=0; i<(BITRATE_TOTAL-1); i++) {
    if ((rate > heq_bitrate[i]) && (rate <= heq_bitrate[i+1])) {
      *index = i+1;
      return BOOT_OK;
    }
  }

  *index = i;  // maximum
  return BOOT_OK;
}

int search_atrac3(int rate, int *index)
{
  int i;

  if (rate <= heq_bitrate_atrac3[0]) {
    *index = 0;
    return BOOT_OK;
  }

  for(i=0; i<(BITRATE_TOTAL_ATRAC3-1); i++) {
    if ((rate > heq_bitrate_atrac3[i]) && (rate <= heq_bitrate_atrac3[i+1])) {
      *index = i+1;
      return BOOT_OK;
    }
  }

  *index = i;  // maximum
  return BOOT_OK;
}

int calc_dataval_from_bitrate(struct snd_forza *chip, int base_value, struct ioctl_struct *p)
{
  int ret = 0;
  int index = 0;
  FORZA_DEBUG_LOG("%s: base_value = %x \n", __FUNCTION__, base_value);
 
  if(base_value != CODEC_ATRAC3_BASE_VAL) {
    if ((ret = search(p->rate, &index)) != 0) {
      FORZA_ERROR_LOG("%s: unsupported bitrate value:%d  \n", __FUNCTION__, p->rate);
      return ret; 
	}
  }
  else {
    if ((ret = search_atrac3(p->rate, &index)) != 0) {
      FORZA_ERROR_LOG("%s: unsupported bitrate value:%d  \n", __FUNCTION__, p->rate);
      return ret; 
	}
  }
 
  heq_prev_data = base_value + (index << 16);
  FORZA_DEBUG_LOG("%s: heq_prev_data = %x \n", __FUNCTION__, heq_prev_data);

  if (set_heq_param_flag) {
    if (( ret = cdsp_heq_onoff(chip, HEQ_ON)) != 0)
      FORZA_ERROR_LOG("%s: cdsp_heq_off failed  \n", __FUNCTION__);
  }

  return ret;
}

int cdsp_set_heq_param(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  FORZA_INFO_LOG("%s: Param mode=0x%x, bitrate=%d, fs=%d, bit=%d\n", __FUNCTION__, p->mode, p->rate, p->fs, p->bit);

  switch(p->mode) {
    case FORZA_HEQ_MODE_OFF:
      heq_prev_data = HEQ_OFF_DATA;

      if (set_heq_param_flag) {
        if (( ret = cdsp_heq_onoff(chip, HEQ_OFF)) != 0)
          FORZA_ERROR_LOG("%s: cdsp_heq_off failed  \n", __FUNCTION__);
      }
    break;

    case SINGER_CODEC_TYPE_WAV_GROUP:
    case SINGER_CODEC_TYPE_WAV:
    case SINGER_CODEC_TYPE_AIFF_GROUP:
    case SINGER_CODEC_TYPE_AIFF:
    case SINGER_CODEC_TYPE_FLAC_GROUP:
    case SINGER_CODEC_TYPE_FLAC:
    case SINGER_CODEC_TYPE_ALAC_GROUP:
    case SINGER_CODEC_TYPE_ALAC:
    case SINGER_CODEC_TYPE_ATRACLossless:
    case SINGER_CODEC_TYPE_WMALossless:
      switch(p->bit) {
        case 16:
          heq_prev_data = LPCM_BASE_VALUE + (SHIFT_FOR_BIT16 << 16);
        break;
        case 24:
          heq_prev_data = LPCM_BASE_VALUE + (SHIFT_FOR_BIT24 << 16);
        break;
        case 32:
          heq_prev_data = LPCM_BASE_VALUE + (SHIFT_FOR_BIT32 << 16);
        break;
        default:
          FORZA_ERROR_LOG("%s: unsupported bit value:%d  \n", __FUNCTION__, p->bit);
          return BOOT_FAILURE; 
      }
  
      if (set_heq_param_flag) {
        if (( ret = cdsp_heq_onoff(chip, HEQ_ON)) != 0)
          FORZA_ERROR_LOG("%s: cdsp_heq_off failed  \n", __FUNCTION__);
      }
    break;

    case SINGER_CODEC_TYPE_MP3_GROUP:
    case SINGER_CODEC_TYPE_MP3:
    case SINGER_CODEC_TYPE_WMA_GROUP:
    case SINGER_CODEC_TYPE_WMAv2:
    case SINGER_CODEC_TYPE_WMAPro:
      if (( ret = calc_dataval_from_bitrate(chip, CODEC_MP3_BASE_VAL, p)) != 0)
        FORZA_ERROR_LOG("%s: calc_dataval_from_bitrate failed  \n", __FUNCTION__);
    break;
 
    case SINGER_CODEC_TYPE_AAC_GROUP:
    case SINGER_CODEC_TYPE_LCAAC:
    case SINGER_CODEC_TYPE_HEAAC:
    case SINGER_CODEC_TYPE_HEAACv2:
      if (( ret = calc_dataval_from_bitrate(chip, CODEC_AAC_BASE_VAL, p)) != 0)
        FORZA_ERROR_LOG("%s: calc_dataval_from_bitrate failed  \n", __FUNCTION__);
    break;

    case SINGER_CODEC_TYPE_ATRAC_GROUP:
    case SINGER_CODEC_TYPE_ATRAC3:
    case SINGER_CODEC_TYPE_ATRACX:
      if (( ret = calc_dataval_from_bitrate(chip, CODEC_ATRAC3_BASE_VAL, p)) != 0)
        FORZA_ERROR_LOG("%s: calc_dataval_from_bitrate failed  \n", __FUNCTION__);
    break;

    case SINGER_CODEC_TYPE_DSD_GROUP:
    case SINGER_CODEC_TYPE_DSD:
    case SINGER_CODEC_TYPE_DSDIFF:
    case SINGER_CODEC_TYPE_VORBIS_GROUP:
    case SINGER_CODEC_TYPE_VORBIS:
      FORZA_ERROR_LOG("%s: unsupported mode option:%d  \n", __FUNCTION__, p->mode);
      ret = BOOT_FAILURE;
    break;
  
    default:
      FORZA_ERROR_LOG("%s: (default case) unsupported mode option:%d  \n", __FUNCTION__, p->mode);
      ret = BOOT_FAILURE;
  }

  return ret;
}

int cdsp_register_read_write(struct snd_forza *chip, struct ioctl_struct *p)
{
  unsigned long treg;

  FORZA_DEBUG_LOG("%s: p->key=%d \n", __FUNCTION__, p->key);

  treg = FORZA_SPI_DSP_REG_BASE + p->key;
  if (treg < FORZA_SPI_DSP_REG_BASE)
    return BOOT_FAILURE;
  if (treg > FORZA_SPI_DSP_REG_BASE + 0x20) 
    return BOOT_FAILURE;

  FORZA_DEBUG_LOG("READ=(%x)\n", FORZA_IORD_32(SPI_DSP_BAR(chip), treg));
  udelay(10);

  FORZA_IOWR_32(SPI_DSP_BAR(chip), treg, p->val);

  udelay(10);
  FORZA_DEBUG_LOG("READ=(%x)\n",FORZA_IORD_32(SPI_DSP_BAR(chip), treg));
  return BOOT_OK;
}

int read_rx_without_read_command(struct snd_forza *chip)
{
  int ret = 0;

  /* Check the DSPINT bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_DSP_INT)) != 0) {
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_DSP_INT  \n", __FUNCTION__);
    return ret;
  }

  /* Send the DUMMY DATA */
  SPI_DSP_TX_REG_W(chip, CDSP_DUMMY_DT);

#ifndef CDSP_TMT_ON
  udelay(19);
#endif

  /* Set RRDY and OE bit as 1, rest 0 */
  //  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_TRRDY | SPI_DSP_OE));

  /* Check the RRDY bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_RRDY)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_RRDY interrupt  \n", __FUNCTION__);
    return ret;
  }
  
  /* read the Rx register */
  return SPI_DSP_RX_REG_R(chip);
}

int cdsp_hardware_version(struct snd_forza *chip)
{
  int ret = 0;

  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  /* write the WRITE command */
  SPI_DSP_TX_REG_W(chip, WRITE_COMMAND);

  /* write DSP version */
  SPI_DSP_TX_REG_W(chip, CDSP_VERSION);

  /* Check the TX_Empty bit from status register */
  if ((ret = check_interrupt(chip, SPI_DSP_TMT)) != 0){
    FORZA_ERROR_LOG("%s: could not get SPI_DSP_TMT  \n", __FUNCTION__);
    return ret;
  }
  /*clear the TMT bit */
  SPI_DSP_STATUS_REG_W(chip, (~SPI_DSP_TMT & SPI_DSP_STATUS_REG_R(chip)));

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  /* set SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (SPI_DSP_SSO | SPI_DSP_CTRL_REG_R(chip)));

  if ((ret = read_rx_register(chip)) != VER_FIX_VALUE) { 
    FORZA_ERROR_LOG("%s: 1st word of version read failed, value:0x%x\n", __FUNCTION__, ret);
    return ret;
  }

  ret = read_rx_without_read_command(chip);
  FORZA_DEBUG_LOG("%s: version info:%x \n", __FUNCTION__, ret);

  /* clear SSO bit of control register */
  SPI_DSP_CTRL_REG_W(chip, (~SPI_DSP_SSO & SPI_DSP_CTRL_REG_R(chip)));

  return ret;
}

int cdsp_write_formats(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  FORZA_DEBUG_LOG("%s: p->fs:%d \n", __FUNCTION__, p->fs);

  switch(p->fs) {
    case 44100:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_4UPSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FONE) ;
      break;
    case 48000:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_4UPSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FTWO);
      break;
    case 88200:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_2UPSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FONE);
      break;
    case 96000:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_2UPSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FTWO);
      break;
    case 176400:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_NONSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FONE);
      break;
    case 192000:
      if ((ret = cdsp_over_sampling_fschange(chip,CONFIG_NONSRC)) != 0)
        return ret;
      ret = cdsp_audio_osc_change(chip, FTWO);
      break;
  }
  return ret;
}
