/*
 *  cdsp_cs48l10.h is forza driver for custom dsp
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef __CDSP_CS48L10_H
#define __CDSP_CS48L10_H

#define CDSP_VERSION	0xf0c00005
#define CDSP_FORMATS	0x00000000
#define VER_FIX_VALUE	0x70c00005

#define INTERRUPT_CLR	0x00000000

#define WRITE_COMMAND	(0x80000000)
#define READ_COMMAND	(0x81000000)

#define CDSP_DUMMY_DT	(0x00000000)

#define BOOT_START	(0x00000001)
#define BOOT_SUCCESS	(0x00000002)
#define APP_START	(0x00000004)
#define BOOT_READY	(0x00000005)

#define SOFT_RESET 	(0x40000000)
#define SLAVE_BOOT 	(0x80000000)

#define SOFTBOOT1	(0x81000009)
#define SOFTBOOT2	(0x00000001)

#define KICKSTART1	(0x81000000)
#define KICKSTART2	(0x00011001)

#define FONE		(44)
#define FTWO		(48)

#define AUDIOFS_COMMON	(0xF0000001)
#define FONE_AUDIOFS 	(0x00010000)
#define FTWO_AUDIOFS 	(0x00000000)


#define BOOT_OK			0
#define BOOT_FAILURE		(0x000000F8)
#define REQ_DSP_RESET           (0x000000FD)
#define INVALID_BOOT_TYPE	(0x000000FE)
#define BOOT_ERROR_CHECKSUM	(0x000000FF)

#define ALLEGRO_DELAY	5
#define MAX_WAIT_TIME	2000   // real measure 800us is found

#define MUTE_UNMUTE_COMMON	(0x83000001)
#define MUTE			1
#define UNMUTE			2
#define MUTE_VALUE		(0x00000001)
#define UNMUTE_VALUE		(0x00000000)

#define CDSP_HEQ_ADDR		(0xF0000003)
#define HEQ_ON			1
#define HEQ_OFF			2
#define HEQ_OFF_DATA		(0x00000000)


#define LPCM_BASE_VALUE		(0x00010000)
#define SHIFT_FOR_BIT16		0
#define SHIFT_FOR_BIT24		1
#define SHIFT_FOR_BIT32		2

#define CODEC_MP3_BASE_VAL	(0x00040000)
#define CODEC_AAC_BASE_VAL	(0x00150000)
#define CODEC_ATRAC3_BASE_VAL	(0x00260000)
#define CODEC_ATRACX_BASE_VAL	(0x00370000)

#define CONFIG_4UPSRC  "/sony/lib/modules/dspfw/cdsp_48L10_3_pre_4upSRC.bin"
#define CONFIG_2UPSRC  "/sony/lib/modules/dspfw/cdsp_48L10_3_pre_2upSRC.bin"
#define CONFIG_NONSRC  "/sony/lib/modules/dspfw/cdsp_48L10_3_pre_nonSRC.bin"


int cdsp_firmware_download(struct snd_forza *chip, struct ioctl_struct *p);
int cdsp_mute_unmute(struct snd_forza *chip, int flag);
int cdsp_register_read_write(struct snd_forza *chip, struct ioctl_struct *p);
int cdsp_hardware_version(struct snd_forza *chip);
int cdsp_write_formats(struct snd_forza *chip, struct ioctl_struct *p);

int cdsp_over_sampling_fschange(struct snd_forza *chip, const char *filename);
int cdsp_audio_osc_change(struct snd_forza *chip, int freq);

int cdsp_heq_onoff(struct snd_forza *chip, int flag);
int cdsp_set_heq_param(struct snd_forza *chip, struct ioctl_struct *p);

int send_soft_boot(struct snd_forza *chip);

#endif
