/*
 *  dsp_ops.c is forza driver for custom dsp
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/delay.h>
#include "forza_lib.h"
#include "forza_hwlow.h"
#include "dsp_ops.h"
#include "adsp_21488.h"
#include "cdsp_cs48l10.h"


/* for Spiritoso */
void adsp_hw_init(struct snd_forza *chip, struct ioctl_struct *p){
  chip->heq_control = DEVICE_ENABLE;
  SPI_DSP_CTRL_REG_W(chip,(~SPI_DSP_CMODE & SPI_DSP_CTRL_REG_R(chip)) );
  if(p->val == 2){ 
     FORZA_DEBUG_LOG("ADSP_HOST_DIS\n");
    SPI_DSP_CTRL_REG_W(chip,(~SPI_DSP_OE & SPI_DSP_CTRL_REG_R(chip)) );
  }else{        
     FORZA_DEBUG_LOG("ADSP_HOST_EN\n");
    SPI_DSP_CTRL_REG_W(chip,SPI_DSP_OE    | SPI_DSP_CTRL_REG_R(chip));

  }
  FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_CTRL_REG_R(chip));
  udelay(20);
  return;
}

void adsp_hw_reset(struct snd_forza *chip, struct ioctl_struct *p){
  if(p->val == 2){ 
    UMODE_IO_REG_W(chip,~UMODE_DSP_RESET_DIS & UMODE_IO_REG_R(chip));
    FORZA_DEBUG_LOG("ADSP_H/W_RESET...:%04x\n",UMODE_IO_REG_R(chip));
  }else if(p->val == 1){ 
    UMODE_IO_REG_W(chip, UMODE_DSP_RESET_DIS | UMODE_IO_REG_R(chip));
    FORZA_DEBUG_LOG("ADSP_H/W_RESET-RELEASE:%04x\n",UMODE_IO_REG_R(chip));
  }else{
    FORZA_DEBUG_LOG("ADSP_HW_RESET_CYCLE RESET->RELEASE\n");
    UMODE_IO_REG_W(chip,~UMODE_DSP_RESET_DIS & UMODE_IO_REG_R(chip));
    udelay(500);
    UMODE_IO_REG_W(chip,UMODE_DSP_RESET_DIS | UMODE_IO_REG_R(chip));
  }
  udelay(500);
  return;
}

int adsp_hw_fwdl(struct snd_forza *chip, struct ioctl_struct *p){
  FORZA_DEBUG_LOG("ADSP_HOST_EN\n");
  SPI_DSP_CTRL_REG_W(chip,SPI_DSP_OE    | SPI_DSP_CTRL_REG_R(chip));//| SPI_DSP_SSO);
  SPI_DSP_CTRL_REG_W(chip,SPI_DSP_CMODE | SPI_DSP_CTRL_REG_R(chip));//| SPI_DSP_SSO);
  udelay(100);
  adsp_21488_fwdl(chip, p);
  FORZA_INFO_LOG("ADSP_FWDL by HOST ..OK\n");
  udelay(800);
  udelay(800);
  udelay(800);
  udelay(800);
  udelay(800);
  return 0;
}

void adsp_hw_mute(struct snd_forza *chip, struct ioctl_struct *p){
  if(p->val ==1){
    FORZA_DEBUG_LOG("ADSP_HW_MUTE\n");
    adsp_21488_send_data(chip, ADSP_MUTE_ENA);
  }else if(p->val==2){
    FORZA_DEBUG_LOG("ADSP_HW_UNMUTE\n");
    adsp_21488_send_data(chip, ADSP_MUTE_DIS);
  }
  return;
}

void adsp_hw_heq(struct snd_forza *chip, struct ioctl_struct *p){
  if(chip->heq_control == DEVICE_DISABLE){
    FORZA_DEBUG_LOG("<<<<FORCE OFF>>>>>>%s \n", __FUNCTION__);
    p->val=FORZA_DIS;
  }
  FORZA_DEBUG_LOG("ADSP_HEQ_REQUESET\n");
  if(p->val ==FORZA_HEQ_RUN_NORMAL){
    FORZA_INFO_LOG("ADSP_HEQ_On(Normal)\n");
    adsp_21488_send_data(chip, ADSP_HEQ_NORM);
  }else if(p->val==FORZA_HEQ_RUN_IMPROVED){
    FORZA_INFO_LOG("ADSP_HEQ_On(Improved)\n");
    adsp_21488_send_data(chip, ADSP_HEQ_IMPR);
  }else if(p->val==FORZA_HEQ_RUN_DISABLE){
    FORZA_INFO_LOG("ADSP_HEQ_Off\n");
    adsp_21488_send_data(chip, ADSP_HEQ_DIS);
  }
  udelay(50);
}

void adsp_hw_filter(struct snd_forza *chip, struct ioctl_struct *p){
  if(p->val == HEQ_FILTER_NORMAL){
    FORZA_INFO_LOG("ADSP_HEQ_FILTER_NORMAL\n");
    adsp_21488_send_data(chip, ADSP_FILTER_MODE_NORMAL);
  }else if(p->val== HEQ_FILTER_HI_PRE){
    FORZA_INFO_LOG("ADSP_HEQ_FILTER_HI_PRE\n");
    adsp_21488_send_data(chip, ADSP_FILTER_MODE_HI_PRE);
  }
  udelay(50);
}

void adsp_hw_status(struct snd_forza *chip,struct ioctl_struct *p){
   FORZA_DEBUG_LOG("ADSP_HW_STATUS\n");
   FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_STATUS_REG_R(chip));
   return;
}

int adsp_hw_formats(struct snd_forza *chip,struct ioctl_struct *p){
  int ret=0;
  adsp_21488_set_formats(chip, p);
  return ret;
}

int adsp_hw_org_params(struct snd_forza *chip,struct ioctl_struct *p){
  int ret=0;
  if(chip->heq_control == DEVICE_DISABLE){
    FORZA_DEBUG_LOG("<<<<FORCE OFF>>>>>>%s \n", __FUNCTION__);
    adsp_21488_send_data(chip, ADSP_HEQ_DIS);
  }else{
    adsp_21488_set_heq_params(chip, p);
  }
  return ret;
}


void adsp_hw_reg_wr(struct snd_forza *chip,struct ioctl_struct *p){
  unsigned long treg;
  treg = FORZA_SPI_DSP_REG_BASE + p->key;
  if(treg < FORZA_SPI_DSP_REG_BASE) return;
  if(treg > FORZA_SPI_DSP_REG_BASE + 0x20) return;
  FORZA_DEBUG_LOG("ADSP_HW_REG_WRITE\n");
  FORZA_DEBUG_LOG("READ=(%x)\n",FORZA_IORD_32(SPI_DSP_BAR(chip),treg));
  FORZA_IOWR_32(SPI_DSP_BAR(chip),treg,p->val);
  FORZA_DEBUG_LOG("READ=(%x)\n",FORZA_IORD_32(SPI_DSP_BAR(chip),treg));
  return;
}


int adsp_hw_version(struct snd_forza *chip){
  unsigned int version;
  version= adsp_21488_read_data(chip, ADSP_VERSION);
  FORZA_INFO_LOG("ADSP_FW_VERSION   ..0x%02x\n",version);
  return (int)version;
}

void adsp_hw_remove(struct snd_forza *chip){
   FORZA_DEBUG_LOG("ADSP_HW_REMOVE\n");
   FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_CTRL_REG_R(chip));
  return;
}

void adsp_hw_recovery(struct snd_forza *chip, struct ioctl_struct *p)
{
    FORZA_INFO_LOG("CDSP_FORCE_RECOVERY!!!!!!\n");
}

void adsp_hw_reload_firmware(struct snd_forza *chip, struct ioctl_struct *p)
{
    FORZA_INFO_LOG("CDSP_RELOAD: it's allegro function\n");
}
/* 
 * ADSP-21488 SHARC DSP for Spilitoso
 * Operations
 */
static struct dsp_hw_access adsp_ops __devinitdata ={
  .init    = adsp_hw_init,
  .reset   = adsp_hw_reset,
  .fwdl    = adsp_hw_fwdl,
  .mute    = adsp_hw_mute,
  .heq     = adsp_hw_heq,
  .filter  = adsp_hw_filter,
  .status  = adsp_hw_status,
  .formats = adsp_hw_formats,
  .heq_params  = adsp_hw_org_params,
  .reg_wr  = adsp_hw_reg_wr,
  .version = adsp_hw_version,
  .remove  = adsp_hw_remove,
  .recover = adsp_hw_recovery,
  .reload  = adsp_hw_reload_firmware,
};



void cdsp_hw_recovery(struct snd_forza *chip, struct ioctl_struct *p)
{
  struct ioctl_struct st;
  st.val = 2;  
  chip->dsp_ops->init(chip, &st);
  udelay(800);

  st.val= 2;
  chip->dsp_ops->reset(chip, &st);
  udelay(800);

  st.val= 1;
  chip->dsp_ops->reset(chip, &st);
  udelay(800);

  st.val= 1; 
  chip->dsp_ops->init(chip, &st);
  udelay(800);

  st.val= 1; 
  chip->dsp_ops->fwdl(chip, &st);
  udelay(800);

  FORZA_INFO_LOG("CDSP_FORCE_RECOVERY!!!!!!\n");

}
void cdsp_hw_init(struct snd_forza *chip, struct ioctl_struct *p)
{
  chip->heq_control = DEVICE_ENABLE;
  if(p->val == 2) { 
    FORZA_DEBUG_LOG("CDSP_HOST_DIS\n");
    SPI_DSP_CTRL_REG_W(chip,(~SPI_DSP_OE & SPI_DSP_CTRL_REG_R(chip)) ); // OE=0, DSP<->FLASH memory
  }
  else if(p->val == 1){  // SET
    FORZA_DEBUG_LOG("CDSP_HOST_ENA\n");
    SPI_DSP_CTRL_REG_W(chip,SPI_DSP_OE | SPI_DSP_CTRL_REG_R(chip)); // OE=1, HOST<->DSP
  }else{
    FORZA_INFO_LOG("CDSP_HOST_CONTROL 1:ENA/2:DIS\n");
  }
  SPI_DSP_CTRL_REG_W(chip,(~SPI_DSP_CMODE &( SPI_DSP_FIFOCLR | SPI_DSP_CTRL_REG_R(chip))));
  udelay(5);
  FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_CTRL_REG_R(chip));
  return;
}

void cdsp_hw_reset(struct snd_forza *chip, struct ioctl_struct *p)
{

  if(p->val == 2) {  // RESET
    UMODE_IO_REG_W(chip,~UMODE_DSP_RESET_DIS & UMODE_IO_REG_R(chip));
    udelay(10);
    SPI_DSP_CTRL_REG_W(chip,SPI_DSP_FIFOCLR); // Default + FIFOCLR
    FORZA_DEBUG_LOG("CDSP_RESET...:%04x\n",UMODE_IO_REG_R(chip));
  }
  else if(p->val == 1) { // RELEASE
    UMODE_IO_REG_W(chip,UMODE_DSP_RESET_DIS | UMODE_IO_REG_R(chip));
    FORZA_DEBUG_LOG("CDSP_RESET-RELEASE:%04x\n",UMODE_IO_REG_R(chip));
  }
  else {
    UMODE_IO_REG_W(chip,~UMODE_DSP_RESET_DIS & UMODE_IO_REG_R(chip));
    udelay(500);
    UMODE_IO_REG_W(chip,UMODE_DSP_RESET_DIS | UMODE_IO_REG_R(chip));
    FORZA_DEBUG_LOG("UMODE=(%x)\n",UMODE_IO_REG_R(chip));
  }
  udelay(500);
  return;
}

int cdsp_hw_fwdl(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret=0;
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  ret = cdsp_firmware_download(chip, p);
  if (ret != 0)
    FORZA_ERROR_LOG("%s: error in firmware downlaod \n", __FUNCTION__);
  return ret;
}

void cdsp_hw_mute(struct snd_forza *chip, struct ioctl_struct *p)
{
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  if (cdsp_mute_unmute(chip, p->val) != 0)
    FORZA_ERROR_LOG("%s: error while MUTE/UNMUTE command \n", __FUNCTION__);
  return;
}

void cdsp_hw_heq(struct snd_forza *chip, struct ioctl_struct *p)
{
  if(chip->heq_control == DEVICE_DISABLE){
    FORZA_DEBUG_LOG("<<<<FORCE OFF>>>>>>%s \n", __FUNCTION__);
    p->val=FORZA_DIS;
  }

  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  if (cdsp_heq_onoff(chip, p->val) != 0)
    FORZA_ERROR_LOG("%s: error in HEQ ON_OFF command \n", __FUNCTION__);

  return;
}

int cdsp_hw_formats(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  if (cdsp_write_formats(chip, p) != 0){
    FORZA_ERROR_LOG("%s: error in writing hw formats  \n", __FUNCTION__);
    ret = -1;
  }
  return ret;
}

int cdsp_hw_heq_params(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret=0;
  if(chip->heq_control == DEVICE_DISABLE){
    FORZA_DEBUG_LOG("<<<<FORCE OFF>>>>>>%s \n", __FUNCTION__);
    p->val=FORZA_DIS;
    if (cdsp_heq_onoff(chip, p->val) != 0)
      FORZA_ERROR_LOG("%s: error in HEQ ON_OFF command \n", __FUNCTION__);
  }else{
    FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
    if (cdsp_set_heq_param(chip, p) != 0)
      FORZA_ERROR_LOG("%s: error in set HEQ param \n", __FUNCTION__);
  }
  return ret;
}

void cdsp_hw_filter(struct snd_forza *chip, struct ioctl_struct *p){
  FORZA_INFO_LOG("CDSP_HEQ_FILTER_NORMAL_ONLY\n");
  udelay(50);
}

void cdsp_hw_status(struct snd_forza *chip, struct ioctl_struct *p)
{
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_STATUS_REG_R(chip));
  return;
}

void cdsp_hw_reg_wr(struct snd_forza *chip, struct ioctl_struct *p)
{
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  if (cdsp_register_read_write(chip, p) != 0)
    FORZA_ERROR_LOG("%s: error in hw register read/write  \n", __FUNCTION__);

  return;
}

int cdsp_hw_version(struct snd_forza *chip)
{
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  return cdsp_hardware_version(chip);
}

void cdsp_hw_remove(struct snd_forza *chip)
{
  FORZA_DEBUG_LOG("%s \n", __FUNCTION__);
  FORZA_DEBUG_LOG("STATUS=(%x)\n",SPI_DSP_CTRL_REG_R(chip));
  return;
}

int full_fw_download = 0;
void cdsp_set_fwdl_flag(int flag)
{
  FORZA_DEBUG_LOG("%s: flag=%d (0:OFF, 1:ON)\n", __FUNCTION__, flag);
  if ((flag != 1) && (flag != 0)) {
    FORZA_ERROR_LOG("%s: Invalid parameter \n", __FUNCTION__);
    return;
  }
  full_fw_download = flag;
  return;
}

int cdsp_hw_reload_firmware(struct snd_forza *chip, struct ioctl_struct *p)
{
  int ret = 0;
  int mode_save = full_fw_download;
  FORZA_DEBUG_LOG("%s (new Fs=%d)\n", __FUNCTION__, chip->cur_st.crate);
  p->fs = chip->cur_st.crate;
  full_fw_download = 1;
  if (cdsp_write_formats(chip, p) != 0){
    FORZA_ERROR_LOG("%s: error in writing hw formats  \n", __FUNCTION__);
    ret = -1;
  }
  full_fw_download = mode_save;
  return ret;
}

static struct dsp_hw_access cdsp_ops __devinitdata ={
  .init    = cdsp_hw_init,
  .reset   = cdsp_hw_reset,
  .fwdl    = cdsp_hw_fwdl,
  .mute    = cdsp_hw_mute,
  .heq     = cdsp_hw_heq,
  .filter  = cdsp_hw_filter,
  .formats = cdsp_hw_formats,
  .heq_params  = cdsp_hw_heq_params,
  .status  = cdsp_hw_status,
  .reg_wr  = cdsp_hw_reg_wr,
  .version = cdsp_hw_version,
  .remove  = cdsp_hw_remove,
  .fwdl_flag = cdsp_set_fwdl_flag,
  .recover  = cdsp_hw_recovery,
  .reload   = cdsp_hw_reload_firmware,
};

/* 
 * Registering DSP Operation to Forza Dev by H/W model Information
 * a = Allegro
 * s = Spilitoso
 */
int dsp_hw_create(struct snd_forza *chip)
{
  if(chip->fmode =='a'){       chip->dsp_ops = &cdsp_ops;
  }else if(chip->fmode == 's'){chip->dsp_ops = &adsp_ops;
  }
  return 0;
}


int dsp_hw_remove(struct snd_forza *chip)
{
  if(chip->fmode =='a'){       chip->dsp_ops->remove(chip);
  }else if(chip->fmode == 's'){chip->dsp_ops->remove(chip);
  }
  return 0;
}


