/*
 *  forza_if.h is forza driver for custom dsp
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef __FORZA_IF_H
#define __FORZA_IF_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * IO Controll Access 
 * # device name "/dev/forza"
 *
 * # ex struct ioctl_struct st; 
 * #    fd = open("/dev/forza", O_RDWR);
 * #    ioctl(fd,FORZA_IOCTL_DSP_SET,&st);
 * #    close (fd);
 * 
 */

typedef enum _API_ACCESS_CMD{
  FORZA_API_INIT = 0,
  FORZA_API_RESET,
  FORZA_API_MUTE,
  FORZA_API_INPUT,
  FORZA_API_SMUTE,
  FORZA_API_S_PCM,
  FORZA_API_HEQ = 10,
  FORZA_API_HEQ_PARAM,
  FORZA_API_FADE,
  FORZA_API_FADE_PARAM,
  FORZA_API_FORMAT,
  FORZA_API_ATT,
  FORZA_API_FORMATTER = 80,
  FORZA_API_HEQ_CONTROL = 97,
  FORZA_API_DEBUG_TEST = 98,
  FORZA_API_DEBUG_LED = 99,
}API_ACCESS_CMD;


typedef enum _FORZA_CTRL{
  FORZA_OK = 0,
  FORZA_ENA ,
  FORZA_DIS ,
  FORZA_NG = -1,  
}FORZA_CTRL;

typedef enum _DSP_CTRL_ACCESS_CMD{
  FORZA_DSP_INIT = 0,
  FORZA_DSP_RESET,
  FORZA_DSP_FWDL,
  FORZA_DSP_MUTE,
  FORZA_DSP_STATUS,
  FORZA_DSP_FORMATS,
  FORZA_DSP_REG_WRITE,
  FORZA_DSP_VERSION,
  FORZA_DSP_REMOVE,
  FORZA_DSP_FWDL_FLAG,
  FORZA_DSP_FILTER,
  FORZA_DSP_RECOVER,
  FORZA_DSP_RELOAD,
}DSP_CTRL_ACCESS_CMD;

typedef enum _DAC_CTRL_ACCESS_CMD{
  FORZA_DAC_INIT = 0,
  FORZA_DAC_RESET,
  FORZA_DAC_DEFAULT,
  FORZA_DAC_MUTE,
  FORZA_DAC_PCM,
  FORZA_DAC_DSD,
  FORZA_DAC_DIR,
  FORZA_DAC_FIR,
}DAC_CTRL_ACCESS_CMD;

typedef enum _HEQ_EFFECT_MODE{
  FORZA_HEQ_MODE_OFF = 0,
  FORZA_HEQ_MODE_LPCM,
  FORZA_HEQ_MODE_MP3,
  FORZA_HEQ_MODE_AAC,
  FORZA_HEQ_MODE_WMA,
  FORZA_HEQ_MODE_ATRAC3,
  FORZA_HEQ_MODE_ATRACX,
}HEQ_EFFECT_MODE;

typedef enum _DEBUG_LED_MODE{
  FORZA_DEBUG_LED_FIFO = 0,
  FORZA_DEBUG_LED_VU,
  FORZA_DEBUG_LED_DEMO,
  FORZA_DEBUG_LED_CPU,
}DEBUG_LED_MODE;

typedef enum _HEQ_FILTER_MODE{
  HEQ_FILTER_NORMAL = 0,
  HEQ_FILTER_HI_PRE,
}HEQ_FILTER_MODE;

typedef enum _FORZA_HEQ_RUN_MODE{
  FORZA_HEQ_RUN_NORMAL = 1,
  FORZA_HEQ_RUN_DISABLE,
  FORZA_HEQ_RUN_IMPROVED,  
}FORZA_RUN_MODE;

struct ioctl_struct{
  int cmd;   
  int key;   
  int val;   
  int mode;  
  int rate;  
  int fs;    
  int bit;   
};

#define FORZA_IOCTL_PATH "/dev/forza"

#define FORZA_IOCTL_MAGIC 0xDF
#define FORZA_IOCTL_API_SET _IOC( _IOC_WRITE, FORZA_IOCTL_MAGIC, 1, sizeof(int))
#define FORZA_IOCTL_DSP_SET _IOC( _IOC_WRITE, FORZA_IOCTL_MAGIC, 2, sizeof(int))
#define FORZA_IOCTL_DAC_SET _IOC( _IOC_WRITE, FORZA_IOCTL_MAGIC, 3, sizeof(int))
#define FORZA_IOCTL_DSET _IOC( _IOC_WRITE, FORZA_IOCTL_MAGIC, 0, sizeof(int))

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // __FORZA_IF_H
