/*
 *  forza.c is alsa driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
#include <linux/module.h> 
#include <linux/kernel.h> 
#include <linux/init.h>
#include <asm/io.h>  
#include <linux/cdev.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/io.h>
#include <linux/pci.h> 
#include <linux/sched.h>
#include <linux/moduleparam.h>
#include <linux/slab.h>
#include <sound/core.h>
#include <asm/cacheflush.h>
#include <sound/initval.h>
#include <sound/info.h>

#include "forza.h"
#include "forza_lib.h"
#include "forza_hwlow.h"

MODULE_AUTHOR("Forza Team");
MODULE_DESCRIPTION("SONY ANAP PCI");
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("{{Sony, Forza}}");


static int enable[SNDRV_CARDS] = SNDRV_DEFAULT_ENABLE;
static int  index[SNDRV_CARDS] = SNDRV_DEFAULT_IDX;
static char   *id[SNDRV_CARDS] = SNDRV_DEFAULT_STR;
static char *fmode = NULL;
module_param(fmode, charp, S_IRUGO);

static const struct pci_device_id snd_forza_ids[] = {
  { PCI_DEVICE(0x1172, 0xE001),},
  {0,}
};

MODULE_DEVICE_TABLE(pci, snd_forza_ids);

/***************************************************************************/
  
static int __devinit snd_forza_probe(struct pci_dev *pci, const struct pci_device_id *pci_id)
{
  int err;
  static int dev;
  struct snd_card *card =NULL;
  struct snd_forza *chip = NULL;

  FORZA_DEBUG_LOG("FORZA_CARD_PROBE...stating >>> \n");
  printk(KERN_DEBUG "probe(dev = 0x%x, pciid = 0x%x)\n", (int)dev, (int)pci_id);

  if (dev >= SNDRV_CARDS)   return -ENODEV;
  if (!enable[dev]) {
    dev++;
    return -ENOENT;
  }

  err = snd_card_create(index[dev], id[dev], THIS_MODULE, 0, &card);
  if(err < 0){
    printk(KERN_ALERT "snd_card_crerate ERROR!! \n");
    return err;
  }
  strncpy(card->driver, DEVICE_NAME, sizeof(DEVICE_NAME));
  strncpy(card->shortname, DEVICE_NAME, sizeof(DEVICE_NAME));
  // Create & Setup the "Forza"
  if(fmode == NULL)    err = snd_forza_create(card, pci, &chip, 'a');
  else                 err = snd_forza_create(card, pci, &chip, *fmode);

  if(err < 0){
    printk(KERN_ALERT "forza_chip_setting ERROR");
    return err;
  }

  FORZA_DEBUG_LOG(" > forza_create:OK\n");
  if (err < 0){
    printk(KERN_ALERT "snd_forza_crerate ERROR!! \n");
    return err;
  }
  err = snd_forza_new_pcm(chip,0);
  if (err < 0)    goto error;

  err = snd_forza_new_pcm_dsd(chip, 1);
  if (err < 0)    goto error;

  err = snd_forza_fpga_ctrl(chip);

  FORZA_DEBUG_LOG(" > forza_new_pcm:OK\n");
  err = snd_card_register(card);
  if (err < 0)    goto error;

  pci_set_drvdata(pci,card);
  dev++;
  chip->dma_ops->init(chip);
  FORZA_INFO_LOG("CARD_PROBE       ...OK ");
  return 0;
  
 error:
  FORZA_ERROR_LOG("PROBE Failed  !! \n");
  snd_card_free(card);
  return err;
}

void __devexit snd_forza_remove(struct pci_dev *pci)
{
  FORZA_INFO_LOG("****FORZA_REMOVE***\n\n");
  snd_card_free(pci_get_drvdata(pci));
  pci_set_drvdata(pci,NULL);

}

static struct pci_driver driver = {
  .name     = DEVICE_NAME,
  .id_table = snd_forza_ids,
  .probe    = snd_forza_probe,
  .remove   = __devexit_p(snd_forza_remove),
};

static int __init alsa_card_forza_init(void)
{
  int err;
  if(fmode == NULL) {
    FORZA_DEBUG_LOG("\n !! install error Please add fmode=a or s\n");
    FORZA_DEBUG_LOG("\n !! Force Allegro mode is selected \n");
  }

  forza_print_build_info();

  err = pci_register_driver(&driver);
  if(err<0)    FORZA_ERROR_LOG("Forza PCIe ERROR!! Couldn't Install Device!!\n");
  else         FORZA_DEBUG_LOG("\n***** << Forza FPGA fully initialized..Ready!! >> *****\n");

  return err;
}

static void __exit alsa_card_forza_exit(void)
{
  FORZA_INFO_LOG("Good-by %s!\n", DEVICE_NAME);
  pci_unregister_driver(&driver);
}

module_init(alsa_card_forza_init);
module_exit(alsa_card_forza_exit);
