/*
 *  forza.h is alsa driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef __SOUND_FORZA_H
#define __SOUND_FORZA_H


#include <sound/pcm.h>
#include <linux/sched.h>
#include "forza_defs.h"


typedef struct {
  void *vortex;
  unsigned int source;
} forza_ctl_t;

typedef struct {
  unsigned int msdelay;
  unsigned int times;
  int          start_pos;
} dma_fifo_delay_t;

typedef struct {
  unsigned int cstream;
  unsigned int crate;
  unsigned int s_pcm;
} cur_substream;

typedef struct {
  int center;
  int gain;
} audio_att;

struct forza_hw_dma;
struct forza_hw_cra;
struct forza_hw_timer;
struct forza_hw_dac;
struct forza_hw_audio;
struct dsp_hw_access;
/*
 * FORZA_CARD_DEVICE_STRUCTURE
 */

struct snd_forza{
  unsigned int device_id; // PCI device ID
  unsigned char rev;      // PCI revision
  char fmode ;            // Model Info 'a' or  's'
  struct pci_dev *pci;
  int irq;
  spinlock_t flock;
  struct wait_queue *con;
  unsigned int int_cnt;

  unsigned long *phy_addr_bar[FORZA_BAR_NUM];
  void * __iomem vir_addr_bar[FORZA_BAR_NUM];
  void * __iomem res_size_bar[FORZA_BAR_NUM];
  unsigned int tx_cra_reg_mask;
  unsigned int tx_cra_reg_base;
  unsigned int txs_dma_reg_base;
  struct snd_dma_buffer work_ptr;
  unsigned int bank_size_playback;
  dma_addr_t bank_base_playback_addr;  
  dma_fifo_delay_t dmfi;
  dma_addr_t dma_phy_addr;
  void * __iomem dma_vir_addr;
  unsigned char *preout;
  unsigned int preout_size;

  struct forza_hw_dma   * dma_ops;
  struct forza_hw_cra   * cra_ops;
  struct forza_hw_timer * timer_ops;
  struct forza_hw_dac   * dac_ops;
  struct forza_hw_audio * audio_ops;
  struct dsp_hw_access  * dsp_ops;
  struct snd_info_entry *proc_entry;

  unsigned int IsPlayReq;
  struct snd_pcm_substream *substream;
  struct snd_pcm_substream *substream_exdsd;
  struct snd_pcm  *pcm;
  struct snd_pcm  *pcm_exdsd;
  struct snd_card *card;

  cur_substream cur_st;
  int output;
  unsigned int board_ver;
  unsigned int pcm_direct;
  unsigned int heq_control;
  unsigned int dsd_fir;
  unsigned int dac_mode;
  audio_att audio_att;
};


#include "forza_hwlow.h"


int __devinit snd_forza_new_pcm(struct snd_forza *chip, int device);
int __devinit snd_forza_new_pcm_dsd(struct snd_forza *chip, int device );
int __devinit snd_forza_create(struct snd_card *card, struct pci_dev *pci,
			       struct snd_forza **rchip, char mode);
int snd_forza_fpga_ctrl(struct snd_forza *chip);
#endif
