/*
 *  forza_audio_controller.c is sound controller for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/delay.h>
#include <asm/delay.h>
#include <linux/timer.h>
#include <linux/slab.h>
#include <linux/unistd.h>

#include "forza.h"
#include "forza_lib.h"
#include "forza_hwlow.h"
#include "dsp_ops.h"
#include "adsp_21488.h"
#include "cdsp_cs48l10.h"
#include "forza_audio_controller.h"

static wait_queue_head_t wq;

static const int ATTtoFade[102] = {
  32768,
  32393,32022,31656,31293,30935,30581,30231,29885,29543,29205,//-1.0dB
  28870,28540,28213,27890,27571,27255,26943,26635,26330,26029,
  25731,25436,25145,24857,24573,24291,24013,23738,23467,23198,
  22932,22670,22410,22154,21900,21650,21402,21157,20915,20675,
  20439,20205,19973,19745,19519,19295,19074,18856,18640,18427,
  18216,18007,17801,17597,17396,17197,17000,16805,16613,16423,
  16235,16049,15865,15684,15504,15327,15151,14978,14806,14637,
  14469,14304,14140,13978,13818,13660,13504,13349,13196,13045,
  12896,12748,12602,12458,12315,12174,12035,11897,11761,11627,
  11493,11362,11232,11103,10976,10851,10726,10604,10482,10362,//-10.0dB
  0
};
static const int AMPtoFade[32] = {
  32768,
  33147,33531,33920,34312,34710,35112,35518,35929,36345,36766,
  37192,37623,38058,38499,38945,39396,39852,40313,40780,41252,
  41730,42213,42702,43197,43697,44203,44715,45232,45756,46286,
  46822
};

static void forza_audio_stop(struct snd_forza *chip);
static void forza_audio_default_set(struct snd_forza *chip, unsigned char fmt);
/* <----------------------------------  AUDIO ----------------------------------------> */
#if (FORZA_AUDIO == DEVICE_ENABLE)

static void forza_audio_init(struct snd_forza *chip, unsigned char fmode){
  struct ioctl_struct st;
  unsigned short version;
  unsigned char tmp;
  int ret=0;
  FORZA_DEBUG_LOG("AUDIO_CORE_INIT:\n");
  chip->dac_ops->init(chip);
  chip->pcm_direct = DEVICE_DISABLE;
  init_waitqueue_head(&wq);
  tmp = UMODE_IO_REG_R(chip);
  UMODE_IO_REG_W(chip, tmp & ~(UMODE_CLOCK_22M_ENA | UMODE_CLOCK_24M_ENA));
  chip->cur_st.cstream = FORZA_AUDIO_UNKNOWN;
  chip->cur_st.crate   = FORZA_AUDIO_UNKNOWN;
  chip->audio_att.center = 0;
  chip->audio_att.gain   = 0;
  //RESET & MUTE
  st.val= 2; 
  chip->dsp_ops->init(chip, &st);
  udelay(800);
  st.val= FORZA_RESET_ENA;
  chip->dsp_ops->reset(chip, &st);
  udelay(800);
  chip->dac_ops->output(chip, FORZA_DIS);
  wait_event_timeout(wq, false, 20); // 3
  udelay(800);
  chip->dac_ops->mute(chip,  PCM1795_MUTE_ENA);
  udelay(800);
  chip->dac_ops->reset(chip,  FORZA_RESET_ENA);
  udelay(800);
  //RELEASE
  chip->dac_ops->reset(chip, FORZA_RESET_DIS);
  st.val= FORZA_RESET_DIS;
  chip->dsp_ops->reset(chip, &st);
  udelay(800);
  st.val= 1; 
  chip->dsp_ops->init(chip, &st);
  
  udelay(800);
  if(chip->fmode == 'a'){
    forza_audio_default_set(chip,chip->fmode); // LRCK required for cdsp platform 
    st.val= 1;  // firmware DL
    ret = chip->dsp_ops->fwdl(chip, &st);
    if(ret != 0) FORZA_DEBUG_LOG("FWDL ERROR:\n");
    udelay(100);
    chip->dsp_ops->version(chip);
  }else if(chip->fmode == 's'){
    st.val= 1;  // firmware DL
    ret = chip->dsp_ops->fwdl(chip, &st);
    if(ret != 0) FORZA_DEBUG_LOG("FWDL ERROR:\n");
    udelay(100);
    version = chip->dsp_ops->version(chip);
    if(version != SPIRITOSO_VERSION){
      FORZA_DEBUG_LOG("NotSupportedVersion:\n");
    }
    udelay(100);
    forza_audio_default_set(chip,chip->fmode); // clock required for dac
    udelay(100);
    wait_event_timeout(wq, false, 2);
  }

  if(chip->fmode == 'a'){
    chip->dac_ops->set_default(chip, 'p');
    udelay(20);
  }else if(chip->fmode == 's'){
    chip->dac_ops->set_default(chip, 'd');
  }
  udelay(100);
}


static void forza_audio_delete(struct snd_forza *chip){
  forza_audio_stop(chip);
  FORZA_DEBUG_LOG("AUDIO_CORE_REMOVE:\n");
}

static void forza_audio_reset(struct snd_forza *chip, int reset){
  FORZA_DEBUG_LOG("AUDIO_CORE_RESET:\n");
}

static int forza_osc_pll_check(struct snd_forza *chip){
  if((UMODE_IO_REG_R(chip) & UMODE_PLL_LOCK_ENA) == UMODE_PLL_LOCK_ENA){
    FORZA_DEBUG_LOG("pll_locked:\n");
    return FORZA_OK;
  }
  FORZA_DEBUG_LOG("pll_unlock..:\n");
  return FORZA_NG;
}

static void forza_osc_wait(struct snd_forza *chip){
  int i=0;
  wait_event_timeout(wq, false, 10);
  while(i < 5){
    if(forza_osc_pll_check(chip) == FORZA_OK) break;
    wait_event_timeout(wq, false, 20);
    i++;
  }
  wait_event_timeout(wq, false, 60);
}

static void forza_audio_low_mute(struct snd_forza *chip, int mute, int shift){
  unsigned int tmp32 = 0;
  static unsigned int mute_bit = 0;
  if(mute == DEVICE_DISABLE){
    mute_bit = mute_bit & ~(1 << shift) ;
  }else{
    mute_bit = mute_bit | (1 << shift);
  }
  if(mute_bit == 0){
    tmp32 = (AUDIO_FUNCTION_REG_R(chip) & ~AUDIO_MUTE_ENA);
    FORZA_DEBUG_LOG("AUDIO_SMUTE_DIS:%x\n",tmp32);
  }else{
    tmp32 = (AUDIO_FUNCTION_REG_R(chip) | AUDIO_MUTE_ENA);
    FORZA_DEBUG_LOG("AUDIO_SMUTE_ENA:%x\n",tmp32);
  }
  AUDIO_FUNCTION_REG_W(chip, tmp32);
}

static void forza_audio_mute(struct snd_forza *chip, int mute){
  forza_audio_low_mute(chip, mute, 1);
}

static void forza_audio_formatter(struct snd_forza *chip, int mode){
  unsigned int AUF_reg = 0;
  AUF_reg= UMODE_AUDIO_REG_R(chip);
  if(mode == DEVICE_ENABLE){
    UMODE_AUDIO_REG_W(chip,AUF_reg | AUDIO_FORMATTER_ENABLE);
    FORZA_INFO_LOG("AUDIO_START\n");
  }else if(mode == DEVICE_DISABLE){
    UMODE_AUDIO_REG_W(chip,AUF_reg & ~AUDIO_FORMATTER_ENABLE);
    FORZA_INFO_LOG("AUDIO_STOP\n");
  }
}

static void forza_audio_default_set(struct snd_forza *chip, unsigned char fmt){
  unsigned char UIO_reg = 0;
  unsigned int  AUF_reg = 0;
  if(chip->fmode == 'a'){
    UIO_reg = (UMODE_DSP_RESET_DIS | 
	       UMODE_DAC_RESET_DIS | 
	       UMODE_CLOCK_22M_ENA | 
	       //	   UMODE_CLOCK_DIS     | 
	       UMODE_OUTPUT_MUTE_ENA);
    UMODE_IO_REG_W(chip, UIO_reg);
    udelay(10);//
    AUF_reg =  (FORMAT_PCM_44_1K    |
		FORMAT_16BIT        |
		AUDIO_FADER1_SEL_REG |
		AUDIO_MODEL_ALLEGRO |
		AUDIO_MUTE_ENA      |
		AUDIO_MSTR_XSLV_REG);
    AUDIO_FUNCTION_REG_W(chip, AUF_reg);
    FORZA_DEBUG_LOG("AUDIO_DEFAULT_SETTING..ALLECGO\n");
  }else if(chip->fmode == 's'){
    UIO_reg = (UMODE_DSP_RESET_DIS | 
	       UMODE_DAC_RESET_DIS | 
	       UMODE_CLOCK_22M_ENA | 
	       //	   UMODE_CLOCK_DIS     | 
	       UMODE_OUTPUT_MUTE_ENA);
    UMODE_IO_REG_W(chip, UIO_reg);
    udelay(10); //
    AUF_reg =  (FORMAT_PCM_44_1K      |
		FORMAT_16BIT          |
		AUDIO_FADER1_SEL_REG   |
		AUDIO_MODEL_SPIRITOSO |
		AUDIO_MUTE_ENA        |
		AUDIO_MSTR_XSLV_REG
		);
    AUDIO_FUNCTION_REG_W(chip, AUF_reg);
    FORZA_DEBUG_LOG("AUDIO_DEFAULT_SETTING..SPIRITOSO\n");
  }else{ 
    FORZA_ERROR_LOG("AUDIO_DEFAULT_SETTING..NONE\n");
  }
  udelay(10);
}

static void forza_audio_stop(struct snd_forza *chip)
{
  unsigned char UIO_reg = 0;
  UIO_reg = (UMODE_DSP_RESET_DIS | 
	       UMODE_DAC_RESET_ENA | 
	       UMODE_CLOCK_DIS     | 
	       UMODE_OUTPUT_MUTE_ENA);
  UMODE_IO_REG_W(chip, UIO_reg);
  FORZA_DEBUG_LOG("AUDIO_CORE_STOP\n");
}

static void forza_audio_fader_sel(struct snd_forza *chip, unsigned char fadeNo){
  unsigned int reg = 0;
  switch(fadeNo){
  case 0:
    FORZA_INFO_LOG("AUDIO_CORE FADER0\n" );
    AUDIO_FUNCTION_REG_W(chip, (reg & ~AUDIO_FADER1_SEL_REG)); 
    break;
  case 1:
    FORZA_INFO_LOG("AUDIO_CORE FADER1\n" );
    AUDIO_FUNCTION_REG_W(chip, (reg | AUDIO_FADER1_SEL_REG)); 
    break;
  default:
    break;
  }
  
}

static void forza_audio_fader_set_val(struct snd_forza *chip, unsigned char fadeNo, unsigned int Val){
  switch(fadeNo){
  case 0:
    AUDIO_FADER_0_REG_W(chip, Val);
    FORZA_INFO_LOG("AUDIO_CORE FADER0:(0x%04x)\n", AUDIO_FADER_0_REG_R(chip));
    break;
  case 1:
    AUDIO_FADER_1_REG_W(chip, Val);
    FORZA_INFO_LOG("AUDIO_CORE FADER1:(0x%04x)\n", AUDIO_FADER_1_REG_R(chip));
    break;
  case 2:
    AUDIO_FADER_0_REG_W(chip, Val);
    AUDIO_FADER_1_REG_W(chip, Val);
    FORZA_INFO_LOG("AUDIO_CORE FADER0:(0x%04x), FADER1:(0x%04x)\n", 
		   AUDIO_FADER_0_REG_R(chip), AUDIO_FADER_1_REG_R(chip));
    break;
  case 99:
    FORZA_INFO_LOG("-- help --" );
    FORZA_INFO_LOG("1 0x08000(32768) " );
    FORZA_INFO_LOG("0 0x00000(0) " );
    break;
  default:
    break;
  }

}

static void forza_audio_att(struct snd_forza *chip, int att_center, int  gain_value){
  chip->audio_att.center = att_center;
  chip->audio_att.gain   = gain_value;
  int fadeval = 32768;
  int fadediff =0;
  int gain = att_center + gain_value;
  if(gain < -1000) gain = -1000;
  if(gain > 300) gain = 0;

  if(gain > 0){
    fadeval = AMPtoFade[(gain)/10];
    fadediff = (AMPtoFade[(gain)/10 + 1 ] - fadeval)/10;
    fadediff *= (gain)%10;
    fadeval += fadediff;
    FORZA_INFO_LOG("AUDIO_AMP_Gain(%d/100),Fade(%d),Diff(%d)\n",gain, fadeval,fadediff);
  }else{
    fadeval = ATTtoFade[(-gain)/10];
    fadediff = (fadeval - ATTtoFade[(-gain)/10 +1])/10;
    fadediff *= (-gain)%10;
    fadeval -= fadediff;
    FORZA_INFO_LOG("AUDIO_ATT_Gain(%d/100),Fade(%d),Diff(%d)\n",gain, fadeval,fadediff);
  }
  forza_audio_fader_set_val(chip, 1, fadeval);
}


static int Is_same_OSC(struct snd_forza *chip, unsigned int SRate){
  unsigned int UIO_reg = UMODE_IO_REG_R(chip);
  if( ((UIO_reg & UMODE_CLOCK_22M_ENA) == UMODE_CLOCK_22M_ENA)&&(SRate%44100== 0)){
    FORZA_DEBUG_LOG("OSC->22MHZ:OK\n");
    return FORZA_OK;
  }
  if( ((UIO_reg & UMODE_CLOCK_24M_ENA) == UMODE_CLOCK_24M_ENA)&&(SRate%48000== 0)){
    FORZA_DEBUG_LOG("OSC->24MHZ:OK\n");
    return FORZA_OK;
  }
  FORZA_DEBUG_LOG("OSC.. 22MHz<->24MHz\n");
  return FORZA_NG;
}


static void pcm_fs_set(struct snd_forza *chip, unsigned int   SRate, unsigned char BWith){
  unsigned int UIO_reg;
  unsigned int UAU_reg;
  unsigned int AUF_reg;
  UAU_reg= (
	   AUDIO_FIFO_DEBUG_OUT    |
	   AUDIO_LSB_FIRST         |
	   AUDIO_ENDIAN_LITTLE     |
	   AUDIO_PCIE_INPUT_ENABLE |
	   AUDIO_FORMATTER_DISABLE
	   );
  UMODE_AUDIO_REG_W(chip, UAU_reg );

  UIO_reg = UMODE_IO_REG_R(chip) & ~(UMODE_CLOCK_22M_ENA | UMODE_CLOCK_24M_ENA);
  AUF_reg = AUDIO_FORMAT_MASK & AUDIO_FUNCTION_REG_R(chip);
  if(chip->fmode == 's'){
    if(chip->pcm_direct == DEVICE_ENABLE){
      FORZA_DEBUG_LOG("PCM_DirectMode:ON\n");
      AUF_reg = AUF_reg  | SPIRITOSO_PCM_DIRECT;
    }else{
      FORZA_DEBUG_LOG("PCM_DirectMode:OFF\n");
      AUF_reg = AUF_reg & ~SPIRITOSO_PCM_DIRECT;
    }
  }
  switch(SRate){
  case 44100:
    FORZA_INFO_LOG("FS.PCM->44.1k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_22M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_44_1K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_1024);
    break;
  case 48000:
    FORZA_INFO_LOG("FS.PCM->48k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_24M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_48K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_2048);
    break;
  case 88200:
    FORZA_INFO_LOG("FS.PCM->88.2k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_22M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_88_2K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_3072);
    break;
  case 96000:
    FORZA_INFO_LOG("FS.PCM->96k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_24M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_96K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_3072);
    break;
  case 176400:
    FORZA_INFO_LOG("FS.PCM->1764k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_22M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_176_4K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_4096);
    break;
  case 192000:
    FORZA_INFO_LOG("FS.PCM->192k\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_24M_ENA);
    udelay(10);
    AUDIO_FUNCTION_REG_W(chip,(AUF_reg | FORMAT_32BIT | FORMAT_PCM_192K));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_4096);
    break;
  }
}

static void dsd_fs_set(struct snd_forza *chip, unsigned int   SRate){
  unsigned int UIO_reg;
  unsigned int UAU_reg;
  unsigned int AUF_reg;
  UAU_reg = (
	     AUDIO_FIFO_DEBUG_OUT    |
	     AUDIO_LSB_FIRST         | 
	     AUDIO_ENDIAN_LITTLE     | 
	     AUDIO_PCIE_INPUT_ENABLE |
	     AUDIO_FORMATTER_DISABLE
	     );
  UMODE_AUDIO_REG_W(chip, UAU_reg );

  UIO_reg = UMODE_IO_REG_R(chip) & ~(UMODE_CLOCK_22M_ENA | UMODE_CLOCK_24M_ENA);
  AUF_reg = AUDIO_FORMAT_MASK & AUDIO_FUNCTION_REG_R(chip);
  switch(SRate){
  case 88200:
    FORZA_INFO_LOG("FS.DSD->2.8M\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_22M_ENA);
    AUDIO_FUNCTION_REG_W(chip, (AUF_reg | FORMAT_DSD_2_8M | FORMAT_32BIT | FORMAT_DSD_8BIT));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_3072);
    break;
  case 96000:
    FORZA_INFO_LOG("FS.DSD->3.1M\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_24M_ENA);
    AUDIO_FUNCTION_REG_W(chip, (AUF_reg | FORMAT_DSD_2_8M | FORMAT_32BIT | FORMAT_DSD_8BIT));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_3072);
    break;
  case 176400:
    FORZA_INFO_LOG("FS.DSD->5.6M\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_22M_ENA);
    AUDIO_FUNCTION_REG_W(chip, (AUF_reg | FORMAT_DSD_5_6M | FORMAT_32BIT | FORMAT_DSD_8BIT));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_4096);
    break;
  case 192000:
    FORZA_INFO_LOG("FS.DSD->6.2M\n");
    UMODE_IO_REG_W(chip, UIO_reg | UMODE_CLOCK_24M_ENA);
    AUDIO_FUNCTION_REG_W(chip, (AUF_reg | FORMAT_DSD_5_6M | FORMAT_32BIT | FORMAT_DSD_8BIT));
    FIFO_SIZE_REG_W(chip,FIFO_SIZE_4096);
    break;
  }
}

#define FIRST_WAITTIME_FOR_RELAY 5
#define LAST_WAITTIME_FOR_RELAY 5
#define FIRST_WAITTIME_FOR_TRMUTE 8
#define LAST_WAITTIME_FOR_TRMUTE 8

/*
 * PCM
 * SamplingRate
 * BitWith
 * CodecFormat
 * CodecsRate
 */

static void forza_audio_set_pcm(struct snd_forza *chip, 
				unsigned int   SRate, unsigned char BWith,
				unsigned short CFormat, unsigned short CRate
				)
{
  struct ioctl_struct st;
  int ret;
  st.val = 0;
  st.bit = 24;
  st.fs  = SRate;

  if(chip->fmode == 'a'){     
      if((chip->cur_st.cstream == FORZA_AUDIO_PCM) && (chip->cur_st.crate == SRate) && 
         (chip->dac_mode == FORZA_AUDIO_PCM)){
        FORZA_INFO_LOG("MODE:Direct.PCM->Setup...pass\n");
	return;
      }
      if(chip->cur_st.cstream == FORZA_AUDIO_PCM) FORZA_INFO_LOG("MODE:Direct.PCM->Direct.PCM\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_DSD) FORZA_INFO_LOG("MODE:Direct.DSD->Direct.PCM\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.PCM\n");

      if(Is_same_OSC(chip,SRate) == FORZA_OK){
	chip->dac_ops->output(chip, FORZA_DIS);
	wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_TRMUTE); 
	chip->dac_ops->reset(chip, FORZA_RESET_ENA);
	chip->dac_ops->reset(chip, FORZA_RESET_DIS);
	pcm_fs_set(chip, SRate, BWith);
	wait_event_timeout(wq, false, 1);
	chip->dac_ops->set_default(chip, 'p');
      }else{
	chip->dac_ops->output(chip, FORZA_DIS);
	wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_TRMUTE);
	chip->dac_ops->reset(chip, FORZA_RESET_ENA);
	chip->dac_ops->reset(chip, FORZA_RESET_DIS);
	pcm_fs_set(chip, SRate, BWith);
	FORZA_DEBUG_LOG("UIO-REG 0x%04x\n",UMODE_IO_REG_R(chip) );
	wait_event_timeout(wq, false, 1); // 3  >> DUMMY
	forza_osc_wait(chip);
	FORZA_DEBUG_LOG("Waiting the stable...\n");
	chip->dac_ops->set_default(chip, 'p');
      }
  retry_dsp_setting:
      ret = chip->dsp_ops->formats(chip,&st);  // alegro is requred fs settings sepalated by HEQ Param
      if(ret == -1){
	FORZA_ERROR_LOG("<<<<<<<<<<<<Recovering DSP...>>>>>>>>>>>>>>>\n");
	chip->dsp_ops->recover(chip,&st);
	wait_event_timeout(wq, false, 10); // 3  >> DUMMY
	goto retry_dsp_setting;
      }
      
      wait_event_timeout(wq, false, LAST_WAITTIME_FOR_TRMUTE);
  } // Allegro Settings

  else if(chip->fmode == 's'){
    if((chip->cur_st.cstream == FORZA_AUDIO_PCM) && (chip->cur_st.crate == SRate)){
      if((chip->dac_mode == FORZA_AUDIO_PCM) && (chip->cur_st.s_pcm == DEVICE_ENABLE) && (chip->pcm_direct == DEVICE_ENABLE)){
	FORZA_INFO_LOG("MODE:Direct.PCM->Setup...pass\n");
	return;
      }
      if((chip->dac_mode == FORZA_AUDIO_DSD) && (chip->cur_st.s_pcm == DEVICE_DISABLE) && (chip->pcm_direct == DEVICE_DISABLE)){
	FORZA_INFO_LOG("MODE:ReMaster.PCM->Setup...pass\n");
	return;
      }
    }
    /*******************
     * from DSD Remaster & Direct DSD
     */
    if(chip->cur_st.s_pcm == DEVICE_DISABLE){
        /* to DSD Remaster(settings)  */
        if(chip->pcm_direct == DEVICE_DISABLE){
	  chip->cur_st.s_pcm = DEVICE_DISABLE;
	    if(chip->cur_st.cstream == FORZA_AUDIO_PCM) FORZA_INFO_LOG("MODE:DSD_ReMaster.PCM->DSD_ReMaster.PCM\n");
	    if(chip->cur_st.cstream == FORZA_AUDIO_DSD) FORZA_INFO_LOG("MODE:Direct.DSD->DSD_ReMaster.PCM\n");
	    if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->DSD_ReMaster.PCM\n");
	    /* FS Only */
	    if(Is_same_OSC(chip,SRate) == FORZA_OK){
	        pcm_fs_set(chip, SRate, BWith);
	    }
	    /* FS & OSC Change */
	    else{
	      chip->dac_ops->output(chip, FORZA_DIS);
	      wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	      pcm_fs_set(chip, SRate, BWith);
	      wait_event_timeout(wq, false, 3);
	      FORZA_DEBUG_LOG("Waiting the stable...\n");
	    }
	}
	/* to PCM Direct */
	else{
	    if(chip->cur_st.cstream == FORZA_AUDIO_PCM)	FORZA_INFO_LOG("MODE:DSD_ReMaster.PCM->Direct.PCM\n");
	    if(chip->cur_st.cstream == FORZA_AUDIO_DSD) FORZA_INFO_LOG("MODE:Direct.DSD->Direct.PCM\n");
	    if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.PCM\n");
	    chip->cur_st.s_pcm = DEVICE_ENABLE;
	    chip->dac_ops->output(chip, FORZA_DIS);
	    wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	    chip->dac_ops->reset(chip, FORZA_RESET_ENA);
	    chip->dac_ops->reset(chip, FORZA_RESET_DIS);
	    if(Is_same_OSC(chip,SRate) == FORZA_OK){
     	        pcm_fs_set(chip, SRate, BWith);
                wait_event_timeout(wq, false, 1);
	    }else{
	        pcm_fs_set(chip, SRate, BWith);
                wait_event_timeout(wq, false, 3);
	        FORZA_DEBUG_LOG("Waiting the stable...\n");
	    }
	    chip->dac_ops->set_default(chip, 'p');
	}
    }
    /*****************
     * from PCM Direct & Direct DSD
     */
    else{  
        /* to DSD Remaster(settings) */
        if(chip->pcm_direct == DEVICE_DISABLE){
	    chip->cur_st.s_pcm = DEVICE_DISABLE;
            if(chip->cur_st.cstream == FORZA_AUDIO_PCM) FORZA_INFO_LOG("MODE:Direct.PCM->DSD_ReMaster.PCM\n");
            if(chip->cur_st.cstream == FORZA_AUDIO_DSD) FORZA_INFO_LOG("MODE:Direct.DSD->DSD_ReMaster.PCM\n");
            if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->DSD_ReMaster.PCM\n");
	    chip->dac_ops->output(chip, FORZA_DIS);
	    wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	    chip->dac_ops->reset(chip, FORZA_RESET_ENA);
	    chip->dac_ops->reset(chip, FORZA_RESET_DIS);
	    if(Is_same_OSC(chip,SRate) == FORZA_OK){
	        pcm_fs_set(chip, SRate, BWith);
	        wait_event_timeout(wq, false, 1);
	    }else{
	        pcm_fs_set(chip, SRate, BWith);
	        wait_event_timeout(wq, false, 3);
	        FORZA_DEBUG_LOG("Waiting the stable...\n");
	    }
	    chip->dac_ops->set_default(chip, 'd');
	}
        /* to PCM Direct */
         else{  
	    chip->cur_st.s_pcm = DEVICE_ENABLE;
	     if(chip->cur_st.cstream == FORZA_AUDIO_PCM) FORZA_INFO_LOG("MODE:Direct.PCM->Direct.PCM\n");
	     if(chip->cur_st.cstream == FORZA_AUDIO_DSD){ FORZA_INFO_LOG("MODE:Direct.DSD->Direct.PCM\n");
	       chip->dac_ops->output(chip, FORZA_DIS);
	       wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	     }
	     if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.PCM\n");
	     if(Is_same_OSC(chip,SRate) == FORZA_OK){
	         pcm_fs_set(chip, SRate, BWith);
	         wait_event_timeout(wq, false, 2);
	     }else{
	       chip->dac_ops->output(chip, FORZA_DIS);
	       wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	       pcm_fs_set(chip, SRate, BWith);
	       wait_event_timeout(wq, false, 3);
	       FORZA_DEBUG_LOG("Waiting the stable...\n");
	     }
	    chip->dac_ops->set_default(chip, 'p');
	 }
    }
    wait_event_timeout(wq, false,  LAST_WAITTIME_FOR_RELAY);
  }// Spiritoso
  else{
    FORZA_ERROR_LOG("%s",__FUNCTION__);
  }// Unknown
  chip->cur_st.cstream = FORZA_AUDIO_PCM;
  chip->cur_st.crate   = SRate;
  return;
}


/*
 * DSD
 * SamplingRate
 *
 */
static void forza_audio_set_dsd(struct snd_forza *chip, unsigned int SRate )
{

  if((chip->cur_st.cstream == FORZA_AUDIO_DSD) && (chip->cur_st.crate == SRate) && (chip->dac_mode == FORZA_AUDIO_DSD)){
    FORZA_INFO_LOG("MODE:Direct.DSD->Setup...pass\n");
    return;
  }

  if(chip->fmode == 'a'){
      if(chip->cur_st.cstream == FORZA_AUDIO_PCM)     FORZA_INFO_LOG("MODE:Direct.PCM->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_DSD)     FORZA_INFO_LOG("MODE:Direct.DSD->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.DSD\n");
      chip->dac_ops->output(chip, FORZA_DIS);
      wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_TRMUTE); 
      chip->dac_ops->reset(chip, FORZA_RESET_ENA);
      chip->dac_ops->reset(chip, FORZA_RESET_DIS);
      dsd_fs_set(chip, SRate);
      forza_osc_wait(chip);     // >>>OSC PLL Lock wait
      chip->dac_ops->set_default(chip, 'd');
      wait_event_timeout(wq, false, LAST_WAITTIME_FOR_TRMUTE);
  }else if(chip->fmode == 's'){
    if(chip->pcm_direct == DEVICE_ENABLE){  // PCM Direct Mode
      if(chip->cur_st.cstream == FORZA_AUDIO_PCM)     FORZA_INFO_LOG("MODE:Direct.PCM->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_DSD)     FORZA_INFO_LOG("MODE:Direct.DSD->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.DSD\n");
      chip->dac_ops->output(chip, FORZA_DIS);
      wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
      chip->dac_ops->reset(chip, FORZA_RESET_ENA);
      chip->dac_ops->reset(chip, FORZA_RESET_DIS);
      dsd_fs_set(chip, SRate);
      wait_event_timeout(wq, false, 1);
      chip->dac_ops->set_default(chip, 'd');

    }else{   // DSD Updampling
      if(chip->cur_st.cstream == FORZA_AUDIO_PCM)     FORZA_INFO_LOG("MODE:DSD_ReMaster.PCM->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_DSD)     FORZA_INFO_LOG("MODE:Direct.DSD->Direct.DSD\n");
      if(chip->cur_st.cstream == FORZA_AUDIO_UNKNOWN) FORZA_INFO_LOG("MODE:None->Direct.DSD\n");
      if(Is_same_OSC(chip,SRate) == FORZA_OK){
	dsd_fs_set(chip, SRate);
      }else{
	chip->dac_ops->output(chip, FORZA_DIS);
	wait_event_timeout(wq, false, FIRST_WAITTIME_FOR_RELAY);
	dsd_fs_set(chip, SRate);
      }
    }
    wait_event_timeout(wq, false,  LAST_WAITTIME_FOR_RELAY);    
  }else{
    FORZA_ERROR_LOG("%s",__FUNCTION__);
  }
  chip->cur_st.cstream = FORZA_AUDIO_DSD;
  chip->cur_st.crate   = SRate;
  return;
}



void timeout_handler(unsigned long data)
{
  struct timer_list *timer = data;
  timer->expires = jiffies + 3*HZ;
  FORZA_DEBUG_LOG("INTERRUPT:\n");
  kfree(timer);
  wake_up_interruptible(&wq);
  FORZA_DEBUG_LOG("WAKEUP:\n");
}

static void forza_audio_delay(struct snd_forza *chip, unsigned char time)
{
  msleep(2);
}

static void forza_debug_test(struct snd_forza *chip, struct ioctl_struct *p){
  struct timer_list *timer;
  FORZA_DEBUG_LOG("AUDIO_DEBUG_TEST:\n");
  timer= kmalloc(GFP_KERNEL, sizeof(struct timer_list));
  init_timer(timer);
  timer->function =timeout_handler;
  timer->data = timer;
  timer->expires = jiffies + 3*HZ;
  add_timer(timer);
  wait_event_interruptible(wq, false);
}

static void forza_debug_led(struct snd_forza *chip, unsigned char mode){
  unsigned int reg;
  reg = UMODE_AUDIO_REG_R(chip) & ~0x00000030;
  switch(mode){
  case FORZA_DEBUG_LED_FIFO:
    reg |= 0x00;
    FORZA_INFO_LOG("AUDIO_DEBUG_FIFO\n");      break;
  case FORZA_DEBUG_LED_VU:
    reg |= 0x01;
    FORZA_INFO_LOG("AUDIO_DEBUG_VU Meter\n");  break;
  case FORZA_DEBUG_LED_DEMO:
    reg |= 0x10;
    FORZA_INFO_LOG("AUDIO_DEBUG_DEMO\n");      break;
  case FORZA_DEBUG_LED_CPU:
    reg |= 0x11;
    FORZA_INFO_LOG("AUDIO_DEBUG_CPU\n");       break;
  }
  UMODE_AUDIO_REG_W(chip, reg);
}

static struct forza_hw_audio audio_ops __devinitdata = {
  .init        = forza_audio_init,
  .reset       = forza_audio_reset,
  .mute        = forza_audio_mute,
  .lmute      = forza_audio_low_mute,
  .formatter   = forza_audio_formatter,
  .set_default = forza_audio_default_set,
  .set_fader_val = forza_audio_fader_set_val,
  .set_fader   = forza_audio_fader_sel,     
  .set_pcm     = forza_audio_set_pcm,
  .set_dsd     = forza_audio_set_dsd,
  .delay       = forza_audio_delay,
  .debug_test  = forza_debug_test,
  .debug_led   = forza_debug_led,
  .delete      = forza_audio_delete,
  .att         = forza_audio_att,
};

#endif


int forza_audio_create(struct snd_forza *chip)
{
  if(FORZA_AUDIO == DEVICE_ENABLE){
    chip->audio_ops = &audio_ops;
    FORZA_DEBUG_LOG(" > AUDIO_OPS\n");
  }
  return 0;
}

int forza_audio_remove(struct snd_forza *chip)
{
  if(FORZA_AUDIO == DEVICE_ENABLE){
    chip->audio_ops->delete(chip);
  }
  return 0;

}


