/*
 *  forza_core.c is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/module.h>  
#include <linux/kernel.h>  
#include <linux/init.h>
#include <asm/io.h>  
#include <linux/cdev.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/io.h>
#include <linux/pci.h> 
#include <linux/sched.h>
#include <linux/moduleparam.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <sound/core.h>
#include <sound/control.h>
#include <sound/asoundef.h>
#include <sound/initval.h>
#include <sound/info.h>
#include <asm/delay.h>
#include <asm/cacheflush.h>

#include "forza.h"
#include "forza_lib.h"
#include "forza_audio_controller.h"
#include "forza_pcm.h"
#include "forza_hwlow.h"
#include "dsp_ops.h"

static int forza_major;
static struct class *forza_class;
static struct snd_forza * ex_forza;

static struct snd_forza *snd_forza_device_get(void){
  return ex_forza;
}

static void snd_forza_proc_read(struct snd_info_entry *entry, struct snd_info_buffer *buffer)
{
  struct snd_forza *chip = entry->private_data;
  snd_iprintf(buffer, "FORZA\n\n");
  snd_iprintf(buffer, "Device ID:0x%x\n", chip->device_id);
  snd_iprintf(buffer, "\n");
}

static int __devinit snd_forza_proc_init(struct snd_card *card, struct snd_forza *forza)
{
  struct snd_info_entry *entry;
  if( ! snd_card_proc_new(card, "forza", &entry))
    snd_info_set_text_ops(entry, forza, snd_forza_proc_read);
  return 0;
}

int snd_forza_timer_handler(struct snd_forza *forza){
  forza->cra_ops->dis_pcie_irq(forza, TIMER_IRQ);
  forza->timer_ops->dis_irq(forza);
  forza->timer_ops->stop(forza);
  forza->cra_ops->clr_pcie_irq(forza, TIMER_IRQ);
  forza->cra_ops->sta_pcie_irq(forza);
  snd_forza_playback_interrupt(forza);
  return 0;
}

int snd_forza_dma_handler(struct snd_forza *forza){
  forza->cra_ops->dis_pcie_irq(forza, SCDMA_IRQ);
  forza->dma_ops->reset(forza);
  if(forza->dmfi.times ==0){
    snd_forza_playback_interrupt(forza);
  }else{
    forza->dmfi.times -= 1;
    if(forza->dmfi.msdelay > 0){
      FORZA_DEBUG_LOG("int-d" );
      forza->cra_ops->ena_pcie_irq(forza, TIMER_IRQ);
      forza->timer_ops->set_period(forza,forza->dmfi.msdelay);
      forza->timer_ops->ena_irq(forza);
      forza->timer_ops->start(forza);
    }
  }
  forza->cra_ops->clr_pcie_irq(forza, SCDMA_IRQ);
  forza->cra_ops->ena_pcie_irq(forza, SCDMA_IRQ);
  return 0;
}


#if !(FORZA_IRQ == PCI_IRQ_DISABLE) 
static irqreturn_t snd_forza_interrupt(int irq, void *dev_id){
  struct snd_forza *forza = dev_id;
  unsigned short irq_status;
  static int a;
  while( 1 ){
    spin_lock(&forza->flock);
    irq_status = forza->cra_ops->sta_pcie_irq(forza);
    spin_unlock(&forza->flock);
    if(irq_status == 0)
      break;
    if((irq_status & TIMER_IRQ) == TIMER_IRQ){
      if(UMODE_SW_REG_R(forza) == 0x1){;
	if(a==1){
	  UMODE_DEBUG_REG_W(forza, 0x1);
	  a = 0;
	}else{
	  UMODE_DEBUG_REG_W(forza, 0x0);
	  a=1;
	}
      }
      snd_forza_timer_handler(forza);
      return IRQ_HANDLED;
    }
    if((irq_status & SCDMA_IRQ) == SCDMA_IRQ){
      snd_forza_dma_handler(forza);
      return IRQ_HANDLED;
    }
    FORZA_DEBUG_LOG("\n***** FORZA_IRQ:UNKNOWN *****" );
    
    
  }
  return IRQ_HANDLED;
}
#endif

static int snd_forza_free(struct snd_forza *chip)
{
  int i;
  struct ioctl_struct st;
  st.val = 2;
  chip->dsp_ops->init(chip,&st);
  chip->audio_ops->delete(chip);  
  device_destroy(forza_class, MKDEV(forza_major,0));
  class_destroy(forza_class);
  unregister_chrdev(forza_major, "forza");
  dsp_hw_remove(chip);
  forza_hw_remove(chip);
  for(i = 0; i < FORZA_BAR_NUM; i++ ){
    if (chip->vir_addr_bar[i])
      iounmap(chip->vir_addr_bar[i]);
  }
  pci_release_regions(chip->pci);
  pci_disable_device(chip->pci);
#if !(FORZA_IRQ == PCI_IRQ_DISABLE) 
  free_irq(chip->irq, chip);
#endif

  if(FORZA_DATA_PREOUT == DEVICE_ENABLE){
    kfree(chip->preout);
    FORZA_DEBUG_LOG("PREOUT_FREE\n");
  }
  kfree(chip);
  return 0;
}

static int snd_forza_dev_free(struct snd_device *device)
{
  struct snd_forza *forza = device->device_data;
  PCIE_IRQ_STATUS_W(forza,1);
  FORZA_DEBUG_LOG("PCIE_IRQ_STATUS:%08x\n",PCIE_IRQ_STATUS_R(forza));
  PCIE_IRQ_ENABLE_W(forza,0);
  return snd_forza_free(forza);
}


// IOCTL
static int forza_ioctl(struct file *file,unsigned int cmd, unsigned long arg ){
  int val;
  int err;

  struct ioctl_struct st;
  struct snd_forza *forza = snd_forza_device_get();
  switch(cmd){
  case FORZA_IOCTL_DSET:
    get_user(val, (int *)arg);
    printk("forza_DSET,val(%d)\n",val);
    break;
  case FORZA_IOCTL_API_SET:
    err = copy_from_user(&st,(void *)arg, sizeof(st));
    FORZA_DEBUG_LOG("FORZA_API_SET\n");
    udelay(500);
    switch(st.cmd){
    case FORZA_API_INIT:       break;
    case FORZA_API_RESET:      break;
    case FORZA_API_MUTE:
      if(st.val == FORZA_ENA){
	FORZA_INFO_LOG("MUTE-Assert");
	forza->dac_ops->output(forza, FORZA_DIS);
      }else if(st.val ==FORZA_DIS){
	FORZA_INFO_LOG("MUTE-Release");
	forza->dac_ops->output(forza, FORZA_ENA);
      }
      break;
    case FORZA_API_INPUT:
      if(st.val == FORZA_DAC_PCM){
	FORZA_INFO_LOG("DAC_PCM");
	forza->dac_ops->set_default(forza,'p');
      }else if(st.val ==FORZA_DAC_DSD){
	FORZA_INFO_LOG("DAC_DSD");
	forza->dac_ops->set_default(forza,'d');
      }else if(st.val ==FORZA_DAC_DIR){
	FORZA_INFO_LOG("DAC_DIR");
	forza->dac_ops->set_default(forza,'p');
	forza->dac_ops->output(forza,DEVICE_ENABLE);
      }
      break;
    case FORZA_API_S_PCM:
      if(st.val == FORZA_ENA){
	forza->pcm_direct = DEVICE_ENABLE;
	FORZA_INFO_LOG("PCM->PCM(DIRECT)\n");
      }else if(st.val == FORZA_DIS){
	forza->pcm_direct = DEVICE_DISABLE;
	FORZA_INFO_LOG("PCM->DSD(default)\n");
      }
      break;
    case FORZA_API_HEQ:         forza->dsp_ops->heq(forza,&st);  break;
    case FORZA_API_HEQ_PARAM:   forza->dsp_ops->heq_params(forza,&st);break;
    case FORZA_API_FADE:        forza->audio_ops->set_fader(forza,st.val);break;
    case FORZA_API_FADE_PARAM:  forza->audio_ops->set_fader_val(forza,st.key, st.val);break;
    case FORZA_API_ATT:         forza->audio_ops->att(forza, st.key,st.val);  break;
    case FORZA_API_SMUTE:
      if(st.val == FORZA_ENA){	      forza->audio_ops->lmute(forza,DEVICE_ENABLE,2);
      }else if(st.val == FORZA_DIS){  forza->audio_ops->lmute(forza,DEVICE_DISABLE,2);}
      break;
    case FORZA_API_FORMAT:      forza->dsp_ops->formats(forza, &st); break;
    case FORZA_API_FORMATTER:
      if(st.val == FORZA_ENA){	        forza->audio_ops->formatter(forza,DEVICE_DISABLE);
      }else if(st.val == FORZA_DIS){	forza->audio_ops->formatter(forza,DEVICE_ENABLE);}
      break;
    case FORZA_API_HEQ_CONTROL: 
      FORZA_INFO_LOG("HEQ_IGNORE_SETTING %d\n", st.val);
      forza->heq_control = st.val;                break;
    case FORZA_API_DEBUG_TEST:  
      forza->audio_ops->debug_test(forza,&st);      break;
    case FORZA_API_DEBUG_LED:   
      forza->audio_ops->debug_led(forza,st.val);    break;
    }
    break;
  case FORZA_IOCTL_DAC_SET:
    FORZA_INFO_LOG("FORZA_DAC_>>>>>>>>>\n");
    err = copy_from_user(&st,(void *)arg, sizeof(st));
    switch(st.cmd){
    case FORZA_DAC_INIT:
      FORZA_INFO_LOG("FORZA_DAC_INITT\n");
      forza->dac_ops->init(forza);
      break;
    case FORZA_DAC_RESET:
      if(st.val== FORZA_ENA){
	FORZA_INFO_LOG("FORZA_DAC_RESET_ENA\n");
	forza->dac_ops->reset(forza,FORZA_RESET_ENA);
      }else{
	FORZA_INFO_LOG("FORZA_DAC_RESET_DIS\n");
	forza->dac_ops->reset(forza,FORZA_RESET_DIS);
      }
      break;
    case FORZA_DAC_DEFAULT:
      FORZA_INFO_LOG("FORZA_DAC_SET_DEFAULT\n");
      forza->dac_ops->set_default(forza, forza->fmode);
      udelay(30);
      break;
    case FORZA_DAC_FIR:
      FORZA_INFO_LOG("FORZA_DAC_SET_FIR-%d\n",st.val);
      switch(st.val){
      case 1:
	forza->dsd_fir = PCM1795_DSD_FIR_1;	break;
      case 2:
	forza->dsd_fir = PCM1795_DSD_FIR_2;	break;
      case 3:
	forza->dsd_fir = PCM1795_DSD_FIR_3;	break;
      case 4:
	forza->dsd_fir = PCM1795_DSD_FIR_4;	break;
      default:
	FORZA_ERROR_LOG("RANGE_ERROR:%d\n",st.val);break;
      }	
      break;
    }
    break;
  case FORZA_IOCTL_DSP_SET:
    err = copy_from_user(&st,(void *)arg, sizeof(st));
    if(err <0)
      FORZA_ERROR_LOG("copy\n");
    switch(st.cmd){
    case FORZA_DSP_INIT:
      FORZA_DEBUG_LOG("FORZA_DSP_INIT:%x:%x\n",st.key, st.val);
      forza->dsp_ops->init(forza,&st);       break;
    case FORZA_DSP_RESET:
      FORZA_DEBUG_LOG("FORZA_DSP_RESET:%x:%x\n",st.key, st.val);
      forza->dsp_ops->reset(forza,&st);       break;
    case FORZA_DSP_FWDL:
      FORZA_DEBUG_LOG("FORZA_DSP_FWDL:%x:%x\n",st.key, st.val);
      forza->dsp_ops->fwdl(forza,&st);        break;
    case FORZA_DSP_MUTE:
      FORZA_DEBUG_LOG("FORZA_DSP_MUTE:%x:%x\n",st.key, st.val);
      forza->dsp_ops->mute(forza,&st);        break;
    case FORZA_DSP_FILTER:
      FORZA_DEBUG_LOG("FORZA_DSP_FILTER:%x:%x\n",st.key, st.val);
      forza->dsp_ops->filter(forza,&st);      break;
    case FORZA_DSP_STATUS:
      FORZA_DEBUG_LOG("FORZA_DSP_STATUS:%x:%x\n",st.key, st.val);
      forza->dsp_ops->status(forza,&st);
      FORZA_DEBUG_LOG("FIFO-ST=%x\n",FIFO_STATUS_REG_R(forza)); 
      FORZA_DEBUG_LOG("FIFO-RE=%x\n",FIFO_CONTROL_REG_R(forza)); 
      FIFO_CONTROL_REG_W(forza,1); 
      FORZA_DEBUG_LOG("FIFO-RE=%x\n",FIFO_CONTROL_REG_R(forza)); 
      break;
    case FORZA_DSP_REG_WRITE:
      FORZA_DEBUG_LOG("FORZA_DSP_REGWR:%x:%x\n",st.key, st.val);
      forza->dsp_ops->reg_wr(forza, &st);      break;
    case FORZA_DSP_VERSION:
      FORZA_DEBUG_LOG("FORZA_DSP_VERSION:%x:%x\n",st.key, st.val);
      forza->dsp_ops->version(forza);          break;
    case FORZA_DSP_REMOVE:
      FORZA_DEBUG_LOG("FORZA_DSP_REMOVEL:%x:%x\n",st.key, st.val);
      forza->dsp_ops->remove(forza);           break;
    case FORZA_DSP_FWDL_FLAG:
      FORZA_DEBUG_LOG("FORZA_DSP_FWDL_FLAG:%x:%x\n",st.key, st.val);
      forza->dsp_ops->fwdl_flag(st.val);       break;
    case FORZA_DSP_RECOVER:
      FORZA_DEBUG_LOG("FORZA_DSP_RECOVER:%x:%x\n",st.key, st.val);
      forza->dsp_ops->recover(forza,&st);      break;
    case FORZA_DSP_RELOAD:
      FORZA_DEBUG_LOG("FORZA_DSP_RELOAD_FIRM:%x:%x\n",st.key, st.val);
      forza->dsp_ops->reload(forza,&st);      break;
    }
    break;
  default:
    printk("forza_DEFAULT\n");
    break;
  };

  return 0;

}


static int forza_open(struct inode *inode, struct file *file){
  return 0;
}
static int forza_close(struct inode *inode, struct file *file){
  return 0;
}

static struct file_operations forza_fops = {
  .owner = THIS_MODULE,
  .unlocked_ioctl   = forza_ioctl,
  .open    = forza_open,
  .release = forza_close,
  
};



int __devinit snd_forza_create(struct snd_card *card, struct pci_dev *pci,
			       struct snd_forza **rchip, char mode)
{
  int err;
  struct snd_forza *forza;
  struct device * dev_class;
  struct ioctl_struct st;

  static struct snd_device_ops ops = {
    .dev_free = snd_forza_dev_free,
  };

  err = pci_enable_device(pci);

  if(err < 0){
    FORZA_ERROR_LOG("Can't enable FORZA_PCI");
    printk(KERN_ALERT "\n");
    return -EIO;
  }

  err = pci_set_dma_mask(pci, DMA_BIT_MASK(FORZA_DMA_MASK_BIT));
  if(err < 0){
    FORZA_ERROR_LOG("Can't set %d BIT DMA MASK",FORZA_DMA_MASK_BIT);
    pci_disable_device(pci);
    return -ENXIO;
  }
  err = pci_set_consistent_dma_mask(pci, DMA_BIT_MASK(FORZA_DMA_MASK_BIT));
  if(err < 0){
    FORZA_ERROR_LOG("Can't set %d BIT DMA consistent MASK",FORZA_DMA_MASK_BIT);
    snd_card_disconnect(card);
    pci_disable_device(pci);
    return -ENXIO;
  }

  forza = kzalloc(sizeof(struct snd_forza),GFP_KERNEL);
  if(forza==NULL){
    FORZA_ERROR_LOG("Can't allocate Kernel mem");
    pci_disable_device(pci);
    return -ENOMEM;
  }


  if(FORZA_DATA_PREOUT == DEVICE_ENABLE){
    forza->preout = kzalloc(FORZA_DATA_PREOUT_SIZE,GFP_KERNEL);
  }
  ex_forza = forza;
  forza->fmode = mode;
  forza->pci  = pci;
  forza->device_id = pci->device;
  forza->rev  =  pci->revision;
  forza->card = card;
  forza->irq  = -1;
  forza->con = 0;

  spin_lock_init(&forza->flock);
  err = pci_request_regions(pci,DEVICE_NAME);
  if(err){
    FORZA_ERROR_LOG("Can't requeset pci regions");
    kfree(forza);
    pci_disable_device(pci);
    return err;
  }

  forza->phy_addr_bar[0] = (unsigned long *)pci_resource_start(pci,0);
  forza->res_size_bar[0] = (unsigned long *)pci_resource_len(pci,0);
  forza->vir_addr_bar[0] = pci_iomap(pci,0, FORZA_BAR0_AREA);
  forza->phy_addr_bar[2] = (unsigned long *)pci_resource_start(pci,2);
  forza->res_size_bar[2] = (unsigned long *)pci_resource_len(pci,2);
  forza->vir_addr_bar[2] = ioremap_nocache((unsigned long)forza->phy_addr_bar[2], FORZA_BAR2_AREA);
  forza->phy_addr_bar[3] = (unsigned long *)pci_resource_start(pci,3);
  forza->res_size_bar[3] = (unsigned long *)pci_resource_len(pci,3);
  forza->vir_addr_bar[3] = ioremap_nocache((unsigned long)forza->phy_addr_bar[3], FORZA_BAR3_AREA);

  pci_set_master(pci);
  

#if !(FORZA_IRQ == PCI_IRQ_DISABLE) 
  err = request_irq(pci->irq, snd_forza_interrupt, IRQF_SHARED, DEVICE_NAME, forza);
  if(err)
    FORZA_ERROR_LOG("Can't get IRQ No");
  else
    forza->irq = pci->irq;
#endif

  forza_print_pci_info(forza,pci);
  forza_print_fpga_config(forza);
  forza_hw_create(forza);
  dsp_hw_create(forza);
  forza_audio_create(forza);

  FORZA_INFO_LOG("FPGA_VERSION     ...0x%x\n",UMODE_VERSION_REG_R(forza));

  forza->cra_ops->init(forza);

#if !(FORZA_IRQ == PCI_IRQ_DISABLE) 
  PCIE_IRQ_ENABLE_W(forza, 0xffff);
  AVMM_IRQ_ENABLE_W(forza, 0xffff);
  FORZA_DEBUG_LOG("PCIE_IRQ_ENABLE_REG_SET:\n > Hex(0x%08x)\n",PCIE_IRQ_ENABLE_R(forza));
#endif

  if(FORZA_TIMER == DEVICE_ENABLE){
    forza->timer_ops->init(forza);
  }

  if(FORZA_AUDIO == DEVICE_ENABLE){
    forza->audio_ops->init(forza, mode);
    forza->audio_ops->set_default(forza, 'p');
  }
  udelay(500);

  forza_major = register_chrdev(0, "forza", &forza_fops);
  if(forza_major < 0)
    FORZA_ERROR_LOG("cant get FORZA_MAJOR\n");
  forza_class = class_create(THIS_MODULE,"forza");
  if(IS_ERR(forza_class))
    FORZA_ERROR_LOG("cant get FORZA_CLASS\n");
  dev_class = device_create(forza_class, NULL, MKDEV(forza_major,0),NULL,"forza");
  if(IS_ERR(dev_class))
    FORZA_ERROR_LOG("cant get DEVICE_CREATE\n");

  err = snd_device_new(card, SNDRV_DEV_LOWLEVEL, forza ,&ops);
  if(err < 0){
    snd_forza_free(forza);
    return err;
  }
  snd_forza_proc_init(card, forza);
  snd_card_set_dev(card, &pci->dev);
  *rchip = forza;
  
  UMODE_DEBUG_REG_W(forza, 0x3333);
  FORZA_DEBUG_LOG("DEBUG_LED_SET=0x3333\n");
  FORZA_DEBUG_LOG("DEBUG_SW_VAL=(0x%04x)\n",UMODE_SW_REG_R(forza));

  st.val = 1;
  forza->dsp_ops->reset(forza,&st);

  if(forza->fmode == 'a'){
    forza->dac_ops->output(forza, FORZA_ENA);
    FORZA_INFO_LOG("OUTPUT ON\n");
  }
 return 0;
}


/*------------------------------------ SND CONTROL ----------------------*/
 static int snd_forza_dsd_att_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo){
   return 0;
 }

 static int snd_forza_dsd_att_get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
 {
   return 0;
 }

 static int snd_forza_dsd_att_set(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
 {
   return 0;
 }

 static struct snd_kcontrol_new snd_forza_audio_att __devinitdata = {
     .iface = SNDRV_CTL_ELEM_IFACE_MIXER,
     .access= SNDRV_CTL_ELEM_ACCESS_READWRITE,
     .name  = "Forza_Att",
     .info  = snd_forza_dsd_att_info,
     .get   = snd_forza_dsd_att_get,
     .put   = snd_forza_dsd_att_set,
     .private_value =0,
     .count = 1,
 };

 static int snd_forza_pro_audio_ctrl_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo){
   return 0;
 }

 static int snd_forza_get_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
 {
   return 0;
 }

 static int snd_forza_put_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
 {
   return 0;
 }

 static struct snd_kcontrol_new snd_forza_audio_mode __devinitdata = {
     .iface = SNDRV_CTL_ELEM_IFACE_MIXER,
     .access= SNDRV_CTL_ELEM_ACCESS_READWRITE,
     .name  = "Forza_Core",
     .info  = snd_forza_pro_audio_ctrl_info,
     .get   = snd_forza_get_info,
     .put   = snd_forza_put_info,
     .private_value =0,
     .count = 1,
 };

int __devinit snd_forza_fpga_ctrl(struct snd_forza *chip){
  struct snd_card *card = chip->card;
  int err;
  struct snd_kcontrol_new ctrl;
  ctrl= snd_forza_audio_mode;
  err = snd_ctl_add(card, snd_ctl_new1(&ctrl, chip));
  if(err < 0)
    return err;
  ctrl= snd_forza_audio_att;
  err = snd_ctl_add(card, snd_ctl_new1(&ctrl, chip));
  if(err < 0)
    return err;
  

   return 0;
 }
