/*
 *  forza_defs.h is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef __FORZA_DEFS_H
#define __FORZA_DEFS_H
#include <linux/ioctl.h>

#include "export/forza_if.h"
/* 
 * FORZA_DEVICE_DEFINITION 
 */
#define FORZA_VER   "00.77"
#define DRV_NAME    "Forza"
#define DEVICE_NAME "Forza_FPGA"

#define DEVICE_DISABLE   0
#define DEVICE_ENABLE    1
#define DEVICE_UNKNOWN   2

#define FORZA_BAR_NUM    5
#define FORZA_BAR0_AREA     (0x0003ffff)
#define FORZA_BAR2_AREA     (0x00014fff)
#define FORZA_BAR3_AREA     (0x00003fff)
#define FORZA_DMA_MASK_BIT  32  
#define CRA_DYNAMIC_ACCESS 1
#define CRA_FIXED_ACCESS   2
#define FORZA_PCIE_MASTER_ACCESS_MODE CRA_DYNAMIC_ACCESS

#define PCI_IRQ_DISABLE 0
#define PCI_REGACY_IRQ  1
#define PCI_MSI_IRQ     2
#define FORZA_IRQ       PCI_REGACY_IRQ
#define IRQ_LOOP_MAX    0  //  0:infinity Other:limitation

#define FORZA_512KB         (1024*64*8) 
#define FORZA_256KB         (1024*64*4) 
#define FORZA_128KB         (1024*64*2) 
#define FORZA_64KB          (1024*64*1) 
      
#define FORZA_DMA_BUFF_SIZE  FORZA_512KB 
#define FORZA_TXS_AREA_MASK  (0xffff0000)

#define FORZA_MEM_PCI_CONSISTENT

#define FORZA_DF_SGDMA       1 
#define FORZA_DF_SCDMA       2 
#define FORZA_DMA            FORZA_DF_SCDMA
#define FORZA_ONCHIP_MEM_BUF 1
#define FORZA_FIFO_MEM_BUF   2
#define DMA_DESTINATION      FORZA_FIFO_MEM_BUF

#define FORZA_FIFO_48K_SIZE   (8192)
#define FORZA_FIFO_96K_SIZE   (16384)
#define FORZA_FIFO_192K_SIZE  (32768)
#define FORZA_FIFO_FIXED_SIZE (32768)

#define FORZA_HW_FIFO_PCM_CHARGE_DELAY TIMEOUT_REG_3ms;
#define FORZA_HW_FIFO_DSD_CHARGE_DELAY TIMEOUT_REG_3ms;
#define PLAYBACK_TRIGGER_DELAY 3

#define FORZA_TIMER DEVICE_ENABLE

#define FORZA_DATA_PREOUT      DEVICE_DISABLE
#define FORZA_DATA_PREOUT_SIZE 4096*12 

#define FORZA_AUDIO DEVICE_ENABLE

#define DAC_PCM1795         1
#define FORZA_DAC           DAC_PCM1795
#define FORZA_AUDIO_UNKNOWN 0
#define FORZA_AUDIO_PCM     1
#define FORZA_AUDIO_DSD     2

//#define FORZA_DSP DEVICE_ENABLE
#define FORZA_DSP           DEVICE_DISABLE
#define ALLEGRO_VERSION     0
#define ALLEGRO_DEFALUT_FIR PCM1795_DSD_FIR_2
#define SPIRITOSO_ET_FIR    PCM1795_DSD_FIR_1
#define SPIRITOSO_DEFALUT_FIR PCM1795_DSD_FIR_2
#define SPIRITOSO_VERSION   0x5


#endif
