/*
 *  forza_hwlow.c is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
#include <asm/delay.h>

#include "forza.h"
#include "forza_lib.h"

struct forza_sgdma {
  unsigned int flag;
};

static void scdma_init(struct snd_forza *chip){
  FORZA_DEBUG_LOG("SCDMA_INIT:\n");
}

static void scdma_print(struct snd_forza *chip){
    FORZA_DEBUG_LOG("SCDMA_SETUP_INFO:\n STATUS (0x%08x)\n READ   (0x%08x)\n WRITE  (0x%08x)\n LENGTH (0x%08x)\n CONTROL(0x%08x)\n",
		  SCDMA_STATUS_REG_R(chip),
		  SCDMA_READ_ADDR_R(chip),
		  SCDMA_WRITE_ADDR_R(chip),
		  SCDMA_LENGTH_R(chip),
		  SCDMA_CONTROL_REG_R(chip));
}

static void scdma_reset(struct snd_forza *chip){
  SCDMA_CONTROL_REG_W(chip, SCDMA_CTL_REST_BIT);
  SCDMA_CONTROL_REG_W(chip, SCDMA_CTL_REST_BIT);
  SCDMA_CONTROL_REG_W(chip, 
		      SCDMA_CTL_32B_BIT | SCDMA_CTL_LEEN_BIT ) ;
}

static void scdma_start(struct snd_forza *chip){
  SCDMA_CONTROL_REG_W(chip, (SCDMA_CONTROL_REG_R(chip) | SCDMA_CTL_GO_BIT));
}

static void scdma_stop(struct snd_forza *chip){
  FORZA_DEBUG_LOG("scdma_stop:\n");
  SCDMA_CONTROL_REG_W(chip, (SCDMA_CONTROL_REG_R(chip) | ~SCDMA_CTL_GO_BIT));
}

static void scdma_setup(struct snd_forza *chip, unsigned int waddr, unsigned int raddr, unsigned int len){
  unsigned int base;
  SCDMA_READ_ADDR_W(chip,  raddr);               
  if(DMA_DESTINATION == FORZA_ONCHIP_MEM_BUF){
    base = FORZA_ONCHIP_BASE;
  }else if(DMA_DESTINATION == FORZA_FIFO_MEM_BUF){
    base = FORZA_FIFO_BASE;
  }else{
    base = 0;
  }
  SCDMA_WRITE_ADDR_W(chip, waddr + base);        
  SCDMA_LENGTH_W(chip, len);                     
  SCDMA_CONTROL_REG_W(chip, 
		      SCDMA_CTL_64B_BIT | SCDMA_CTL_LEEN_BIT | 
		      SCDMA_CTL_IRQ_EN_BIT); 
}

static void scdma_update(struct snd_forza *chip){
  FORZA_DEBUG_LOG("scdma_update:\n");
}

static unsigned int scdma_read_pointer(struct snd_forza *chip){
  return SCDMA_READ_ADDR_R(chip);            
}

static void scdma_dis_irq(struct snd_forza *chip){
}

static void scdma_ena_irq(struct snd_forza *chip){
}

static struct forza_hw_dma scdma_ops __devinitdata = {
  //  .owner = THIS_MODULE,
  .init    = scdma_init,
  .print   = scdma_print,
  .reset   = scdma_reset,
  .start   = scdma_start,
  .stop    = scdma_stop,
  .setup   = scdma_setup,
  .dis_irq = scdma_dis_irq,
  .ena_irq = scdma_ena_irq,
  .update  = scdma_update,
  .rdptr   = scdma_read_pointer, 
};

/* 
 * SCATTER GATHER DMA operators
 */

#define sgdma_descriptor_packed __attribute__ ((packed, aligned(STANDARD_DESCRIPTOR_SIZE)))

typedef struct {
  u32 *read_address;
  u32 *write_address;
  u32 transfer_length;
  u32 control;
}sgdma_descriptor_packed sgdma_std_descriptor;


static void sgdma_init(struct snd_forza *chip){
}

static void sgdma_print(struct snd_forza *chip){
}


static void sgdma_reset(struct snd_forza *chip){
  FORZA_DEBUG_LOG("sgdma_reset:\n");
  CSR_CONTROL_W(chip, CSR_RESET_MASK);  
}

static void sgdma_start(struct snd_forza *chip){
  unsigned int temporary_control;
  FORZA_DEBUG_LOG("sgdma_start:\n");

  temporary_control = CSR_CONTROL_R(chip) & (CSR_STOP_MASK ^ 0xFFFFFFFF);  
  CSR_CONTROL_W(chip, temporary_control);
}

static void sgdma_stop(struct snd_forza *chip){
  unsigned int temporary_control;
  FORZA_DEBUG_LOG("sgdma_stop:\n");
  temporary_control = CSR_CONTROL_R(chip) & (CSR_STOP_DESCRIPTORS_MASK ^ 0xFFFFFFFF);
  CSR_CONTROL_W(chip, temporary_control);
}

static void sgdma_setup(struct snd_forza *chip, unsigned int waddr, unsigned int raddr, unsigned int len){
  static sgdma_std_descriptor descriptor;
  FORZA_DEBUG_LOG("scdma_setup:\n");
  descriptor.read_address = (u32 *)raddr;
  printk(KERN_ALERT "SGDMA_SRC(%x)\n" ,(int)descriptor.read_address );
  descriptor.write_address = 0x00000;
  descriptor.transfer_length = 0x20;
  descriptor.control = (DESCRIPTOR_CONTROL_EARLY_DONE_ENABLE_OFFSET |DESCRIPTOR_CONTROL_GO_MASK);
  SGDMA_DESC_READ_ADDR_W(chip,(u32)descriptor.read_address);
  SGDMA_DESC_WRITE_ADDR_W(chip,(u32)descriptor.write_address);
  SGDMA_DESC_LENGTH_W(chip,(u32)descriptor.transfer_length);
  SGDMA_DESC_CONTROL_STD_W(chip,(u32)descriptor.control);
}

static void sgdma_update(struct snd_forza *chip){
  FORZA_DEBUG_LOG("scdma_update:\n");
}

static struct forza_hw_dma sgdma_ops __devinitdata = {
  //  .owner = THIS_MODULE,
  .init    = sgdma_init,
  .print   = sgdma_print,
  .reset   = sgdma_reset,
  .start   = sgdma_start,
  .stop    = sgdma_stop,
  .setup   = sgdma_setup,
  .update  = sgdma_update,
};

static void cra_init(struct snd_forza *chip)
{
  CRA_TRMAPH0_W(chip, 0x0);        
  CRA_TRMAPL0_W(chip, 0xFFFFFFFC); 
  CRA_TRMAPH1_W(chip, 0x0);        
  CRA_TRMAPL1_W(chip, 0xFFFFFFFC); 
  chip->tx_cra_reg_mask =(int)CRA_TRMAPL0_R(chip);
}


static unsigned int txs_register(struct snd_forza *chip, unsigned int phy){
#if (FORZA_PCIE_MASTER_ACCESS_MODE  == CRA_DYNAMIC_ACCESS)
  chip->tx_cra_reg_base = chip->tx_cra_reg_mask & phy;
  CRA_TRMAPL0_W(chip, chip->tx_cra_reg_base); 
  CRA_TRMAPL1_W(chip, chip->tx_cra_reg_base); 
  chip->txs_dma_reg_base = (~CRA_TRMAPL0_R(chip) & phy); 
#else 
  CRA_TRMAPL0_W(chip, phy);
  CRA_TRMAPL1_W(chip, phy);
  chip->tx_cra_reg_mask = (unsigned int*)CRA_TRMAPL0_R(chip);
  chip->txs_dma_reg_base = 0;
#endif 
  return 0;
}

static unsigned int irq_pcie_register(struct snd_forza *chip, unsigned int BIT){
  unsigned int cur, reg;
  cur = PCIE_IRQ_ENABLE_R(chip);
  reg = cur |BIT;
  PCIE_IRQ_ENABLE_W(chip, reg);
  return reg;
}

static unsigned int irq_avmm_register(struct snd_forza *chip, unsigned int BIT){
  unsigned int cur, reg;
  cur = AVMM_IRQ_ENABLE_R(chip);
  reg = cur |BIT;
  FORZA_DEBUG_LOG("IRQ_AVMM_ENA:(0x%08x)->(0x%08x)\n",cur, reg);
  AVMM_IRQ_ENABLE_W(chip, reg);
  return reg;
}

static unsigned int irq_pcie_remove(struct snd_forza *chip, unsigned int BIT){
  unsigned int cur,reg;
  cur = PCIE_IRQ_ENABLE_R(chip);
  reg = cur & ~BIT;
  PCIE_IRQ_ENABLE_W(chip, reg);
  return reg;
}

static unsigned int irq_avmm_remove(struct snd_forza *chip, unsigned int BIT){
  unsigned int cur, reg;
  cur = AVMM_IRQ_ENABLE_R(chip);
  reg = cur & ~BIT;
  FORZA_DEBUG_LOG("IRQ_AVMM_DIS:(0x%08x)->(0x%08x)\n",cur, reg);
  AVMM_IRQ_ENABLE_W(chip, reg);
  return reg;
}

static unsigned int irq_pcie_status(struct snd_forza *chip){
  unsigned int cur = PCIE_IRQ_STATUS_R(chip);
  return cur;
}

static unsigned int irq_avmm_status(struct snd_forza *chip){
  unsigned int cur = AVMM_IRQ_STATUS_R(chip);
  FORZA_DEBUG_LOG("IRQ_AVMM_STATUS:(0x%08x)\n",cur);
  return cur;
}

static void irq_pcie_clear(struct snd_forza *chip, unsigned int BIT){
  PCIE_IRQ_STATUS_W(chip, BIT);
}

static void irq_avmm_clear(struct snd_forza *chip, unsigned int BIT){
  AVMM_IRQ_STATUS_W(chip, BIT);
}


static struct forza_hw_cra cra_ops __devinitdata = {
  .init    = cra_init,
  .set_txa = txs_register,
  .ena_pcie_irq = irq_pcie_register,
  .ena_avmm_irq = irq_avmm_register,
  .dis_pcie_irq = irq_pcie_remove,
  .dis_avmm_irq = irq_avmm_remove,
  .sta_pcie_irq = irq_pcie_status,
  .sta_avmm_irq = irq_avmm_status,
  .clr_pcie_irq = irq_pcie_clear,
  .clr_avmm_irq = irq_avmm_clear,
};


static void timer_init(struct snd_forza *chip){
  FORZA_DEBUG_LOG("PERIOD_TIMER_INIT:\n");
  TIMER_STATUS_REG_W(chip, 0);
  TIMER_CONTROL_REG_W(chip, 0);
  TIMER_PERIODL_REG_W(chip, 0x1);
  TIMER_PERIODH_REG_W(chip, 0x0);
}

static void timer_set_period(struct snd_forza *chip, unsigned int period){
  TIMER_PERIODL_REG_W(chip, (0xffff & period));
  TIMER_PERIODH_REG_W(chip, (period >> 16));
}

static void timer_start(struct snd_forza *chip){
  TIMER_CONTROL_REG_W(chip, ((TIMER_CONTROL_REG_R(chip) |TIMER_START_BIT |TIMER_CONTINU_BIT ) &~TIMER_STOP_BIT));
}

static void timer_stop(struct snd_forza *chip){
  TIMER_CONTROL_REG_W(chip, ((TIMER_CONTROL_REG_R(chip) |TIMER_STOP_BIT) &~TIMER_START_BIT));
}

static void timer_clear(struct snd_forza *chip){
  TIMER_CONTROL_REG_W(chip, TIMER_STOP_BIT);
  TIMER_PERIODL_REG_W(chip, 0x1);
  TIMER_PERIODH_REG_W(chip, 0x0);
  TIMER_STATUS_REG_W(chip,  0);
}

static void timer_status(struct snd_forza *chip){
  FORZA_DEBUG_LOG("TIMER_STATUS :(%x)\n",TIMER_STATUS_REG_R(chip));
  FORZA_DEBUG_LOG("TIMER_CONTROL:(%x)\n",TIMER_CONTROL_REG_R(chip));
  FORZA_DEBUG_LOG("TIMER_PERIODL:(%x)\n",TIMER_PERIODL_REG_R(chip));
  FORZA_DEBUG_LOG("TIMER_PERIODH:(%x)\n",TIMER_PERIODH_REG_R(chip));
}

static void timer_dis_irq(struct snd_forza *chip){
  TIMER_CONTROL_REG_W(chip, (TIMER_CONTROL_REG_R(chip) & ~TIMER_IRQ_EN_BIT ));
  TIMER_STATUS_REG_W(chip,  0);
}

static void timer_ena_irq(struct snd_forza *chip){
  TIMER_STATUS_REG_W(chip,  0);
  TIMER_CONTROL_REG_W(chip, (TIMER_CONTROL_REG_R(chip) | TIMER_IRQ_EN_BIT));
}

static void timer_remove(struct snd_forza *chip){
  FORZA_DEBUG_LOG("PERIOD_TIMER_REMOVE:\n");
  TIMER_CONTROL_REG_W(chip, 0x0);
  TIMER_PERIODL_REG_W(chip, 0x1);
  TIMER_STATUS_REG_W(chip,  0);
  TIMER_PERIODH_REG_W(chip, 0x0);
}


static struct forza_hw_timer timer_ops __devinitdata = {
  .init    = timer_init,
  .set_period = timer_set_period,
  .start   = timer_start,
  .stop    = timer_stop,
  .clear   = timer_clear,
  .status  = timer_status,
  .dis_irq = timer_dis_irq,
  .ena_irq = timer_ena_irq,
  .remove  = timer_remove,
};


#if (FORZA_DAC == DAC_PCM1795)
static void pcm1795_init(struct snd_forza *chip){
  SPI_DAC_INT_REG_W(chip, 0x000);
  if(chip->fmode == 'a'){
     chip->dsd_fir = ALLEGRO_DEFALUT_FIR;
     FORZA_DEBUG_LOG("PCM1795_DAC_INIT:wFIR(%x)\n",chip->dsd_fir);
  }else if(chip->fmode == 's'){
    if(chip->board_ver == SPIRITOSO_ET){
      chip->dsd_fir = SPIRITOSO_ET_FIR;
    }else{
      chip->dsd_fir = SPIRITOSO_DEFALUT_FIR;
    }
     FORZA_DEBUG_LOG("PCM1795_DAC_INIT:wFIR(%x)\n",chip->dsd_fir);
  }
}

static void pcm1795_reset(struct snd_forza *chip, int reset){
  if(reset == FORZA_RESET_DIS){
    FORZA_DEBUG_LOG("PCM1795_DAC_RESET_DIS:\n");
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) | UMODE_DAC_RESET_DIS));
    udelay(300);
  }else if(reset == FORZA_RESET_ENA){
    FORZA_DEBUG_LOG("PCM1795_DAC_RESET_ENA:\n");
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) & ~UMODE_DAC_RESET_DIS)); 
    udelay(300);
  }else{
    FORZA_DEBUG_LOG("PCM1795_DAC_RESET_DIS-ENA-DIS:\n");
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) | UMODE_DAC_RESET_DIS));
    udelay(100);
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) & ~UMODE_DAC_RESET_DIS)); 
    udelay(300);
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) | UMODE_DAC_RESET_DIS));
    udelay(300);
  }
}

static void pcm1795_mute(struct snd_forza *chip, int mute){
  FORZA_DEBUG_LOG("PCM1795_DAC_RESET:\n");
  if(mute == PCM1795_MUTE_ENA){  }
}

static void pcm1795_set_default(struct snd_forza *chip, unsigned char fmt ){
  udelay(20);
  if(chip->fmode == 'a'){
    if(fmt == 'p'){
      FORZA_DEBUG_LOG("PCM1795_DAC_MODE:Allegro PCM Settings:\n");
      chip->dac_ops->set_data(chip,PCM1795_REG16,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG17,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG18,0x10 ); // for EPP
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG19,0x00 ); 
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG20,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG21,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG22,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG23,0x00 );
      chip->dac_mode = FORZA_AUDIO_PCM;
    }else if(fmt == 'd'){
      FORZA_DEBUG_LOG("PCM1795_DAC_MODE:Allegro DSD Settings:Fir-%d\n",(1+(chip->dsd_fir >> 2)));
      chip->dac_ops->set_data(chip,PCM1795_REG16,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG17,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG18,0x50 | chip->dsd_fir );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG19,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG20,0x20 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG21,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG22,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG23,0x00 );
      chip->dac_mode = FORZA_AUDIO_DSD;
    }
  }else if(chip->fmode == 's'){
    if(fmt == 'p'){
      FORZA_DEBUG_LOG("PCM1795_DAC_MODE:Spiritoso PCM Settings:\n");
      chip->dac_ops->set_data(chip,PCM1795_REG16,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG17,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG18,0x10 ); 
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG19,0x80 );  // 20140513 REV ON
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG20,0x10 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG21,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG22,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG23,0x00 );
      chip->dac_mode = FORZA_AUDIO_PCM;
    }else{
      FORZA_DEBUG_LOG("PCM1795_DAC_MODE:Spiritoso DSD Settings:Fir-%d\n",(1+(chip->dsd_fir >>2)));
      chip->dac_ops->set_data(chip,PCM1795_REG16,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG17,0xFF );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG18,0x50 | chip->dsd_fir );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG19,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG20,0x20 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG21,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG22,0x00 );
      udelay(10);
      chip->dac_ops->set_data(chip,PCM1795_REG23,0x00 );
      chip->dac_mode = FORZA_AUDIO_DSD;
    }
  }else{
    FORZA_DEBUG_LOG("PCM1795_DAC_MODE:Default Error:\n");
  }
  udelay(20);
}

static void pcm1795_set_data(struct snd_forza *chip, unsigned char addr, unsigned char data){
  unsigned short tx_data;
  tx_data = addr << 8;
  tx_data = tx_data | data;
  SPI_DAC_TX_REG_W(chip, tx_data);
}

static unsigned short spi_dac_status(struct snd_forza *chip){
  unsigned short status;
  status = /*0x170 & */SPI_DAC_STATUS_REG_R(chip);
  FORZA_DEBUG_LOG("SPI_DAC_STATUS (0x%03x)\n", status);
  if((status &  SPI_DAC_TOE) == SPI_DAC_TOE)
    FORZA_DEBUG_LOG("SPI_DAC_TX_OVER_WRITE_ERROR \n");
  if((status & SPI_DAC_TMT) == SPI_DAC_TMT)
    FORZA_DEBUG_LOG("SPI_DAC_TX_EMPTY \n");
  if((status & SPI_DAC_TRDY) == SPI_DAC_TRDY)
    FORZA_DEBUG_LOG("SPI_DAC_TX_READY \n");
  if((status & SPI_DAC_ERROR) == SPI_DAC_ERROR)
    FORZA_DEBUG_LOG("SPI_DAC_ERROR \n");
  return status;
}


static void spi_dac_ena_irq(struct snd_forza *chip){
  SPI_DAC_INT_REG_W(chip, 0xfff);
  FORZA_DEBUG_LOG("SPI_DAC_IRQ_REG (0x%3x)\n",SPI_DAC_INT_REG_R(chip));
}

static void spi_dac_dis_irq(struct snd_forza *chip){
  FORZA_DEBUG_LOG("SPI_DAC_IRQ_DISABLE:\n");
}

static void pcm1795_remove(struct snd_forza *chip){
  FORZA_DEBUG_LOG("PCM1795_DAC_REMOVE:\n");
}


static void system_out(struct snd_forza *chip, int output){
  if(FORZA_ENA == output){
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) | UMODE_OUTPUT_MUTE_DIS));
    FORZA_DEBUG_LOG("H/W Mute..ENA(%02x)\n",UMODE_IO_REG_R(chip));
    chip->output = FORZA_ENA;
  }else if(FORZA_DIS == output){
    UMODE_IO_REG_W(chip, (UMODE_IO_REG_R(chip) & ~UMODE_OUTPUT_MUTE_DIS));
    FORZA_DEBUG_LOG("H/W Mute..DIS(%02x)\n",UMODE_IO_REG_R(chip));
    chip->output = FORZA_DIS;
  }else{
  }
}


static struct forza_hw_dac dac_ops __devinitdata = {
  .init        = pcm1795_init,
  .reset       = pcm1795_reset,
  .mute        = pcm1795_mute,
  .output      = system_out,
  .set_default = pcm1795_set_default,
  .set_data    = pcm1795_set_data,
  .status      = spi_dac_status,
  .dis_irq     = spi_dac_dis_irq,
  .ena_irq     = spi_dac_ena_irq,
  .remove      = pcm1795_remove,
};
#endif

/* <----------------------------------DSP ----------------------------------------> */

#if (FORZA_DSP == DEVICE_ENABLE)
static void dsp_spi_init(struct snd_forza *chip){
  FORZA_DEBUG_LOG("DSP_INIT:\n");
  SPI_DSP_INT_REG_W(chip, 0x000);
}

static void dsp_spi_reset(struct snd_forza *chip){
  FORZA_DEBUG_LOG("DSP_SPI_RESET:\n");
}

static void pcm1795_set_data(struct snd_forza *chip, unsigned char addr, unsigned char data){
  unsigned short tx_data;
  tx_data = addr << 8;
  tx_data = tx_data | data;
  FORZA_DEBUG_LOG("PCM1795_DAC_SETDATA:0x%02x\n",tx_data);
  SPI_DAC_TX_REG_W(chip, tx_data);
  SPI_DAC_TX_REG_W(chip, tx_data);
}

static unsigned short spi_dac_status(struct snd_forza *chip){
  unsigned short status;
  status = /*0x170 & */SPI_DAC_STATUS_REG_R(chip);
  FORZA_DEBUG_LOG("SPI_DAC_STATUS (0x%03x)\n", status);
  if((status &  SPI_DAC_TOE) == SPI_DAC_TOE)
    FORZA_DEBUG_LOG("SPI_DAC_TX_OVER_WRITE_ERROR \n");
  if((status & SPI_DAC_TMT) == SPI_DAC_TMT)
    FORZA_DEBUG_LOG("SPI_DAC_TX_EMPTY \n");
  if((status & SPI_DAC_TRDY) == SPI_DAC_TRDY)
    FORZA_DEBUG_LOG("SPI_DAC_TX_READY \n");
  if((status & SPI_DAC_ERROR) == SPI_DAC_ERROR)
    FORZA_DEBUG_LOG("SPI_DAC_ERROR \n");
  return status;
}


static void spi_dac_ena_irq(struct snd_forza *chip){
  SPI_DAC_INT_REG_W(chip, 0xfff);
  FORZA_DEBUG_LOG("SPI_DAC_IRQ_REG (0x%3x)\n",SPI_DAC_INT_REG_R(chip));
}

static void spi_dac_dis_irq(struct snd_forza *chip){
  FORZA_DEBUG_LOG("SPI_DAC_IRQ_DISABLE:\n");
}

static void pcm1795_remove(struct snd_forza *chip){
  FORZA_DEBUG_LOG("PCM1795_DAC_REMOVE:\n");
}

static struct forza_hw_dac dac_ops __devinitdata = {
  .init        = spi_dsp_init,
  .reset_spi   = spi_dsp_reset,
  .reset_dsp   = dsp_reset,
  .spi_mode    = spi_mode_set,  // CONTROL or FWDL
  .w_data      = spi_write_data,
  .r_data      = spi_read_data,
  .spi_status  = spi_status,
  .dis_irq     = spi_dsp_dis_irq,
  .ena_irq     = spi_dsp_ena_irq,
  .remove      = spi_dsp_remove,
};
#endif



/* <---------------------------------- Register/Remove -----------------------------> */

/*
 * Register FPGA H/W
 *
 */
int forza_hw_create(struct snd_forza *chip)
{
  if(chip->fmode == 'a'){
    FORZA_INFO_LOG("REGISTRATION     ...Allegro\n");
    chip->board_ver = ALLEGRO_ET;
  }else if(chip->fmode == 's'){
    FORZA_INFO_LOG("UMODE_REG...0x%x\n",UMODE_IO_REG_R(chip));
    if((UMODE_IO_REG_R(chip) & UMODE_BOARD_DET_EPP) == UMODE_BOARD_DET_EPP){
      chip->board_ver = SPIRITOSO_EPP;
      FORZA_INFO_LOG("REGISTRATION     ...Spilitoso-EPP\n");
    }else{
      chip->board_ver = SPIRITOSO_ET;
      FORZA_INFO_LOG("REGISTRATION     ...Spilitoso-ET\n");
    }
  }else{
    FORZA_DEBUG_LOG("FORZA_HW_UNKNOWN \n");
  }

  chip->cra_ops = &cra_ops;
  FORZA_DEBUG_LOG(" > CRA_OPS\n");

  if(FORZA_DMA == FORZA_DF_SGDMA){
    chip->dma_ops = &sgdma_ops;
    FORZA_DEBUG_LOG(" > SGDMA_OPS\n");
  }else if(FORZA_DMA == FORZA_DF_SCDMA){
    chip->dma_ops  = &scdma_ops;
     FORZA_DEBUG_LOG(" > SCDMA_OPS\n");
  }else{
  }

  if(FORZA_TIMER == DEVICE_ENABLE){
    chip->timer_ops = &timer_ops;
  }    

  if(FORZA_DAC == DAC_PCM1795){
    chip->dac_ops = &dac_ops;
    FORZA_DEBUG_LOG(" > PCM1795_DAC_OPS\n");
  }
  return 0;
}

/*
 * REMOVE FPGA H/W
 *
 */
int forza_hw_remove(struct snd_forza *chip)
{
  FORZA_DEBUG_LOG("FORZA_HW_REGISTRATION_LIST:\n");
  if(FORZA_DAC != DEVICE_DISABLE){
    chip->dac_ops->remove(chip);
  }

  if(FORZA_TIMER == DEVICE_ENABLE){
    chip->timer_ops->remove(chip);
    chip->timer_ops->status(chip);
    chip->timer_ops = NULL;
    chip->cra_ops->clr_pcie_irq(chip, TIMER_IRQ);
    chip->cra_ops->sta_pcie_irq(chip);
    chip->cra_ops->clr_avmm_irq(chip, TIMER_IRQ);
    chip->cra_ops->sta_avmm_irq(chip);
  }    
  return 0;

}
