/*
 *  forza_hwlow.h is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */
#ifndef __SOUND_FORZA_HWLOW_H
#define __SOUND_FORZA_HWLOW_H

#include <linux/types.h>

//***********************[TXs:DMA Master Acces]***********
//
// << Basic IO Access >>
//
#define READ_FORZA_ULONG(p, offsets)    ioread32(p + offsets)
#define WRITE_FORZA_ULONG(p,offsets, x) iowrite32(x, p + offsets)
#define FORZA_IORD_32(p, offsets)       ioread32(p + offsets)
#define FORZA_IOWR_32(p,offsets, x)     iowrite32(x, p + offsets)
#define FORZA_IORD_16(p, offsets)       ioread16(p + offsets)
#define FORZA_IOWR_16(p,offsets, x)     iowrite16(x, p + offsets)
#define FORZA_IORD_8(p, offsets)        ioread8(p + offsets)
#define FORZA_IOWR_8(p,offsets, x)      iowrite8(x, p + offsets)

//***********************[BAR 3]***************************
//
// << TEST IO >>
//
#define TEST_BAR(chip) (chip->vir_addr_bar[3])

// MODE
#define FORZA_SW_MODE_BASE              (0)
#define SW_MODE_ADDR                    (FORZA_SW_MODE_BASE + 0)
#define SW_MODE_R(c)               FORZA_IORD_32(TEST_BAR(c),SW_MODE_ADDR)

// TEST SPI_BASE
#define FORZA_SPI_BASE                  (0x3000)
#define TEST_SPI_READ_ADDR              (FORZA_SPI_BASE + 0x0)
#define TEST_SPI_WRITE_ADDR             (FORZA_SPI_BASE + 0x1)
#define BAR3_SPI(c, x)             FORZA_IOWR_32(TEST_BAR(c),TEST_SPI_WRITE_ADDR,x)

// LED
#define FORZA_GPIO_BASE                 (0x2000)
#define DEBUG_LED_ADDR                  (FORZA_GPIO_BASE + 0x0)
#define BAR3_LED_W(c, x)           FORZA_IOWR_32(TEST_BAR(c),DEBUG_LED_ADDR,x)
#define BAR3_LED_R(c)              FORZA_IORD_32(TEST_BAR(c),DEBUG_LED_ADDR)

// SW
#define FORZA_SW_BASE                   (0x1000)
#define BAR3_SW_R(c)               FORZA_IORD_16(TEST_BAR(c),FORZA_SW_BASE)

//***********************[BAR 0]***************************
//
// << ONCHIP_MEMORY >>
//
#define ONCHIP_BAR(chip) (chip->vir_addr_bar[0])
//#define FORZA_ONCHIP_BASE               (0x20000)
#define FORZA_ONCHIP_BASE               (0x4000)
#define ONCHIP_MEM_R(c,offsets)     FORZA_IORD_32(ONCHIP_BAR(c),FORZA_ONCHIP_BASE + offsets)
#define ONCHIP_MEM_W(c,offsets,x)   FORZA_IOWR_32(ONCHIP_BAR(c),FORZA_ONCHIP_BASE + offsets,x)
#define ONCHIP_MEM_8R(c,offsets)    FORZA_IORD_8(ONCHIP_BAR(c),FORZA_ONCHIP_BASE + offsets)
#define ONCHIP_MEM_8W(c,offsets,x)  FORZA_IOWR_8(ONCHIP_BAR(c),FORZA_ONCHIP_BASE + offsets,x)


//***********************[BAR 2]***************************
// << FIFO_MEMORY >>
//
#define FIFO_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_FIFO_BASE                  (0x8000)
#define FORZA_FIFO_CRA_BASE              (0x13000)
#define FIFO_STATUS_REG                  (FORZA_FIFO_CRA_BASE + 0x0)
#define FIFO_CONTROL_REG                 (FORZA_FIFO_CRA_BASE + 0x2)
#define FIFO_SIZE_REG                    (FORZA_FIFO_CRA_BASE + 0x4)
#define FIFO_SIZE_1024                   0x1
#define FIFO_SIZE_2048                   0x2
#define FIFO_SIZE_3072                   0x3
#define FIFO_SIZE_4096                   0x4
#define FIFO_SIZE_5120                   0x5
#define FIFO_SIZE_6144                   0x6
#define FIFO_SIZE_7168                   0x7
#define FIFO_SIZE_8192                   0x8

#define FIFO_STATUS_REG_R(c)            FORZA_IORD_16(FIFO_BAR(c),FIFO_STATUS_REG)
#define FIFO_CONTROL_REG_W(c,x)          FORZA_IOWR_16(FIFO_BAR(c),FIFO_CONTROL_REG,x)
#define FIFO_CONTROL_REG_R(c)            FORZA_IORD_16(FIFO_BAR(c),FIFO_CONTROL_REG)
#define FIFO_SIZE_REG_W(c,x)            FORZA_IOWR_16(FIFO_BAR(c),FIFO_SIZE_REG,x)
//
// << PCIE CRA >> 0x0000_0000 ~ 0x0000_3fff
//
#define PCIE_CRA_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_PCIE_CRA_BASE             (0)


#define FORZA_PCIE_CRA_TXMAP_REG_BASE   (0x1000)
#define PCIE_IRQ_STATUS_REG              (FORZA_PCIE_CRA_BASE + 0x40)
#define PCIE_IRQ_ENABLE_REG              (FORZA_PCIE_CRA_BASE + 0x50)
#define AVMM_IRQ_STATUS_REG              (FORZA_PCIE_CRA_BASE + 0x3060)
#define AVMM_IRQ_ENABLE_REG              (FORZA_PCIE_CRA_BASE + 0x3070)

#define PCIE_IRQ0_BIT                    (1)
#define PCIE_IRQ1_BIT                    (1<<1)
#define PCIE_IRQ2_BIT                    (1<<2)
#define PCIE_IRQ3_BIT                    (1<<3)
#define PCIE_IRQ4_BIT                    (1<<4)
#define PCIE_IRQ5_BIT                    (1<<5)
#define PCIE_IRQ6_BIT                    (1<<6)
#define PCIE_IRQ7_BIT                    (1<<7)

#define SCDMA_IRQ  PCIE_IRQ0_BIT
#define TIMER_IRQ  PCIE_IRQ4_BIT
#define SPIDAC_IRQ PCIE_IRQ2_BIT
#define SPIDSP_IRQ PCIE_IRQ1_BIT

#define PCIE_IRQ_STATUS_R(c)      FORZA_IORD_32(PCIE_CRA_BAR(c),PCIE_IRQ_STATUS_REG)
#define PCIE_IRQ_STATUS_W(c,x)    FORZA_IOWR_32(PCIE_CRA_BAR(c),PCIE_IRQ_STATUS_REG,x)
#define PCIE_IRQ_ENABLE_R(c)      FORZA_IORD_32(PCIE_CRA_BAR(c),PCIE_IRQ_ENABLE_REG)
#define PCIE_IRQ_ENABLE_W(c,x)    FORZA_IOWR_32(PCIE_CRA_BAR(c),PCIE_IRQ_ENABLE_REG,x)
#define AVMM_IRQ_STATUS_R(c)      FORZA_IORD_32(PCIE_CRA_BAR(c),AVMM_IRQ_STATUS_REG)
#define AVMM_IRQ_STATUS_W(c,x)    FORZA_IOWR_32(PCIE_CRA_BAR(c),AVMM_IRQ_STATUS_REG,x)
#define AVMM_IRQ_ENABLE_R(c)      FORZA_IORD_32(PCIE_CRA_BAR(c),AVMM_IRQ_ENABLE_REG)
#define AVMM_IRQ_ENABLE_W(c,x)    FORZA_IOWR_32(PCIE_CRA_BAR(c),AVMM_IRQ_ENABLE_REG,x)

#define CRA_TRMAPL0_R(c)         FORZA_IORD_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE)
#define CRA_TRMAPL0_W(c,x)       FORZA_IOWR_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE,x)
#define CRA_TRMAPH0_W(c,x)       FORZA_IOWR_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE+0x4,x)

#define CRA_TRMAPL1_R(c)         FORZA_IORD_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE+0x8)
#define CRA_TRMAPL1_W(c,x)       FORZA_IOWR_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE+0x8,x)
#define CRA_TRMAPH1_W(c,x)       FORZA_IOWR_32(PCIE_CRA_BAR(c),FORZA_PCIE_CRA_TXMAP_REG_BASE+0xC,x)


// 
// << Single Command DMA >> 0x0001_0000 ~ 0x0001_003f
//
// PCI_BAR_SETTING
#define SCDMA_BAR(chip) (chip->vir_addr_bar[2])
// BAR_OFFSET_REGISTER
#define FORZA_SCDMA_REG_BASE (0x4000)   // Single Command DMA R/W Register
// BASIC_REGISTER
#define SCDMA_STATUS_REG           (FORZA_SCDMA_REG_BASE + 0)
#define SCDMA_READ_START_ADDR_REG  (FORZA_SCDMA_REG_BASE + 0x4)
#define SCDMA_WRITE_START_ADDR_REG (FORZA_SCDMA_REG_BASE + 0x8)
#define SCDMA_LENGTH_REG           (FORZA_SCDMA_REG_BASE + 0xC)
#define SCDMA_RESERVED_1_REG       (FORZA_SCDMA_REG_BASE + 0x10)
#define SCDMA_RESERVED_2_REG       (FORZA_SCDMA_REG_BASE + 0x14)
#define SCDMA_CONTROL_REG          (FORZA_SCDMA_REG_BASE + 0x18)
// SCDMA_STATUS_BIT
#define SCDMA_ST_DONE_BIT          (1)      // when DMA transaction is COMPLETE
#define SCDMA_ST_BUSY_BIT          (1<<1)   // In Progress
#define SCDMA_ST_REOF_BIT          (1<<2)   // finished READ  transaction by end of packet
#define SCDMA_ST_WEOP_BIT          (1<<3)   // finished WRITE transaction by end of packet
#define SCDMA_ST_LEN_BIT           (1<<4)   // when LENGTH go to Zero
// SCDMA_CONTROL_BIT
#define SCDMA_CTL_8B_BIT           (1)      //  8BIT transfer reqest
#define SCDMA_CTL_16B_BIT          (1<<1)   // 16BIT transfer request 
#define SCDMA_CTL_32B_BIT          (1<<2)   // 32BIT transfer request
#define SCDMA_CTL_GO_BIT           (1<<3)   // DMA GO 
#define SCDMA_CTL_IRQ_EN_BIT       (1<<4)   // enabel INTERRUPT request when status is done.
#define SCDMA_CTL_LEEN_BIT         (1<<7)
#define SCDMA_CTL_RCON_BIT         (1<<8)   // Fixed the Read Address
#define SCDMA_CTL_WCON_BIT         (1<<9)   // Fixed the Write Address
#define SCDMA_CTL_64B_BIT          (1<<10)  // 64BIT transfer request
#define SCDMA_CTL_REST_BIT         (1<<12)  // Reset the DMA (twice write is required)

// DISCRIPTOR_Standard_Access 
#define SCDMA_STATUS_REG_W(c, d)    FORZA_IOWR_32(SCDMA_BAR(c),SCDMA_STATUS_REG, d)
#define SCDMA_STATUS_REG_R(c)       FORZA_IORD_32(SCDMA_BAR(c),SCDMA_STATUS_REG)
#define SCDMA_READ_ADDR_W(c, d)     FORZA_IOWR_32(SCDMA_BAR(c),SCDMA_READ_START_ADDR_REG,d)
#define SCDMA_READ_ADDR_R(c)        FORZA_IORD_32(SCDMA_BAR(c),SCDMA_READ_START_ADDR_REG)
#define SCDMA_WRITE_ADDR_W(c, d)    FORZA_IOWR_32(SCDMA_BAR(c),SCDMA_WRITE_START_ADDR_REG,d)
#define SCDMA_WRITE_ADDR_R(c)       FORZA_IORD_32(SCDMA_BAR(c),SCDMA_WRITE_START_ADDR_REG)
#define SCDMA_LENGTH_W(c, d)        FORZA_IOWR_32(SCDMA_BAR(c),SCDMA_LENGTH_REG,d)
#define SCDMA_LENGTH_R(c)           FORZA_IORD_32(SCDMA_BAR(c),SCDMA_LENGTH_REG)
#define SCDMA_CONTROL_REG_W(c, d)   FORZA_IOWR_32(SCDMA_BAR(c),SCDMA_CONTROL_REG,d) 
#define SCDMA_CONTROL_REG_R(c)      FORZA_IORD_32(SCDMA_BAR(c),SCDMA_CONTROL_REG) 


//
// << ScatterGatherDMA >>
//
#define SGDMA_BAR(chip) (chip->vir_addr_bar[2])

// masks and offsets for the sequence number and programmable burst counts
#define DESCRIPTOR_SEQUENCE_NUMBER_MASK                  (0xFFFF)
#define DESCRIPTOR_SEQUENCE_NUMBER_OFFSET                (0)
#define DESCRIPTOR_READ_BURST_COUNT_MASK                 (0x00FF0000)
#define DESCRIPTOR_READ_BURST_COUNT_OFFSET               (16)
#define DESCRIPTOR_WRITE_BURST_COUNT_MASK                (0xFF000000)
#define DESCRIPTOR_WRITE_BURST_COUNT_OFFSET              (24)
// masks and offsets for the read and write strides
#define DESCRIPTOR_READ_STRIDE_MASK                      (0xFFFF)
#define DESCRIPTOR_READ_STRIDE_OFFSET                    (0)
#define DESCRIPTOR_WRITE_STRIDE_MASK                     (0xFFFF0000)
#define DESCRIPTOR_WRITE_STRIDE_OFFSET                   (16)
// masks and offsets for the bits in the descriptor control field
#define DESCRIPTOR_CONTROL_TRANSMIT_CHANNEL_MASK         (0xFF)
#define DESCRIPTOR_CONTROL_TRANSMIT_CHANNEL_OFFSET       (0)
#define DESCRIPTOR_CONTROL_GENERATE_SOP_MASK             (1<<8)
#define DESCRIPTOR_CONTROL_GENERATE_SOP_OFFSET           (8)
#define DESCRIPTOR_CONTROL_GENERATE_EOP_MASK             (1<<9)
#define DESCRIPTOR_CONTROL_GENERATE_EOP_OFFSET           (9)
#define DESCRIPTOR_CONTROL_PARK_READS_MASK               (1<<10)
#define DESCRIPTOR_CONTROL_PARK_READS_OFFSET             (10)
#define DESCRIPTOR_CONTROL_PARK_WRITES_MASK              (1<<11)
#define DESCRIPTOR_CONTROL_PARK_WRITES_OFFSET            (11)
#define DESCRIPTOR_CONTROL_END_ON_EOP_MASK               (1<<12)
#define DESCRIPTOR_CONTROL_END_ON_EOP_OFFSET             (12)
#define DESCRIPTOR_CONTROL_TRANSFER_COMPLETE_IRQ_MASK    (1<<14)
#define DESCRIPTOR_CONTROL_TRANSFER_COMPLETE_IRQ_OFFSET  (14)
#define DESCRIPTOR_CONTROL_EARLY_TERMINATION_IRQ_MASK    (1<<15)
#define DESCRIPTOR_CONTROL_EARLY_TERMINATION_IRQ_OFFSET  (15)
#define DESCRIPTOR_CONTROL_ERROR_IRQ_MASK                (0xFF<<16)  // the read master will use this as the transmit error, the dispatcher will use this to generate an interrupt if any of the error bits are asserted by the write master
#define DESCRIPTOR_CONTROL_ERROR_IRQ_OFFSET              (16)
#define DESCRIPTOR_CONTROL_EARLY_DONE_ENABLE_MASK        (1<<24)
#define DESCRIPTOR_CONTROL_EARLY_DONE_ENABLE_OFFSET      (24)
#define DESCRIPTOR_CONTROL_GO_MASK                       (1<<31)  // at a minimum you always have to write '1' to this bit as it commits the descriptor to the dispatcher
#define DESCRIPTOR_CONTROL_GO_OFFSET                     (31)

// DISCRIPTOR_BASE
#define FORZA_SGDMA_DISC_REG_BASE (0x12020)   // Scatter Gather DMA R/W Dispacher FIFO Register
#define STANDARD_DESCRIPTOR_SIZE 16
// DISCRIPTOR_Standard
#define SGDMA_DESC_READ_ADDR_REG             (FORZA_SGDMA_DISC_REG_BASE + 0x0)
#define SGDMA_DESC_WRITE_ADDR_REG            (FORZA_SGDMA_DISC_REG_BASE + 0x4)
#define SGDMA_DESC_LENGTH_REG                (FORZA_SGDMA_DISC_REG_BASE + 0x8)
#define SGDMA_DESC_CONTROL_STD_REG           (FORZA_SGDMA_DISC_REG_BASE + 0xC)
// DISCRIPTOR_Externd
#define SGDMA_DESC_SEQUENCE_NUMBER_REG       (FORZA_SGDMA_DISC_REG_BASE + 0xC)
#define SGDMA_DESC_READ_BURST_REG            (FORZA_SGDMA_DISC_REG_BASE + 0xE)
#define SGDMA_DESC_WRITE_BURST_REG           (FORZA_SGDMA_DISC_REG_BASE + 0xF)
#define SGDMA_DESC_READ_STRIDE_REG           (FORZA_SGDMA_DISC_REG_BASE + 0x10)
#define SGDMA_DESC_WRITE_STRIDE_REG          (FORZA_SGDMA_DISC_REG_BASE + 0x12)
#define SGDMA_DESC_CONTROL_ENH_REG           (FORZA_SGDMA_DISC_REG_BASE + 0x1C)
// DISCRIPTOR_Standard_Access 
#define SGDMA_DESC_READ_ADDR_W(c, d)   FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_DESC_READ_ADDR_REG,d)
#define SGDMA_DESC_WRITE_ADDR_W(c, d)  FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_DESC_WRITE_ADDR_REG,d)
#define SGDMA_DESC_LENGTH_W(c, d)      FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_DESC_LENGTH_REG,d)
#define SGDMA_DESC_CONTROL_STD_W(c, d) FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_DESC_CONTROL_STD_REG,d) 
// DISCRIPTOR_Extend_Access 
// this pushes the descriptor into the read/write FIFOs when standard descriptors are used
#define SGDMA_DESC_SEQUENCE_NUM(c, d)  FORZA_IOWR_16(SGDMA_BAR(c),SGDMA_DESC_SEQUENCE_NUM_REG,d)
#define SGDMA_DESC_READ_BURST(c, d)    FORZA_IOWR_8(SGDMA_BAR(c),SGDMA_DESC_READ_BURST_REG,d)
#define SGDMA_DESC_WRITE_BURST(c, d)   FORZA_IOWR_8(SGDMA_BAR(c),SGDMA_DESC_WRITE_BURST_REG,d)
#define SGDMA_DESC_READ_STRIDE(c, d)   FORZA_IOWR_16(SGDMA_BAR(c),SGDMA_DESC_READ_STRIDE_REG,d)
#define SGDMA_DESC_WRITE_STRIDE(c, d)  FORZA_IOWR_16(SGDMA_BAR(c),SGDMA_DESC_WRITE_STRIDE_REG,d)
#define SGDMA_DESC_CONTROL_ENH(c, d)   FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_DESC_CONTROL_ENH_REG,d)   // this pushes the descriptor into the read/write FIFOs when the extended descriptors are used

// CONTROL & STATUS REGSTER_BASE
#define FORZA_SGDMA_CSR_REG_BASE  (0x12000)   // Scatter Gather DMA Control & Status Register
// CONTROL & STATUS Register
#define SGDMA_CSR_STATUS_REG                 (FORZA_SGDMA_CSR_REG_BASE + 0x0)
#define SGDMA_CSR_CONTROL_REG                (FORZA_SGDMA_CSR_REG_BASE + 0x4)
#define SGDMA_CSR_DESCRIPTOR_FILL_LEVEL_REG  (FORZA_SGDMA_CSR_REG_BASE + 0x8)
#define SGDMA_CSR_RESPONSE_FILL_LEVEL_REG    (FORZA_SGDMA_CSR_REG_BASE + 0xC)
#define SGDMA_CSR_SEQUENCE_NUMBER_REG        (FORZA_SGDMA_CSR_REG_BASE + 0x10)  // this register only exists when the enhan
#define CSR_BUSY_MASK                           (1)
#define CSR_BUSY_OFFSET                         (0)
#define CSR_DESCRIPTOR_BUFFER_EMPTY_MASK        (1<<1)
#define CSR_DESCRIPTOR_BUFFER_EMPTY_OFFSET      (1)
#define CSR_DESCRIPTOR_BUFFER_FULL_MASK         (1<<2)
#define CSR_DESCRIPTOR_BUFFER_FULL_OFFSET       (2)
#define CSR_RESPONSE_BUFFER_EMPTY_MASK          (1<<3)
#define CSR_RESPONSE_BUFFER_EMPTY_OFFSET        (3)
#define CSR_RESPONSE_BUFFER_FULL_MASK           (1<<4)
#define CSR_RESPONSE_BUFFER_FULL_OFFSET         (4)
#define CSR_STOP_STATE_MASK                     (1<<5)
#define CSR_STOP_STATE_OFFSET                   (5)
#define CSR_RESET_STATE_MASK                    (1<<6)
#define CSR_RESET_STATE_OFFSET                  (6)
#define CSR_STOPPED_ON_ERROR_MASK               (1<<7)
#define CSR_STOPPED_ON_ERROR_OFFSET             (7)
#define CSR_STOPPED_ON_EARLY_TERMINATION_MASK   (1<<8)
#define CSR_STOPPED_ON_EARLY_TERMINATION_OFFSET (8)
#define CSR_IRQ_SET_MASK                        (1<<9)
#define CSR_IRQ_SET_OFFSET                      (9)
// masks for the control register bits
#define CSR_STOP_MASK                           (1)
#define CSR_STOP_OFFSET                         (0)
#define CSR_RESET_MASK                          (1<<1)
#define CSR_RESET_OFFSET                        (1)
#define CSR_STOP_ON_ERROR_MASK                  (1<<2)
#define CSR_STOP_ON_ERROR_OFFSET                (2)
#define CSR_STOP_ON_EARLY_TERMINATION_MASK      (1<<3)
#define CSR_STOP_ON_EARLY_TERMINATION_OFFSET    (3)
#define CSR_GLOBAL_INTERRUPT_MASK               (1<<4)
#define CSR_GLOBAL_INTERRUPT_OFFSET             (4)
#define CSR_STOP_DESCRIPTORS_MASK               (1<<5)
#define CSR_STOP_DESCRIPTORS_OFFSET             (5)
// masks for the FIFO fill levels and sequence number
#define CSR_READ_FILL_LEVEL_MASK                (0xFFFF)
#define CSR_READ_FILL_LEVEL_OFFSET              (0)
#define CSR_WRITE_FILL_LEVEL_MASK               (0xFFFF0000)
#define CSR_WRITE_FILL_LEVEL_OFFSET             (16)
#define CSR_RESPONSE_FILL_LEVEL_MASK            (0xFFFF)
#define CSR_RESPONSE_FILL_LEVEL_OFFSET          (0)
#define CSR_READ_SEQUENCE_NUMBER_MASK           (0xFFFF)
#define CSR_READ_SEQUENCE_NUMBER_OFFSET         (0)
#define CSR_WRITE_SEQUENCE_NUMBER_MASK          (0xFFFF0000)
#define CSR_WRITE_SEQUENCE_NUMBER_OFFSET        (16)
#define CSR_STATUS_W(c, d)             FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_CSR_STATUS_REG,d)
#define CSR_CONTROL_W(c, d)            FORZA_IOWR_32(SGDMA_BAR(c),SGDMA_CSR_CONTROL_REG,d)
#define CSR_STATUS_R(c)                FORZA_IORD_32(SGDMA_BAR(c),SGDMA_CSR_STATUS_REG)
#define CSR_CONTROL_R(c)               FORZA_IORD_32(SGDMA_BAR(c),SGDMA_CSR_CONTROL_REG)
#define CSR_DESC_FILL_LEVEL_R(c)       FORZA_IORD_32(SGDMA_BAR(c),SGDMA_CSR_DESC_FILL_LEVEL_REG)
#define CSR_RESP_FILL_LEVEL_R(c)       FORZA_IORD_32(SGDMA_BAR(c),SGDMA_CSR_RESP_FILL_LEVEL_REG)
#define CSR_SEQUENCE_NUM_R(c)          FORZA_IORD_32(SGDMA_BAR(c),SGDMA_CSR_SEQUENCE_NUM_REG)

//
// << SPI_DSP_REG >>  0x0000_5000 ~ 0x0000_601f
//
#define SPI_DSP_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_SPI_DSP_REG_BASE     (0x5000)
			// Rx
#define SPI_DSP_RX_REG             (FORZA_SPI_DSP_REG_BASE + 0)
			// Tx
#define SPI_DSP_TX_REG             (FORZA_SPI_DSP_REG_BASE + 0x4) 
			// STATUS
#define SPI_DSP_STATUS_REG          (FORZA_SPI_DSP_REG_BASE + 0x8)
#define SPI_DSP_ROE                (1<<3)   // Rx OverRun Error
#define SPI_DSP_TOE                (1<<4)   // Tx OverWrite Error
#define SPI_DSP_TMT                (1<<5)   // Tx EMPTY
#define SPI_DSP_TRDY               (1<<6)   // Tx Write Ready
#define SPI_DSP_RRDY               (1<<7)   // Rx Read  Ready 
#define SPI_DSP_ERROR              (1<<8)   // SPI Error
#define SPI_DSP_DSP_INT            (1<<9)   // DSP Interrupt
			// INT Ctrl
#define SPI_DSP_CTRL_REG          (FORZA_SPI_DSP_REG_BASE + 0xC)
#define SPI_DSP_OE                 (1<<0)   // SPI OUTPUT Enable
#define SPI_DSP_CMODE              (1<<1)   // CS48L10 mode
#define SPI_DSP_FIFOCLR            (1<<2)   // FIFO Clear
#define SPI_DSP_IROE               (1<<3)   // Rx Interrupt Enable
#define SPI_DSP_ITOE               (1<<4)   // Tx Interrupt Enable
#define SPI_DSP_ITMT               (1<<5)   // Tx Empty Interrupt Enable
#define SPI_DSP_ITRDY              (1<<6)   // Tx Ready Interrupt Enable
#define SPI_DSP_TRRDY              (1<<7)   // Rx Ready Interrupt Enable
#define SPI_DSP_IE                 (1<<8)   // Error Interrupt Enable
#define SPI_DSP_DSP_INTE           (1<<9)   // DSP Interrupt Enable
#define SPI_DSP_SSO                (1<<10)
			// Opt
#define SPI_DSP_OPT_REG            (FORZA_SPI_DSP_REG_BASE + 0x14) 
#define SPI_DSP_CS_MASK            (1<<0)   // CS Output Mask

#define SPI_DSP_RX_REG_R(c)        FORZA_IORD_32(SPI_DSP_BAR(c),SPI_DSP_RX_REG)
#define SPI_DSP_TX_REG_W(c,d)      FORZA_IOWR_32(SPI_DSP_BAR(c),SPI_DSP_TX_REG,d)
#define SPI_DSP_STATUS_REG_W(c,d)  FORZA_IOWR_32(SPI_DSP_BAR(c),SPI_DSP_STATUS_REG,d)
#define SPI_DSP_STATUS_REG_R(c)    FORZA_IORD_32(SPI_DSP_BAR(c),SPI_DSP_STATUS_REG)
#define SPI_DSP_CTRL_REG_W(c,d)    FORZA_IOWR_32(SPI_DSP_BAR(c),SPI_DSP_CTRL_REG,d)
#define SPI_DSP_CTRL_REG_R(c)      FORZA_IORD_32(SPI_DSP_BAR(c),SPI_DSP_CTRL_REG)
#define SPI_DSP_OPT_REG_W(c,d)     FORZA_IOWR_32(SPI_DSP_BAR(c),SPI_DSP_OPT_REG,d)

//
// << SPI_DAC_REG >>  0x0000_6000 ~ 0x0000_600f
//
#define SPI_DAC_BAR(chip) (chip->vir_addr_bar[2])
			//#define FORZA_SPI_DAC_REG_BASE     (0x6000)
#define FORZA_SPI_DAC_REG_BASE     (0x6000)
			// Rx ( Dont Use )
#define SPI_DAC_RX_REG             (FORZA_SPI_DAC_REG_BASE + 0)
			// Tx
#define SPI_DAC_TX_REG             (FORZA_SPI_DAC_REG_BASE + 0x2) 
			// STATUS
#define SPI_DAC_STATUS_REG         (FORZA_SPI_DAC_REG_BASE + 0x4) 
#define SPI_DAC_TOE                (1<<4)   // Tx OverWrite Error
#define SPI_DAC_TMT                (1<<5)   // Tx EMPTY
#define SPI_DAC_TRDY               (1<<6)   // Tx Write Ready
#define SPI_DAC_ERROR              (1<<7)   // SPI Error
			// INT Ctrl
#define SPI_DAC_INT_REG            (FORZA_SPI_DAC_REG_BASE + 0x6) 
#define SPI_DSP_FIFOCLR            (1<<2)   // FIFO Clear
#define SPI_DAC_ITOE               (1<<4)   // Tx OverWrite Error Interrupt Enable
#define SPI_DAC_ITRDY              (1<<6)   // Tx Write Ready Interrupt Enable
#define SPI_DAC_IE                 (1<<8)   // Error Interrupt Enable
#define SPI_DAC_FORCE_CS_EN        (1<<10)  // CS Force Enable       
			// Opt
#define SPI_DAC_OPT_REG            (FORZA_SPI_DAC_REG_BASE + 0x8) 
			// Access Macro
#define SPI_DAC_TX_REG_W(c,d)      FORZA_IOWR_16(SPI_DAC_BAR(c),SPI_DAC_TX_REG,d)
#define SPI_DAC_TX_REG_R(c)        FORZA_IORD_16(SPI_DAC_BAR(c),SPI_DAC_TX_REG)
#define SPI_DAC_STATUS_REG_W(c,d)  FORZA_IOWR_16(SPI_DAC_BAR(c),SPI_DAC_STATUS_REG,d)
#define SPI_DAC_STATUS_REG_R(c)    FORZA_IORD_16(SPI_DAC_BAR(c),SPI_DAC_STATUS_REG)
#define SPI_DAC_INT_REG_W(c,d)     FORZA_IOWR_16(SPI_DAC_BAR(c),SPI_DAC_INT_REG,d)
#define SPI_DAC_INT_REG_R(c)       FORZA_IORD_16(SPI_DAC_BAR(c),SPI_DAC_INT_REG)
#define SPI_DAC_OPT_REG_W(c,d)     FORZA_IOWR_16(SPI_DAC_BAR(c),SPI_DAC_OPT_REG,d)

//
// - PCM1795 REG

#define PCM1795_REG16              0x10    // PCM L ATTENUATOR
#define PCM1795_REG17              0x11    // PCM R ATTENUATOR

#define PCM1795_REG18              0x12    // 
#define PCM1795_ATLD_ENA           (1<<7)
#define PCM1795_FMT_16b_RJ         (0<<4)  // 16bit_std_right-justified BCK>= x32fs
#define PCM1795_FMT_32b_RJ         (1<<4)  // 32bit_std_right-justified BCK>= x64fs
#define PCM1795_FMT_24b_RJ         (2<<4)  // 24bit_std_right-justified BCK>= x48fs
#define PCM1795_FMT_24b_LJ         (3<<4)  // 24bit_MSB_first left-justified BCK>= x48fs
#define PCM1795_FMT_32b_I2S        (4<<4)  // 32bit_I2S BCK>= x64fs
#define PCM1795_FMT_24b_I2S        (5<<4)  // 24bit_I2S BCK>= x48fs   <default>
#define PCM1795_DMF_DIS            (0<<1)  // De-Emphasis 
#define PCM1795_MUTE_ENA           (1<<0)  // DAC Mute Control

#define PCM1795_REG19              0x13
#define PCM1795_REV_OUT            (1<<7)  // Reversal Output
#define PCM1795_ATT_E_LRCK         (0<<5)  // Attenauation Rate Every LRCK <default>
#define PCM1795_ATT_H_LRCK         (1<<5)  // half
#define PCM1795_DAC_OPE            (1<<4)  // DAC Operation enabled <default>
#define PCM1795_DFMS_STEREO        (1<<2)  // Stereo DF MODE
#define PCM1795_DFMS_MONO          (0<<2)  // Monaural DF MODE
#define PCM1795_FLT_SHARP          (0<<1)  // DigitalFilter Roll-Off Sharp <deault>
#define PCM1795_FLT_SLOW           (1<<1)  // DifitalFilter Roll-Off Slow
#define PCM1795_INFI_ZERO_DET_DIS  (0)     // Infinite zero detect mute Disable <default>
#define PCM1795_INFI_ZERO_DET_ENA  (1)     // Infinite zero detect mute Enable

#define PCM1795_REG20              0x14
#define PCM1795_RESET              (1<<6)  // RESET
#define PCM1795_DSD_DIS            (0<<5)  // DSD I/F DISABLE  <default>
#define PCM1795_DSD_ENA            (1<<5)  // DSD I/F ENABLE
#define PCM1795_DF_ENA             (0<<4)  // Digital Filter Enabled <default>
#define PCM1795_DF_THR             (1<<4)  // Digital Filter Byassed 
#define PCM1795_MODE_STEREO        (0<<3)  // DAC Stereo Mode  <default>
#define PCM1795_MODE_MONO          (1<<3)  // DAC MONO   Mode
#define PCM1795_MONO_LEFT          (0<<2)  // Left ch <default>
#define PCM1795_MONO_RIGHT         (1<<2)  // Right ch  
// Delta Sigma OverSampling Rate       
#define PCM1795_OVER_SAMPLE_64     (0<<0)     
#define PCM1795_OVER_SMAPLE_32     (1<<0)
#define PCM1795_OVER_SMAPLE_128    (2<<0)


#define PCM1795_REG21               0x15
#define PCM1795_REG22               0x16
#define PCM1795_REG23               0x17
#define PCM1795_DSD_FIR_1          (0<<2)
#define PCM1795_DSD_FIR_2          (1<<2)
#define PCM1795_DSD_FIR_3          (2<<2)
#define PCM1795_DSD_FIR_4          (3<<2)
//
// << AUDIO_REG >> 0x0000_7000 ~ 0x0000_700f
//

#define AUDIO_CTL_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_AUDIO_CTL_REG_BASE   (0x7000)
#define AUDIO_FADER_0_REG          (FORZA_AUDIO_CTL_REG_BASE + 0x0)
#define AUDIO_FADER_1_REG          (FORZA_AUDIO_CTL_REG_BASE + 0x4)
// 0x00 bit 19~12
// 0x01 bit 11~4
// 0x02 bit 3 ~0
#define AUDIO_FUNCTION_REG         (FORZA_AUDIO_CTL_REG_BASE + 0x8)
#define AUDIO_FORMAT_FS_REG        (1<<4)
#define FORMAT_PCM_44_1K           (0x0<<4)
#define FORMAT_PCM_48K             (0x1<<4)
#define FORMAT_PCM_88_2K           (0x2<<4)
#define FORMAT_PCM_96K             (0x3<<4)   
#define FORMAT_PCM_176_4K          (0x4<<4)
#define FORMAT_PCM_192K            (0x5<<4)
#define FORMAT_PCM_352_8K          (0x6<<4)
#define FORMAT_PCM_384K            (0x7<<4)
#define FORMAT_DSD_2_8M            (0x8<<4)
#define FORMAT_DSD_RSV             (0x9<<4)
#define FORMAT_DSD_5_6M            (0xA<<4)
#define AUDIO_FORMAT_MASK          (0xffffff00)
#define FORMAT_16BIT               (0x0<<2)
#define FORMAT_20BIT               (0x1<<2)
#define FORMAT_24BIT               (0x2<<2)
#define FORMAT_32BIT               (0x3<<2)
#define FORMAT_DSD_8BIT            (1)
#define FORMAT_DSD_32BIT           (3)
#define AUDIO_BIT_LEN_MASK         (0xfffffff3)
// 00:16bit 01:20bit 10:24bit 11:32bit
#define AUDIO_FADER1_SEL_REG        (1<<(7+8))
// 0:FADER_0  1:FADER_1
#define AUDIO_MODEL_SET_REG        (1<<(6+8))
#define AUDIO_MODEL_SPIRITOSO      (1<<(6+8))
#define AUDIO_MODEL_ALLEGRO        (0<<(6+8))
// 0:Spiritoso
// 1:Allegro
#define AUDIO_MUTE_ENA             (1<<(5+8))
#define AUDIO_MUTE_DIS             (0<<(5+8))
// 0:OFF 1:ON
#define AUDIO_MSTR_XSLV_REG        (1<<(4+8))
// 0:XMOS SLAVE_MODE
// 1:FPGA MASTER_MODE
#define SPIRITOSO_PCM_DIRECT       (1<<(1+8))
//
#define AUDIO_DEBUG_REG              (FORZA_AUDIO_CTL_REG_BASE + 0xC)
#define AUDIO_FADER_0_REG_W(c, d)    FORZA_IOWR_32(AUDIO_CTL_BAR(c),AUDIO_FADER_0_REG, d)
#define AUDIO_FADER_0_REG_R(c)       FORZA_IORD_32(AUDIO_CTL_BAR(c),AUDIO_FADER_0_REG)
#define AUDIO_FADER_1_REG_W(c, d)    FORZA_IOWR_32(AUDIO_CTL_BAR(c),AUDIO_FADER_1_REG, d)
#define AUDIO_FADER_1_REG_R(c)       FORZA_IORD_32(AUDIO_CTL_BAR(c),AUDIO_FADER_1_REG)
#define AUDIO_FUNCTION_REG_W(c, d)   FORZA_IOWR_32(AUDIO_CTL_BAR(c),AUDIO_FUNCTION_REG, d)
#define AUDIO_FUNCTION_REG_R(c)      FORZA_IORD_32(AUDIO_CTL_BAR(c),AUDIO_FUNCTION_REG)
#define AUDIO_DEBUG_REG_R(c)         FORZA_IORD_8(AUDIO_CTL_BAR(c),AUDIO_DEBUG_REG)

//
// << USER_REG >> 0x0000_8000 ~ 0x0000_800f
//
#define USER_MODE_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_USER_MODE_REG_BASE   (0x8000)
#define UMODE_IO_REG               (FORZA_USER_MODE_REG_BASE + 0)
#define UMODE_BOARD_DET_EPP        (1<<6)
#define UMODE_DSP_RESET_DIS        (1<<5)
#define UMODE_DSP_RESET_ENA        (0<<5)
#define UMODE_DAC_RESET            (1<<4)
#define UMODE_DAC_RESET_DIS        (1<<4)
#define UMODE_DAC_RESET_ENA        (0<<4)
#define UMODE_PLL_LOCK_ENA         (1<<3)
#define UMODE_CLOCK_22M_ENA        (0x1<<1)
#define UMODE_CLOCK_24M_ENA        (0x2<<1)
#define UMODE_CLOCK_DIS            (0x0<<1)
#define UMODE_OUTPUT_MUTE_ENA      (0<<0)
#define UMODE_OUTPUT_MUTE_DIS      (1<<0)
#define UMODE_AUDIO_REG            (FORZA_USER_MODE_REG_BASE + 0x4)
#define AUDIO_FIFO_DEBUG_OUT       (0<<4)
#define AUDIO_DATA_DEBUG_OUT       (1<<4)
#define AUDIO_LSB_FIRST            (0<<3)
#define AUDIO_MSB_FIRST            (1<<3)
#define AUDIO_ENDIAN_LITTLE        (0<<2)
#define AUDIO_ENDIAN_BIG           (1<<2)
#define AUDIO_EXTEND_INPUT_ENABLE  (1<<1)
#define AUDIO_PCIE_INPUT_ENABLE    (0<<1)
#define AUDIO_FORMATTER_ENABLE     (1)    // Always ON 
#define AUDIO_FORMATTER_DISABLE    (0)

#define UMODE_DEBUG_REG            (FORZA_USER_MODE_REG_BASE + 0x8)
#define UMODE_INPUT_REG            (FORZA_USER_MODE_REG_BASE + 0xc)
#define UMODE_VERSION_REG          (FORZA_USER_MODE_REG_BASE + 0x10)


#define UMODE_IO_REG_W(c, d)      FORZA_IOWR_8(USER_MODE_BAR(c),UMODE_IO_REG, d)
#define UMODE_IO_REG_R(c)         FORZA_IORD_8(USER_MODE_BAR(c),UMODE_IO_REG)
#define UMODE_IO_REG8_W(c, d)     FORZA_IOWR_8(USER_MODE_BAR(c),UMODE_IO_REG, d)
#define UMODE_IO_REG8_R(c)        FORZA_IORD_8(USER_MODE_BAR(c),UMODE_IO_REG)
#define UMODE_AUDIO_REG_W(c, d)   FORZA_IOWR_8(USER_MODE_BAR(c),UMODE_AUDIO_REG, d)
#define UMODE_AUDIO_REG_R(c)      FORZA_IORD_8(USER_MODE_BAR(c),UMODE_AUDIO_REG)
#define UMODE_DEBUG_REG_W(c, d)   FORZA_IOWR_32(USER_MODE_BAR(c),UMODE_DEBUG_REG, d)
#define UMODE_DEBUG_REG_R(c)      FORZA_IORD_32(USER_MODE_BAR(c),UMODE_DEBUG_REG)
#define UMODE_SW_REG_R(c)         FORZA_IORD_32(USER_MODE_BAR(c),UMODE_INPUT_REG)
#define UMODE_VERSION_REG_R(c)    FORZA_IORD_32(USER_MODE_BAR(c),UMODE_VERSION_REG)

//
// << TEST_REG >> 0x0001_1000 ~ 0x0000_800f
//
#define TEST_IOO_BAR(chip)  (chip->vir_addr_bar[2])
#define FORZA_TEST_IOO_REG_BASE       (0x11000)
#define TEST_IOO_REG_R(c)             FORZA_IORD_32(TEST_IOO_BAR(c),FORZA_TEST_IOO_REG_BASE)


//
// << Timer_REG >> 0x0001_4000 ~ 0x0001_403f
#define TIMEOUT_REG_1S                 (0x3B9ACA0)  // from Application Clock 62.5MHz
#define TIMEOUT_REG_500us              (0x0007A12)
#define TIMEOUT_REG_1ms                (0x000F424)
#define TIMEOUT_REG_2ms                (0x001E848)
#define TIMEOUT_REG_3ms                (0x002DC6C)
#define TIMEOUT_REG_5ms                (0x004C4B5)
#define TIMEOUT_REG_10ms               (0x009896A)
#define TIMEOUT_REG_20ms               (0x01312D4)
#define TIMEOUT_REG_30ms               (0x01C9C3E)
#define TIMEOUT_REG_50ms               (0x02FAF08)
#define TIMEOUT_REG_100ms              (0x05F5E10)
#define TIMEOUT_REG_200ms              (0x0BEBC20)
#define PERIOD_TIMER_BAR(chip) (chip->vir_addr_bar[2])
#define FORZA_PERIOD_TIMER_REG_BASE    (0x14000)
#define TIMER_TIMEOUT_BIT              (1)
#define TIMER_RUN_BIT                  (1 << 1)
#define TIMER_IRQ_EN_BIT               (1)
#define TIMER_CONTINU_BIT              (1 << 1)
#define TIMER_START_BIT                (1 << 2)
#define TIMER_STOP_BIT                 (1 << 3)
#define TIMER_STATUS_REG               (FORZA_PERIOD_TIMER_REG_BASE + 0x0)
#define TIMER_CONTROL_REG              (FORZA_PERIOD_TIMER_REG_BASE + 0x4)
#define TIMER_PERIODL_REG              (FORZA_PERIOD_TIMER_REG_BASE + 0x8)
#define TIMER_PERIODH_REG              (FORZA_PERIOD_TIMER_REG_BASE + 0xC)
#define TIMER_SNAPL_REG                (FORZA_PERIOD_TIMER_REG_BASE + 0x10)
#define TIMER_SNAPH_REG                (FORZA_PERIOD_TIMER_REG_BASE + 0x14)
#define TIMER_STATUS_REG_W(c, d)       FORZA_IOWR_16(PERIOD_TIMER_BAR(c),TIMER_STATUS_REG,d)
#define TIMER_STATUS_REG_R(c)          FORZA_IORD_16(PERIOD_TIMER_BAR(c),TIMER_STATUS_REG)
#define TIMER_CONTROL_REG_W(c, d)      FORZA_IOWR_16(PERIOD_TIMER_BAR(c),TIMER_CONTROL_REG,d)
#define TIMER_CONTROL_REG_R(c)         FORZA_IORD_16(PERIOD_TIMER_BAR(c),TIMER_CONTROL_REG)
#define TIMER_PERIODL_REG_W(c, d)      FORZA_IOWR_32(PERIOD_TIMER_BAR(c),TIMER_PERIODL_REG,d)
#define TIMER_PERIODH_REG_W(c, d)      FORZA_IOWR_32(PERIOD_TIMER_BAR(c),TIMER_PERIODH_REG,d)
#define TIMER_PERIODL_REG_R(c)         FORZA_IORD_32(PERIOD_TIMER_BAR(c),TIMER_PERIODL_REG)
#define TIMER_PERIODH_REG_R(c)         FORZA_IORD_32(PERIOD_TIMER_BAR(c),TIMER_PERIODH_REG)
#define TIMER_SNAPL_REG_R(c)           FORZA_IORD_16(PERIOD_TIMER_BAR(c),TIMER_SNAPL_REG,d)
#define TIMER_SNAPH_REG_R(c)           FORZA_IORD_16(PERIOD_TIMER_BAR(c),TIMER_SNAPH_REG,d)

#define SPIRITOSO_ET   1
#define SPIRITOSO_EPP  2
#define ALLEGRO_ET     1


#define FORZA_RESET_ENA 1
#define FORZA_RESET_DIS 0
/*
 * prototypes
 */
int forza_hw_create(struct snd_forza *chip);
int forza_hw_remove(struct snd_forza *chip);

/*
 * <<FPGA_PCI Core Control>>
 *  - Txs BASE ADDRESS
 *  - INT Controller 
 */
struct forza_hw_cra {
  void (*init)(struct snd_forza *chip);
  unsigned int (*set_txa)(struct snd_forza *chip, unsigned int phy);
  unsigned int (*ena_pcie_irq)(struct snd_forza *chip, unsigned int BIT);
  unsigned int (*ena_avmm_irq)(struct snd_forza *chip, unsigned int BIT);
  unsigned int (*dis_pcie_irq)(struct snd_forza *chip, unsigned int BIT);
  unsigned int (*dis_avmm_irq)(struct snd_forza *chip, unsigned int BIT);
  unsigned int (*sta_pcie_irq)(struct snd_forza *chip);
  unsigned int (*sta_avmm_irq)(struct snd_forza *chip);
  void (*clr_pcie_irq)(struct snd_forza *chip, unsigned int BIT);
  void (*clr_avmm_irq)(struct snd_forza *chip, unsigned int BIT);
};


/* 
 * <<FPGA_DMA Control Operators>>
 */
struct forza_hw_dma {
  void (*init)(struct snd_forza *chip);
  void (*print)(struct snd_forza *chip);
  void (*reset)(struct snd_forza *chip);
  void (*start)(struct snd_forza *chip);
  void (*stop)(struct snd_forza *chip);
  void (*setup)(struct snd_forza *chip, unsigned int waddr, unsigned int raddr, unsigned int len);
  void (*update)(struct snd_forza *chip);
  //  void (*phyaddr)(struct snd_forza *chip, dma_addr_t addr);
  void (*dis_irq)(struct snd_forza *chip);
  void (*ena_irq)(struct snd_forza *chip);
  unsigned int (*rdptr)(struct snd_forza *chip);
};


/* 
 * <<FPGA_TIMER Control Operators>>
 */
struct forza_hw_timer {
  void (*init)(struct snd_forza *chip);
  void (*set_period)(struct snd_forza *chip, unsigned int period);
  void (*start)(struct snd_forza *chip);
  void (*stop)(struct snd_forza *chip);
  void (*clear)(struct snd_forza *chip);
  void (*status)(struct snd_forza *chip);
  void (*dis_irq)(struct snd_forza *chip);
  void (*ena_irq)(struct snd_forza *chip);
  void (*remove)(struct snd_forza *chip);
};

/* 
 * <<FPGA_SPI Control Operators>>
 */
struct forza_hw_dac {
  void (*init)(struct snd_forza *chip);
  void (*reset)(struct snd_forza *chip, int reset);
  void (*mute)(struct snd_forza *chip, int mute);
  void (*output)(struct snd_forza *chip, int output);
  void (*set_default)(struct snd_forza *chip, unsigned char fmt);
  void (*set_data)(struct snd_forza *chip, unsigned char addr, unsigned char data);
  unsigned short (*status)(struct snd_forza *chip);
  void (*dis_irq)(struct snd_forza *chip);
  void (*ena_irq)(struct snd_forza *chip);
  void (*remove)(struct snd_forza *chip);
};

#endif

