/*
 *  forza_lib.c is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#include <linux/pci.h> 
#include "forza.h"
#include "forza_lib.h"
#include <sound/pcm.h>

void forza_print_runtime_info(struct snd_pcm_runtime *runtime, char *comments){
#if 0
  FORZA_DEBUG_LOG("\n***** << Forza Show Runtime Informations:(%s) >> *****\n",comments);
  /* status */
  FORZA_DEBUG_LOG(">hw_ptr_base  (%d)\n", (int)runtime->hw_ptr_base);
  /* HW param */
  FORZA_DEBUG_LOG(">access       (%d)\n", runtime->access);
  FORZA_DEBUG_LOG(">format       (%d)\n", runtime->format);
  FORZA_DEBUG_LOG(">rate         (%d)\n", runtime->rate);
  FORZA_DEBUG_LOG(">channels     (%d)\n", runtime->channels);
  FORZA_INFO_LOG( "dmabuf(%d),period(%d),times(%d)\n", 
		  (int)runtime->dma_bytes,(int)runtime->period_size, (int)runtime->periods);
   /* SW param */
  FORZA_DEBUG_LOG(">period_step  (%x)\n", (int)runtime->period_step);
  FORZA_DEBUG_LOG(">start_thresh (%x)\n", (int)runtime->start_threshold);
  FORZA_DEBUG_LOG(">stop_thresh  (%x)\n", (int)runtime->stop_threshold);
  /* DMA */
  FORZA_DEBUG_LOG(">dma_addr     (%x)\n", runtime->dma_addr );
  FORZA_DEBUG_LOG(">dma_area     (%x)\n", (int)runtime->dma_area);
  FORZA_DEBUG_LOG(">dma_buf size(%d)\n", runtime->dma_bytes);
  FORZA_DEBUG_LOG("******* << End\n\n");
#else
  /* HW param */
  FORZA_DEBUG_LOG("rate         (%d)\n", runtime->rate);
  FORZA_INFO_LOG( "dmabuf(%d),period(%d),times(%d)\n", 
		  (int)runtime->dma_bytes,(int)runtime->period_size, (int)runtime->periods);
}
#endif
void forza_print_substream_info(struct snd_pcm_substream *st){
#if 0
  FORZA_DEBUG_LOG("\n***** << Forza Show Substream Informations >> *****\n");
  FORZA_DEBUG_LOG(">number      (%d)\n", (int)st->number);
  FORZA_DEBUG_LOG(">name        (%s)\n", st->name);
  FORZA_DEBUG_LOG(">steam       (%d)\n", (int)st->stream);
  FORZA_DEBUG_LOG(">buf_byte_max(%d)\n", (int)st->buffer_bytes_max);
  FORZA_DEBUG_LOG(">dma_buf_id  (%d)\n", (int)st->dma_buf_id);
  FORZA_DEBUG_LOG(">dma_max     (%d)\n", (int)st->dma_max);
  FORZA_DEBUG_LOG("******* << End\n\n");
#endif
}

void forza_print_fpga_config(struct snd_forza *forza){
  FORZA_DEBUG_LOG("\n***** << Forza PCI-E Config Space   >> *****\n");
}

void forza_print_build_info(void){
  FORZA_DEBUG_LOG("\n***** << Forza BUILD Definitions >> *****\n");
  FORZA_DEBUG_LOG("NAME             ...%s\n",DRV_NAME);
  FORZA_INFO_LOG("VERSION          ...%s\n",FORZA_VER);
  FORZA_DEBUG_LOG("Kernel Info:\n");
  FORZA_DEBUG_LOG(" >DMA_BUFFER_SIZE ..0x%08x\n", FORZA_DMA_BUFF_SIZE);
  if (FORZA_IRQ == PCI_REGACY_IRQ) 
    FORZA_DEBUG_LOG(" >IRQ            ...%s\n", "REGACY_EN");
  else if(FORZA_IRQ == PCI_MSI_IRQ) 
    FORZA_DEBUG_LOG(" >IRQ            ...%s\n", "MSI_EN");
  else
    FORZA_DEBUG_LOG(" >IRQ            ...%s\n", "DISABLED");

  FORZA_DEBUG_LOG("FPGA   Info:\n");

  if(FORZA_DMA == FORZA_DF_SCDMA)
    FORZA_DEBUG_LOG(" >DMA_MODULES    ...%s\n", "SIMPLE_COMMAND");
  else if(FORZA_DMA == FORZA_DF_SGDMA)
    FORZA_DEBUG_LOG(" >DMA_MODULES    ...%s\n", "SCATTER_GATHER");
  else
    FORZA_DEBUG_LOG(" >DMA_MODULES    ...%s\n", "NOT_SELECTED");

  if(DMA_DESTINATION == FORZA_ONCHIP_MEM_BUF){
    FORZA_DEBUG_LOG(" >DMA_DESTINATION ..%s\n", "ONCHIP_ENABLED");
  }else if(DMA_DESTINATION == FORZA_FIFO_MEM_BUF){
    FORZA_DEBUG_LOG(" >DMA_DESTINATION ..%s\n", "FIFO_ENABLED");
  }
  if(FORZA_TIMER == DEVICE_ENABLE){
    FORZA_DEBUG_LOG(" >TIMER           ..%s\n", "ENABLED");
  }
  if(FORZA_DAC != DEVICE_DISABLE){
    FORZA_DEBUG_LOG(" >DAC CONTROL     ..%s\n", "ENABLED");
  }
  if(FORZA_DSP != DEVICE_DISABLE){
    FORZA_DEBUG_LOG(" >DSP CONTROL     ..%s\n", "ENABLED");
  }
  
  FORZA_DEBUG_LOG("***** << End\n\n");
}


void forza_print_pci_info(struct snd_forza *forza,  struct pci_dev *pci){
  u16 cval;
  FORZA_DEBUG_LOG("\n***** << Forza PCI-E   Info    >> *****\n");

  pci_read_config_word(pci,PCI_VENDOR_ID, &cval);
  pci_read_config_word(pci,PCI_DEVICE_ID, &cval);
  FORZA_DEBUG_LOG("DEVICE_ID (%x)\n" ,(int)cval );
  FORZA_DEBUG_LOG("ID        (%x)\n" ,pci->device );
  FORZA_DEBUG_LOG("BAR[0]:phy(0x%08x), vir(0x%08x), size(0x%08x)\n",
	 (int)forza->phy_addr_bar[0],
	 (int)forza->vir_addr_bar[0],
	 (int)forza->res_size_bar[0]);

  FORZA_DEBUG_LOG("BAR[2]:phy(0x%08x), vir(0x%08x), size(0x%08x)\n",
	 (int)forza->phy_addr_bar[2],
	 (int)forza->vir_addr_bar[2],
	 (int)forza->res_size_bar[2]);

  FORZA_DEBUG_LOG("BAR[3]:phy(0x%08x), vir(0x%08x), size(0x%08x)\n",
	 (int)forza->phy_addr_bar[3],
	 (int)forza->vir_addr_bar[3],
	 (int)forza->res_size_bar[3]);
  
  FORZA_DEBUG_LOG("IRQ       (%d)\n", (int)pci->irq);
}

void forza_set_onchip_mem(struct snd_forza *forza, unsigned int offset , unsigned int val, size_t size){
  int i =0;
  for(i=0 ;i < size/4; i++){
    ONCHIP_MEM_W(forza, offset+i*4, val);
  }
}

void forza_print_onchip_mem_info(struct snd_forza *forza, 
				 unsigned char * comments, unsigned int offset,size_t size, int max_line){
  unsigned int p =offset;
#if 0
  int i ;
  for(i=0; i < size/16; i++){
    FORZA_DEBUG_LOG("0x%08x:%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
		    (int)(ONCHIP_BAR(forza) + FORZA_ONCHIP_BASE + p),
		    ONCHIP_MEM_8R(forza, p),ONCHIP_MEM_8R(forza, p+1),
		    ONCHIP_MEM_8R(forza, p+2),ONCHIP_MEM_8R(forza, p+3),
		    ONCHIP_MEM_8R(forza, p+4),ONCHIP_MEM_8R(forza, p+5),
		    ONCHIP_MEM_8R(forza, p+6),ONCHIP_MEM_8R(forza, p+7),
		    ONCHIP_MEM_8R(forza, p+8),ONCHIP_MEM_8R(forza, p+9),
		    ONCHIP_MEM_8R(forza, p+0xa),ONCHIP_MEM_8R(forza, p+0xb),
		    ONCHIP_MEM_8R(forza, p+0xc),ONCHIP_MEM_8R(forza, p+0xd),
		    ONCHIP_MEM_8R(forza, p+0xe),ONCHIP_MEM_8R(forza, p+0xf));
    p+=0x10;
  }    
#else
  int x, y;
  int start= offset%0x10;
  size_t end = 0;
  FORZA_DEBUG_LOG("***** << DUMP Forza OnChip Memory :%s >> *****\n",comments);
  for(y=0; y <= size/16; y++){
    if(y >= max_line)
      return;
    if(end < size) 
      printk("0x%08x:",(int)(ONCHIP_BAR(forza) + FORZA_ONCHIP_BASE + (p/0x10)*0x10));
    for(x=0; x < 16; x++){
      if(start !=0){
	printk("   ");
	start -=1;
      }else{  
	printk("%02x ",ONCHIP_MEM_8R(forza,p+x));
	end += 1;
      }
      if(end == size )	return;
    }
    p+=0x10;
    printk("\n");
  }
#endif
  FORZA_DEBUG_LOG("***** << End\n\n");
}

void forza_print_mem_dump(void *addr, int size, char *comment){
  unsigned char *p = addr;
  int i ;
  for(i=0; i < size/16; i++){
     FORZA_DEBUG_LOG("0x%08x:%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
		   	   (int)addr, 
	   (int)(*(p+0)),(int)(*(p+1)),(int)(*(p+2)),(int)(*(p+3)),
	   (int)(*(p+4)),(int)(*(p+5)),(int)(*(p+6)),(int)(*(p+7)),
	   (int)(*(p+8)),(int)(*(p+9)),(int)(*(p+10)),(int)(*(p+11)),
	   (int)(*(p+12)),(int)(*(p+13)),(int)(*(p+14)),(int)(*(p+15)));
    p+=0x10;
    addr += 0x10;
  }
}

void forza_preout_data_to_file(struct snd_forza *chip){

#if 0
  struct file *fp;
  char * filename = "/mnt/internal/internal/preout.txt";
  mm_segment_t oldfs;
  oldfs = get_fs();
  set_fs(get_ds());
  fp = filp_open(filename, O_WRONLY | O_CREAT,777);
  if(!fp){
    FORZA_ERROR_LOG(">>FileOpenError\n");
    return;
  }
  fp->f_op->write(fp,chip->preout,chip->preout_size -1 ,&fp->f_pos);
  filp_close(fp, NULL);
  set_fs(oldfs);
#else
  forza_print_mem_dump(chip->preout,chip->preout_size -1,"pre");
#endif
  return;
}
