/*
 *  forza_lib.h is sound driver for custom fpga
 *
 *  Copyright 2013,2014 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
 */

#ifndef __SOUND_FORZA_LIB_H
#define __SOUND_FORZA_LIB_H

#include <linux/pci.h> 
#include "forza.h"
#include "forza_lib.h"

void forza_print_fpga_config(struct snd_forza *forza);
void forza_print_runtime_info(struct snd_pcm_runtime *runtime, char *comments);
void forza_print_build_info(void);
void forza_print_pci_info(struct snd_forza *forza,  struct pci_dev *pci);
void forza_print_substream_info(struct snd_pcm_substream *st);
void forza_print_mem_dump(void *addr, int size, char *comment);
void forza_print_onchip_mem_info(struct snd_forza *forza, unsigned char * comments, 
				 unsigned int offset,size_t size, int max_line);
void forza_set_onchip_mem(struct snd_forza *forza, unsigned int offset , unsigned int val, size_t size);
void forza_preout_data_to_file(struct snd_forza *chip);

u32 forza_null_printk(
    const char *format,
    ...);


#define FORZA_LOG_ERROR (1)
#define FORZA_LOG_WARN  (1 <<1)
#define FORZA_LOG_INFO  (1 <<3)
#define FORZA_LOG_DEBUG (1 <<2)
#define FORZA_LOG_LEVEL (FORZA_LOG_ERROR | FORZA_LOG_WARN | FORZA_LOG_INFO | FORZA_LOG_DEBUG)


#define  FORZA_DEBUG_LOG(...) \
  ((FORZA_LOG_DEBUG & FORZA_LOG_LEVEL) ?  printk(KERN_DEBUG "[FORZA]:"__VA_ARGS__) : forza_null_printk(__VA_ARGS__))

#define  FORZA_INFO_LOG(...) \
  ((FORZA_LOG_INFO & FORZA_LOG_LEVEL) ?  printk(KERN_INFO  "[FORZA]:"__VA_ARGS__) : forza_null_printk(__VA_ARGS__))

#define  FORZA_WARN_LOG(...) \
  ((FORZA_LOG_WARN & FORZA_LOG_LEVEL) ?  printk(KERN_WARN  "[FORZA]:WARN! > "__VA_ARGS__) : forza_null_printk(__VA_ARGS__))

#define  FORZA_ERROR_LOG(...) \
  ((FORZA_LOG_ERROR & FORZA_LOG_LEVEL) ?  printk(KERN_ERR  "[FORZA]:ERROR! > "__VA_ARGS__) : forza_null_printk(__VA_ARGS__))

#endif

